﻿

$Global:Parsers = @{}


<#
.SYNOPSIS
Searches LDAP server with given filter

.DESCRIPTION
Searches LDAP server identified by given domain name with given parameters using System.DirectoryServices.Protocols

.PARAMETER dnsDomain
DNS name of the domain that will be searched against

.PARAMETER attributes
Attributes to return from the search, should be lowercase as convention

.PARAMETER baseDN
Root distinguished name for the search

.PARAMETER scope
Scope of the search

.PARAMETER includeDeleted
Determine if we search for deleted items

.PARAMETER filter
Filter to be applied to the search

.PARAMETER pageSize
Page size for the search
Default = 1000

.PARAMETER ASQ
Attribute name for Attribute Scope Query

.PARAMETER scriptblockFilter
Scriptblock that gets an entry as parameter, analyze the entry and returns whether the indicator needs this entry or not
This parameter is used to do analysis of the ldap results before building a list of object to reduce memory usage

.EXAMPLE
Get all users with their SamAccountName and UserAccountControl
$DN = Get-DN test.lab
Search-AD -dnsDomain test.lab -attributes "samaccountname", "useraccountcontrol" -baseDN $DN -scope "Subtree" -filter "(&(objectCategory=person)(objectClass=user))"
#>
Function Search-AD
{
    param(
        [parameter(Mandatory = $true)]
        [string]$dnsDomain,

        [parameter(Mandatory = $false)]
        [string[]]$attributes = "",

        [parameter(Mandatory = $false)]
        [string]$baseDN,

        [parameter(Mandatory = $false)]
        [string]$scope = "subtree",

        [parameter(Mandatory = $false)]
        [switch]$includeDeleted,

        [parameter(Mandatory = $true)]
        [string]$filter,

        [parameter(Mandatory = $false)]
        [int]$pageSize = 1000,

        [parameter(Mandatory = $false)]
        [string]$ASQ,

        [parameter(Mandatory = $false)]
        [scriptblock[]]$scriptblockFilter
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    Add-Type -Path "$env:windir\Microsoft.NET\Framework64\v4.0.30319\system.directoryservices.protocols.dll"

    # Update filter to support a LDAP filter for a distinguished name which contains an asterisk
    if ($filter -match "distinguishedName=") {
        $filter = $filter.Replace("*","\2A")
    }

    # We don't want the client to timeout in large environments
    $Timeout = [System.TimeSpan]::FromDays(10000)
    $results = New-Object "System.Collections.Generic.List[System.DirectoryServices.Protocols.SearchResultEntry]"

    # If we're querying an attribute that is not normally replicated to a RODC, we will connect to RWDC
    $checkAttributes = @("lmpwdhistory","supplementalcredentials","ntpwdhistory","unicodepwd","dbcspwd","pwdlastset")
    $locatorOptions = [System.DirectoryServices.ActiveDirectory.LocatorOptions]::WriteableRequired
    $contextType = [System.DirectoryServices.ActiveDirectory.DirectoryContextType]::Domain
    $directoryContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext($contextType,$dnsDomain)
    try {
        $computerSiteName = [System.DirectoryServices.ActiveDirectory.ActiveDirectorySite]::GetComputerSite().Name.ToString()
        if (Compare-Object -IncludeEqual -ExcludeDifferent $attributes $checkAttributes) {
            $dcObj = ([System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($directoryContext).FindDomainController($computerSiteName, $locatorOptions)).Name
        }
        else {
            $dcObj = ([System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($directoryContext).FindDomainController($computerSiteName)).Name
        }
    }
    catch {
        if (Compare-Object -IncludeEqual -ExcludeDifferent $attributes $checkAttributes) {
            $dcObj = ([System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($directoryContext).FindDomainController($locatorOptions)).Name
        }
        else {
            $dcObj = ([System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($directoryContext).FindDomainController()).Name
        }
    }
    [System.DirectoryServices.Protocols.LdapConnection] $conn = New-Object System.DirectoryServices.Protocols.LdapConnection($dcObj)

    [System.DirectoryServices.Protocols.SearchRequest] $search = New-Object System.DirectoryServices.Protocols.SearchRequest($baseDN,$filter,$scope,$attributes)
    $search.TimeLimit = $Timeout
    [System.DirectoryServices.Protocols.PageResultRequestControl] $pageRequest = New-Object System.DirectoryServices.Protocols.PageResultRequestControl($pageSize)
    [void]$search.Controls.Add($pageRequest)

    [System.DirectoryServices.Protocols.SearchOptionsControl] $searchOptions = New-Object System.DirectoryServices.Protocols.SearchOptionsControl([System.DirectoryServices.Protocols.SearchOption]::DomainScope)
    [void]$search.Controls.Add($searchOptions)

    # Those flags let us query nTSecurityDescriptor when we don't have access to the SACL
    [System.DirectoryServices.Protocols.SecurityDescriptorFlagControl] $searchSecurityFlags = New-Object System.DirectoryServices.Protocols.SecurityDescriptorFlagControl(7)
    [void]$search.Controls.Add($searchSecurityFlags)

    If ($includeDeleted) {
        [System.DirectoryServices.Protocols.ShowDeletedControl] $showDeletedControl = New-Object System.DirectoryServices.Protocols.ShowDeletedControl
        [void]$search.Controls.Add($showDeletedControl)
    }

    # Attribute Scope Query
    if ($ASQ) {
        [System.DirectoryServices.Protocols.AsqRequestControl] $asqControl = New-Object System.DirectoryServices.Protocols.AsqRequestControl($ASQ)
        [void]$search.Controls.Add($asqControl)
    }

    [int] $pageCount = 0

    if ($attributes) {
        $rangeRetrievalNeeded = Test-RangeRetrievalNeed -Attributes $attributes
    }
    while ($true) {
        $pageCount++
        [System.DirectoryServices.Protocols.SearchResponse] $response = [System.DirectoryServices.Protocols.SearchResponse]$conn.SendRequest($search,$Timeout)
        [System.DirectoryServices.Protocols.PageResultResponseControl] $pageResponse = $response.Controls | Where-Object {$_.Type -eq "1.2.840.113556.1.4.319"}
        if ($response.Entries.Count -gt 0) {
            foreach ($entry in $response.Entries) {
                if ($rangeRetrievalNeeded) {
                    $entry = RangeQueryHelper -Attributes $Attributes -Entries $entry
                }
                if ($attributes) {
                    $entry = Bandage -Attributes $Attributes -Entries $entry
                }
                if ($scriptblockFilter) {
                    foreach ($scriptBlock in $scriptblockFilter) {
                        if ($entry) {
                            $entry = & $scriptblock $entry
                        }
                    }
                    if ($entry) {
                        $results.Add($entry)
                    }
                }
                else {
                    $results.Add($entry)
                }
            }
        }
        if ($pageResponse.Cookie.Length -eq 0) {
            break
        }
        $pageRequest.Cookie = $pageResponse.Cookie
    }
    return $results
}

<#
.SYNOPSIS
Gets the distinguished name of a given domain

.PARAMETER dnsDomain
DNS name of the domain whose distinguished name we want

.EXAMPLE
Get-DN "test.lab"
#>
Function Get-DN {
    param(
        [parameter(Mandatory = $true)]
        [string]$dnsDomain
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    return "DC=$($dnsDomain -replace '\.',',DC=')"
}

<#
.SYNOPSIS
Gets the SID of a given domain

.DESCRIPTION
Queries a given domain for his SID

.PARAMETER dnsDomain
DNS name of the domain whose SID we want

.EXAMPLE
Get-DomainSID "test.lab"
#>
Function Get-DomainSID{ # Gets the domain SID
    param(
        [parameter(Mandatory = $true)]
        [string]$dnsDomain
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    $domContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext([System.DirectoryServices.ActiveDirectory.DirectoryContextType]::Domain, $dnsDomain);
    $selectedDomain = [System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($domContext);
    $sid = $selectedDomain.GetDirectoryEntry().Properties["objectSID"][0]
    return (New-Object System.Security.Principal.SecurityIdentifier @($sid,0)).Value
}

<#
.SYNOPSIS
Gets the SID of a given forest

.DESCRIPTION
Queries a given domain for the forest SID

.PARAMETER forestName
DNS name of the forest whose SID we want

.PARAMETER domainNames
List of domain names within forest

.EXAMPLE
Get-ForestSID -forestName "test.lab" -domainNames "test.lab","child.test.lab"
#>
Function Get-ForestSID{ # Gets the forest SID
    param(
        [parameter(Mandatory = $true)]
        [string]$forestName,

        [parameter(Mandatory = $true)]
        [String[]]$domainNames
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    if ($domainNames.Contains($forestName.ToLower()) -and (Confirm-DomainAvailability $forestName)) {
        $domContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext([System.DirectoryServices.ActiveDirectory.DirectoryContextType]::Domain, $forestName)
        $selectedDomain = [System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($domContext)
        $sid = $selectedDomain.GetDirectoryEntry().Properties["objectSID"][0]
        return (New-Object System.Security.Principal.SecurityIdentifier @($sid,0)).Value
    }
    else {
        foreach ($domain in $domainNames) {
            if (!$domain.ToLower().Equals($forestName.ToLower()) -and (Confirm-DomainAvailability $domain)) {
                $DN = Get-DN $domain

                $results = Search-AD -dnsDomain $domain -attributes "ntsecuritydescriptor" -baseDN $DN -scope "base" `
                    -filter "(DistinguishedName=$DN)"

                if ($results.count -gt 0) {
                    $bytes = $results.Attributes.'ntsecuritydescriptor'.GetValues([Byte[]])[0]
                    $securityDescriptor = New-Object System.DirectoryServices.ActiveDirectorySecurity
                    $securityDescriptor.SetSecurityDescriptorBinaryForm($bytes)

                    foreach ($access in $securityDescriptor.Access) {
                        try {
                            $identityAccount = new-object System.Security.Principal.NTAccount($access.IdentityReference.Value)
                            $identitySID = $identityAccount.Translate([System.Security.Principal.SecurityIdentifier]).Value
                        }
                        catch {
                            $identitySID = $access.IdentityReference.Value
                        }

                        if ($identitySID.EndsWith("-519")) {
                            return $identitySID.Substring(0, $identitySID.LastIndexOf("-"))
                        }
                    }
                }
            }
        }
    }
}

<#
.SYNOPSIS
Gets the availability of a given domain

.DESCRIPTION
Queries a given domain for it's availability

.PARAMETER dnsDomain
DNS name of the domain whose availability we want

.EXAMPLE
Confirm-DomainAvailability "test.lab"
#>
Function Confirm-DomainAvailability{
    param(
        [parameter(Mandatory = $true)]
        [string]$dnsDomain
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    try {
        $domContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext([System.DirectoryServices.ActiveDirectory.DirectoryContextType]::Domain, $dnsDomain)
        [System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($domContext)
        return $true
    } catch {
        return $false
    }
}


<#
.SYNOPSIS
Translate User Account Control value into string

.DESCRIPTION
Translate each of the active flags in a User Account Control value and returns a string of all of them

.PARAMETER uac
User Account Control value to be translated

.EXAMPLE
Get-UACSet 514
#>
Function Get-UACSet{
    param(
        [parameter(Mandatory = $true)]
        [int]$uac
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    $uacvalues = @{
        1 = "Script"
        2 = "AccountDisabled"
        8 = "HomeDirectoryRequired"
        16 = "AccountLockedOut"
        32 = "PasswordNotRequired"
        64 = "PasswordCannotChange"
        128 = "EncryptedTextPasswordAllowed"
        256 = "TempDuplicateAccount"
        512 = "NormalAccount"
        2048 = "InterDomainTrustAccount"
        4096 = "WorkstationTrustAccount"
        8192 = "ServerTrustAccount"
        65536 = "PasswordDoesNotExpire"
        131072 = "MnsLogonAccount"
        262144 = "SmartCardRequired"
        524288 = "TrustedForDelegation"
        1048576 = "AccountNotDelegated"
        2097152 = "UseDesKeyOnly"
        4194304 = "DontRequirePreauth"
        8388608 = "PasswordExpired"
        16777216 = "TrustedToAuthenticateForDelegation"
        33554432 = "NoAuthDataRequired"
        67108864 = "PartialSecretsAccount"
    }

    $uaclist = [System.Collections.ArrayList]@()
    foreach($k in $uacvalues.Keys){
        if($uac -band $k){
            $uaclist.Add($uacvalues[$k]) | out-null
        }
    }
    $uaclist
}

<#
.SYNOPSIS
Performs range retrieval when needed

.DESCRIPTION
Checks if the results of Search-AD has partial results for some attributes that need a ranged retrieval

.PARAMETER Attributes
Array of the attributes requested from Search-AD

.PARAMETER Entries
Array of results returned from Search-AD that will be checked to valid full attribute result

.EXAMPLE
RangeQueryHelper -Attributes "SamAccountName" -Entries $Entries

.NOTES
This function is internal and not exported

#>
Function RangeQueryHelper {
    param(
        [parameter(Mandatory = $true)]
        [string[]]$Attributes,

        [parameter(Mandatory = $true)]
        [System.DirectoryServices.Protocols.SearchResultEntry[]]$Entries
    )

    foreach ($entry in $Entries){
        $resRangeResults = $entry.Attributes.AttributeNames.Where({$PSItem -match "^[\w-]+;range=0-\d+$"})
        foreach ($attribute in $resRangeResults){
            $attribute -match "^([\w-]+);range=0-\d+$" | Out-Null
            $attrName = $Matches[1]
            $attrData =  $entry.Attributes[$attribute]
            $inc = $attrData.count # default is 1500
            $start = 0
            $doneSearching = $false
            while (!($doneSearching)) {
                $start += $inc
                $end = ($start + $inc -1)
                $search.Filter = "distinguishedName=$($entry.DistinguishedName)"
                $search.Scope = "base"
                $search.DistinguishedName = $entry.DistinguishedName
                $attr = "$attrName;range=$start-$end"
                $search.Attributes.Clear()
                $search.Attributes.Insert(0,"$attr")
                $response = [System.DirectoryServices.Protocols.SearchResponse]$conn.SendRequest($search)

                # Number of entries should be 1 or something unexpected happened
                foreach ($resEntry in $response.Entries) {
                    if ($resEntry.Attributes[$attr]) {
                        $attrData += $resEntry.Attributes[$attr]
                        continue
                    }

                    # In case we reached the last chunk or we are stuck in the while loop
                    ElseIf ($resEntry.Attributes["$attrName;range=$start-*"]) {
                        $attr = "$attrName;range=$start-*"
                        $attrData += $resEntry.Attributes[$attr]
                    }
                    $doneSearching = $true
                }
            }

            # Done going through the range, now fill the original attribute
            $directoryAttribute = New-Object System.DirectoryServices.Protocols.DirectoryAttribute($attrName,$attrData)
            $entry.Attributes[$attrName] = $directoryAttribute

            # Remove the partial attribute
            $entry.Attributes.Remove($attribute)
        }
    }
    return $Entries
}

# THIS FUNCTION IS TEMPORARY
# This is a temporary work around for cases that the results is missing some attributes
Function Bandage {
    param(
        [parameter(Mandatory = $true)]
        [string[]]$Attributes,

        [parameter(Mandatory = $true)]
        [System.DirectoryServices.Protocols.SearchResultEntry[]]$Entries
    )
    $attributesCount = $Attributes.Count
    foreach ($entry in $Entries) {
        if ($attributesCount -ne ($entry.Attributes.AttributeNames | Where-Object {!($_ -match ";range=")}).Count) {
            foreach ($attribute in $Attributes) {
                if (!($entry.Attributes.contains($attribute))) {
                    $entry.Attributes.Add($attribute,"")
                }
            }
        }
    }
    return $Entries
}

<#
.SYNOPSIS
Translate SearchFlag value into string

.DESCRIPTION
Translate each of the active flags in a SearchFlag value and returns a string of all of them

.PARAMETER searchFlags
SearchFlag value to be translated

.EXAMPLE
Get-ADSearchFlag 904
#>
Function Get-ADSearchFlag {
    param(
        [parameter(Mandatory = $true)]
        [int]$searchFlags
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    $searchflagvalues = @{
        1 = "Index"
        2 = "ContainerIndex"
        4 = "ANR"
        8 = "PreserveOnDelete"
        16 = "Copy"
        32 = "TupleIndex"
        64 = "SubtreeIndex"
        128 = "Confidential"
        256 = "NeverValueAudit"
        512 = "RODCFiltered"
        1024 = "ExtendedLinkTracking"
        2048 ="BaseOnly"
        4096 ="PartitionSecret"

    }

    $searchflagslist = [System.Collections.ArrayList]@()
    foreach($k in $searchflagvalues.Keys){
        if($searchFlags -band $k){
            $searchflagslist.Add($searchflagvalues[$k]) | out-null
        }
    }
    $searchflagslist
}

<#
.SYNOPSIS
Trim and replace special characters from a string and convert it to xml

.DESCRIPTION
Trim and replace special characters from a string and convert it to xml

.PARAMETER Xml
Xml in a string format to be converted to xml format

.EXAMPLE
Convert-StringToXml "<pszAttributeName>member</pszAttributeName>"
#>
Function Convert-StringToXml {
    param (
        [parameter(Mandatory = $true)]
        [string]$Xml
    )

    $Xml = $Xml -replace "`0$" -replace "&", "&amp;" -replace "`"", "&quot;" -replace "'", "&apos;" -replace "\\<", "&lt;" -replace "\\>", "&gt;"
    $parsedXML = [xml]$Xml
    return $parsedXML
}

<#
.SYNOPSIS
Translate TrustAttributes value into string

.DESCRIPTION
Translate each of the active flags in a TrustAttributes value and returns a string of all of them

.PARAMETER TrustAttributes
TrustAttributes value to be translated

.EXAMPLE
Get-TrustAttributesSet 64
#>
Function Get-TrustAttributesSet {
    param (
        [parameter(Mandatory = $true)]
        [int]$TrustAttributes
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    $trustAttributesValues = @{
        1 = "TRUST_ATTRIBUTE_NON_TRANSITIVE"
        2 = "TRUST_ATTRIBUTE_UPLEVEL_ONLY"
        4 = "TRUST_ATTRIBUTE_QUARANTINED_DOMAIN"
        8 = "TRUST_ATTRIBUTE_FOREST_TRANSITIVE"
        16 = "TRUST_ATTRIBUTE_CROSS_ORGANIZATION"
        32 = "TRUST_ATTRIBUTE_WITHIN_FOREST"
        64 = "TRUST_ATTRIBUTE_TREAT_AS_EXTERNAL"
        128 = "TRUST_ATTRIBUTE_USES_RC4_ENCRYPTION"
        512 = "TRUST_ATTRIBUTE_CROSS_ORGANIZATION_NO_TGT_DELEGATION"
        1024 = "TRUST_ATTRIBUTE_PIM_TRUST"
        2048 = "TRUST_ATTRIBUTE_CROSS_ORGANIZATION_ENABLE_TGT_DELEGATION"
    }

    $attrList = [System.Collections.ArrayList]@()
    foreach($k in $trustAttributesValues.Keys){
        if($TrustAttributes -band $k){
            $attrList.Add($trustAttributesValues[$k]) | out-null
        }
    }
    $attrList
}

<#
.SYNOPSIS
Gets an entry with msds-ReplAttributeMetadata, filter just for the relevant attribute and returns it in a new attribute called attributemetadata

.DESCRIPTION
Gets an entry with msds-ReplAttributeMetadata, filter just for the relevant attribute and returns it in a new attribute called attributemetadata

.PARAMETER Entry
Entry with a msds-ReplAttributeMetadata that needs filter and map

.PARAMETER PropertyFilter
The attribute to filter for.

.PARAMETER StartOriginatingChangeThreshold
Starting time for time filter

.PARAMETER EndOriginatingChangeThreshold
End time for time filter

.PARAMETER MinimumVersionThreshold
Minimum version number that should be retrieved

.PARAMETER MaximumVersionThreshold
Maximum version number that should be retrieved

.EXAMPLE
Update-AttributeMetadata -Entry $entry -PropertyFilter "sidHistory" -MinimumVersion 2 -StartOriginatingChangeThreshold $startOriginatingChangeThreshold

#>
Function Update-AttributeMetadata {
    [CmdletBinding(SupportsShouldProcess)]
    [System.Diagnostics.CodeAnalysis.SuppressMessageAttribute("PSUseSingularNouns", "", Justification="False negative")]
    param (
        [parameter(Mandatory = $true)]
        [System.DirectoryServices.Protocols.SearchResultEntry] $Entry,

        [parameter(Mandatory = $true)]
        [string] $PropertyFilter,

        [parameter(Mandatory = $false)]
        [string] $StartOriginatingChangeThreshold = "1000-01-01T00:00:00Z",

        [parameter(Mandatory = $false)]
        [string] $EndOriginatingChangeThreshold = "9999-01-01T00:00:00Z",

        [parameter(Mandatory = $false)]
        [string] $MinimumVersionThreshold = "0",

        [parameter(Mandatory = $false)]
        [string] $MaximumVersionThreshold = "99999"
    )
    $attributesObject = [System.Collections.ArrayList]@()
    if($PSCmdlet.ShouldProcess($Entry)){
        if(-not $Parsers[$PropertyFilter]) { $Parsers[$PropertyFilter] = [System.Text.RegularExpressions.Regex]::new(".*<pszattributename>$($PropertyFilter.ToLower())<\/pszattributename>.*", 'Compiled') }
        if ($Entry.Attributes.'msds-replattributemetadata') {
            $xml = $Entry.Attributes.'msds-replattributemetadata'.GetValues("string")
            if ($parsers[$PropertyFilter].IsMatch($xml.ToLower()))
            {
                foreach ($attr in $xml) {
                    $attrMetaData = ([xml]($attr -replace '\x00','')).DS_REPL_ATTR_META_DATA
                    if ($attrMetaData) {
                        if ($attrMetaData.pszAttributeName.ToLower() -match $propertyFilter.ToLower()) {
                            if ($attrMetaData.ftimeLastOriginatingChange -lt $EndOriginatingChangeThreshold `
                                    -and $attrMetaData.ftimeLastOriginatingChange -gt $StartOriginatingChangeThreshold )
                            {
                                $attributeMetadata = [PSCustomObject][Ordered] @{
                                    LastOriginatingChange = $attrMetaData.ftimeLastOriginatingChange
                                    AttributeName = $attrMetaData.pszAttributeName
                                    Version = $attrMetaData.dwVersion
                                    LastOriginatingDsaInvocationID = $attrMetadata.uuidLastOriginatingDsaInvocationID
                                    USNOriginatingChange = $attrMetadata.usnOriginatingChange
                                    USNLocalChange = $attrMetadata.usnLocalChange
                                    LastOriginatingDsaDN = $attrMetadata.pszLastOriginatingDsaDN
                                }
                                [void]$attributesObject.add($attributeMetadata)
                            }
                        }
                    }
                }
            }
            If ($attributesObject.Count -gt 0) {
                $Entry.Attributes["attributemetadata"] = $attributesObject
                $Entry.Attributes.Remove("msds-replattributemetadata")
                return $Entry
            }
            Else {
                return $null
            }
        }
    }
}

<#
.SYNOPSIS
Gets an entry with msds-ReplValueMetadata, filter just for the relevant Value and returns it in a new attribute called valuemetadata

.DESCRIPTION
Gets an entry with msds-ReplValueMetadata, filter just for the relevant attribute and returns it in a new attribute called valuemetadata

.PARAMETER Entry
Entry with a msds-ReplValueMetadata that needs filter and map

.PARAMETER PropertyFilter
The attribute to filter for.

.PARAMETER StartOriginatingChangeThreshold
Starting time for time filter

.PARAMETER EndOriginatingChangeThreshold
End time for time filter

.PARAMETER MinimumVersionThreshold
Minimum version number that should be retrieved

.PARAMETER MaximumVersionThreshold
Maximum version number that should be retrieved

.EXAMPLE
Update-ValueMetadata -Entry $entry -PropertyFilter "member" -MinimumVersion 2 -StartOriginatingChangeThreshold $startOriginatingChangeThreshold

#>
Function Update-ValueMetadata {
    [CmdletBinding(SupportsShouldProcess)]
    [System.Diagnostics.CodeAnalysis.SuppressMessageAttribute("PSUseSingularNouns", "", Justification="False negative")]
    param (
        [parameter(Mandatory = $true)]
        [System.DirectoryServices.Protocols.SearchResultEntry] $Entry,

        [parameter(Mandatory = $true)]
        [string] $PropertyFilter,

        [parameter(Mandatory = $false)]
        [string] $StartOriginatingChangeThreshold = "1000-01-01T00:00:00Z",

        [parameter(Mandatory = $false)]
        [string] $EndOriginatingChangeThreshold = "9999-01-01T00:00:00Z",

        [parameter(Mandatory = $false)]
        [string] $MinimumVersionThreshold = "0",

        [parameter(Mandatory = $false)]
        [string] $MaximumVersionThreshold = "99999"
    )
    if($PSCmdlet.ShouldProcess($Entry)){
        if ($Entry.Attributes.'msds-replvaluemetadata') {
            $valuesObject = [System.Collections.ArrayList]@()
            foreach ($attr in $Entry.Attributes.'msds-replvaluemetadata'.GetValues("string")) {
                $attrMetaData = Convert-StringToXml $attr | Select-Object -ExpandProperty 'DS_REPL_VALUE_META_DATA' -ErrorAction SilentlyContinue
                if ($attrMetaData) {
                    if ($attrMetaData.pszAttributeName.ToLower() -match $propertyFilter.ToLower()) {
                        if ($attrMetaData.ftimeLastOriginatingChange -lt $EndOriginatingChangeThreshold `
                                -and $attrMetaData.ftimeLastOriginatingChange -gt $StartOriginatingChangeThreshold `
                                -and $attrMetaData.dwVersion -ge $MinimumVersionThreshold `
                                -and $attrMetaData.dwVersion -le $MaximumVersionThreshold) {
                            $attributeMetadata = [PSCustomObject][Ordered] @{
                                LastOriginatingChange = $attrMetaData.ftimeLastOriginatingChange
                                AttributeName = $attrMetaData.pszAttributeName
                                TimeDeleted = $attrMetaData.ftimeDeleted
                                TimeCreated = $attrMetaData.ftimeCreated
                                Version = $attrMetaData.dwVersion
                                LastOriginatingDsaInvocationID = $attrMetadata.uuidLastOriginatingDsaInvocationID
                                USNOriginatingChange = $attrMetadata.usnOriginatingChange
                                USNLocalChange = $attrMetadata.usnLocalChange
                                LastOriginatingDsaDN = $attrMetadata.pszLastOriginatingDsaDN
                                ParentDN = $Entry.DistinguishedName
                                ValueDN = $attrMetaData.pszObjectDn
                            }
                            [void]$valuesObject.add($attributeMetadata)
                        }
                    }
                }
            }
            If ($valuesObject.Count -gt 0) {
                $Entry.Attributes["valuemetadata"] = $valuesObject
                $Entry.Attributes.Remove("msds-replvaluemetadata")
                return $Entry
            }
            Else {
                return $null
            }
        }
    }
}

<#
.SYNOPSIS
Gets an entry with ntSecurityDescriptor, filter just the relevant ACEs and returns it in a new attribute

.DESCRIPTION
Gets an entry with ntSecurityDescriptor, filter just the relevant ACEs and returns it in a new attribute

.PARAMETER Entry
Entry with a ntSecurityDescriptor that needs filter and map

.PARAMETER GuidHTFilter
Hashtable of Guids to filter for. Use just if only specific object types are needed.

.PARAMETER PropagationFlagsFilter
Wanted propagation flags as string, divided by '|', to be filtered by

.PARAMETER AccessControlTypeFilter
Wanted Access control type (e.g. Allow)

.PARAMETER ExcludedSIDs
List of SIDs that are allowed and should be excluded

.PARAMETER RightsFilter
Access rights that we care about, divided by "|"
For example - "GenericAll|GenericWrite"

.EXAMPLE
Update-NTSecurityDescriptor -Entry $entry -RightsFilter "GenericAll|GenericWrite" -PropagationFlagsFilter "None|NoPropagateInherit"

#>
$codeUpdateNtSecurityDescriptor = @"
using System.DirectoryServices.Protocols;
using System.DirectoryServices;
using System.Collections.Generic;
using System.Collections;
using System;
using System.Security.Principal;
using System.Security.AccessControl;
using System.Text.RegularExpressions;
namespace UpdateNtSecurityDescriptor
{
    public class NtSecurityDescriptor
    {
        static public List<ActiveDirectoryAccessRule> Update(SearchResultEntry entry, Hashtable guidHTFilter, string propagationFlagsFilter, string accessControlTypeFilter, string []excludedSIDs, string rightsFilter, string inheritanceTypeFilter)
        {
            string identitySID = "";
            List<ActiveDirectoryAccessRule> relevantACEs = new List<ActiveDirectoryAccessRule>();
            byte[][] descriptorBytes= (byte[][])entry.Attributes["ntsecuritydescriptor"].GetValues(typeof(byte[]));
            ActiveDirectorySecurity securityDescriptor = new ActiveDirectorySecurity();
            securityDescriptor.SetSecurityDescriptorBinaryForm(descriptorBytes[0]);
            foreach (AuthorizationRule rule in securityDescriptor.GetAccessRules(true, true, typeof(NTAccount)))

            {
                ActiveDirectoryAccessRule oar = rule as ActiveDirectoryAccessRule;
                if (guidHTFilter.Keys.Count == 0 || guidHTFilter.Contains(oar.ObjectType.ToString()))

                {
                    if (!Regex.IsMatch(oar.PropagationFlags.ToString(),propagationFlagsFilter))
                    {
                        continue;
                    }
                    if (!Regex.IsMatch(oar.InheritanceType.ToString(),inheritanceTypeFilter))
                    {
                        continue;
                    }
                    try
                    {
                        NTAccount identityAccount = new NTAccount(oar.IdentityReference.Value);
                        identitySID = identityAccount.Translate(typeof(SecurityIdentifier)).Value;
                    }
                    catch
                    {
                        identitySID = oar.IdentityReference.Value;
                    }
                    bool containsSID = false;
                    foreach (string sid in excludedSIDs)
                    {
                        if (sid == identitySID)
                        {
                            containsSID = true;
                            break;
                        }
                    }
                    if (!(containsSID))
                    {
                        if (oar.AccessControlType.ToString() == accessControlTypeFilter)
                        {
                            if (Regex.IsMatch(oar.ActiveDirectoryRights.ToString(),rightsFilter))
                            {
                                relevantACEs.Add(oar);
                            }
                      }
                    }
                }
            }
            return relevantACEs;
        }
    }
}
"@
if ($PSVersionTable.PSEdition -eq "Core"){
    $referencingassembliesNtSD = ("System.DirectoryServices.Protocols","System.Collections","System.Security","System.DirectoryServices","System.Security.AccessControl","System.Text.RegularExpressions","System.Collections.NonGeneric","System.IO.FileSystem.AccessControl","System.Security.Principal.Windows")
}
else{
    $referencingassembliesNtSD = ("System.DirectoryServices.Protocols","System.Collections","System.Security","System.DirectoryServices")
}
Add-Type -TypeDefinition $codeUpdateNtSecurityDescriptor -Language CSharp -ReferencedAssemblies $referencingassembliesNtSD


Function Update-NTSecurityDescriptor {
    [CmdletBinding(SupportsShouldProcess)]
    param (
        [parameter(Mandatory = $true)]
        [System.DirectoryServices.Protocols.SearchResultEntry] $Entry,

        [parameter(Mandatory = $false)]
        [Hashtable] $GuidHTFilter = @{},

        [parameter(Mandatory = $false)]
        [string] $PropagationFlagsFilter,

        [parameter(Mandatory = $false)]
        [string] $InheritanceTypeFilter = "None|All|Descendents|Children|SelfAndChildren",

        [parameter(Mandatory = $false)]
        [string] $AccessControlTypeFilter = "Allow",

        [parameter(Mandatory = $false)]
        [string[]] $ExcludedSIDs = @(),

        [parameter(Mandatory = $false)]
        [string] $RightsFilter
    )

    $relevantACEs = [UpdateNtSecurityDescriptor.NtSecurityDescriptor]::Update($Entry,$GuidHTFilter,$PropagationFlagsFilter,$AccessControlTypeFilter,$ExcludedSIDs,$RightsFilter,$InheritanceTypeFilter)
    if ($relevantACEs) {
        $Entry.Attributes["aces"] = $relevantACEs

        # Remove the full ntSecurityDescriptor attribute
        $Entry.Attributes.Remove("ntsecuritydescriptor")
        return $Entry
    }
    else {
        return $null
    }
}
<#
.SYNOPSIS
Check whether a multi valued attribute was requested

.DESCRIPTION
Checking for a need for range retrieval query will require iterating over every entry in the results,
this function should prevent doing this check when no multi valued attributed was requested and therefore no chance for range retrieval need

.PARAMETER Attributes
Array of the attributes requested from Search-AD

.EXAMPLE
Test-RangeRetrievalNeed -Attributes "SamAccountName"

.NOTES
This function is internal and not exported

#>
Function Test-RangeRetrievalNeed {
    param (
        [parameter(Mandatory = $true)]
        [string[]]$Attributes
    )

    # Determine if ranged search might be needed, trying to reduce cost of the check for ranged results
    $rangedSearch = $false
    $rangedAttributes = @("member","memberof","serviceprincipalname", "*", "msds-replattributemetadata", "msds-replvaluemetadata", "serviceprincipalname", "msds-revealedlist") # Attributes that are multi valued and might have more then 1500 entries
    foreach ($attribute in $Attributes) {
        if($rangedAttributes -contains $attribute.ToLower()) {
            $rangedSearch = $true
            break
        }
    }
    return $rangedSearch
}

<#
.DESCRIPTION
Returns SIDs or DNs for privileged group membesrs in requested domains
.PARAMETER DomainNames
Array of existing domains
.PARAMETER ReturnSIDs
Only specify if SIDs are needed
.EXAMPLE
Get-AdminsInDomain -DomainNames $DomainNames
Get-AdminsInDomain -DomainNames $DomainNames -ReturnSIDs
#>
Function Get-AdminsInDomain {
    param (
        [parameter(Mandatory = $true)]
        [string]$forestName,

        [parameter(Mandatory = $true)]
        [string[]]$DomainNames,

        [parameter(Mandatory = $false)]
        [switch]$ReturnSIDs
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    $forestDN = Get-DN $forestName
    $privilegedGroupsDN = [System.Collections.ArrayList]@()
    $admins = [System.Collections.ArrayList]@()
    $failedToEnumerate = [System.Collections.ArrayList]@()
    $largeGroups = [System.Collections.ArrayList]@()
    $EnumeratedDN = [System.Collections.ArrayList]@()
    $unavailableDomains = [System.Collections.ArrayList]@()

    # Build filter for primary group ID search
    $privilegedRIDs = [System.Collections.ArrayList]@()
    [void]$privilegedRIDs.AddRange(@("551","552","548","549","550","544","512","516","518","519","521"))

    $primaryGroupFilter = "(|"
    foreach($privilegedRID in $privilegedRIDs) {
        $primaryGroupFilter += "(primaryGroupID=$privilegedRID)"
    }
    $primaryGroupFilter += ")"

    foreach ($domain in $DomainNames) {
        if ($unavailableDomains.Contains($domain)) {
            continue
        }
        else {
            if (-not (Confirm-DomainAvailability $domain)) {
                [void]$unavailableDomains.Add($domain)
                continue
            }
        }

        $DN = Get-DN $domain
        $domainSID = Get-DomainSID $domain

        if ($domain -eq $ForestName) {
            # Get a list of the forest groups DN
            $forestSID = Get-DomainSID $ForestName
            $forestPrivilegedGroups = @("519","518")

            foreach ($groupRID in $forestPrivilegedGroups) {
                $forestGroupObject = Search-AD -dnsDomain $ForestName -baseDN $forestDN -scope "subtree" `
                    -filter "(&(objectSid=$forestSID-$groupRID)(objectCategory=group))"
                if ($forestGroupObject) {
                    [void]$privilegedGroupsDN.add($forestGroupObject.DistinguishedName)
                }
            }
        }

        $privilegedGroups = @("S-1-5-32-551","S-1-5-32-552","S-1-5-32-548","S-1-5-32-549","S-1-5-32-550","S-1-5-32-544",
            "$domainSID-512","$domainSID-516","$domainSID-521")

        # Get a list of the groups DN
        foreach ($sid in $privilegedGroups) {
            $groupObjectSearchParams = @{
                dnsDomain = $domain
                baseDN = $DN
                scope = "subtree"
                filter = "(&(objectSid=$sid)(objectCategory=group))"
            }

            $groupObject = Search-AD @groupObjectSearchParams

            if ($groupObject) {
                [void]$privilegedGroupsDN.add($groupObject.DistinguishedName)
            }
        }

        # Get users who are admins via primaryGroupID
        $primarySearchParams = @{
            dnsDomain = $domain
            attributes = "objectsid"
            baseDN = $DN
            scope = "subtree"
            filter = $primaryGroupFilter
        }

        $primaryResults = Search-AD @primarySearchParams

        foreach ($primaryResult in $primaryResults) {
            if (!($EnumeratedDN.Contains($primaryResult.DistinguishedName))) {
                [void]$EnumeratedDN.Add($primaryResult.DistinguishedName)
            }

            if ($ReturnSIDs) {
                $adminSID = (New-Object System.Security.Principal.SecurityIdentifier @($primaryResult.Attributes."objectsid".GetValues("byte[]")[0],0)).Value
                if (!($admins.Contains($adminSID))) {
                    [void]$admins.Add($adminSID)
                }
            }
        }
    }

    # The above creation of $privilegedGroupsDN will result in only groups existing in one of the domains provided within $DomainNames
    foreach($groupDN in $privilegedGroupsDN) {
        try {
            $groupDomain = $groupDN.Substring($groupDN.IndexOf("DC=")).replace("DC=","").replace(",",".")
            $groupDomainNETBios = Get-NetBIOSName -dnsDomain $groupDomain -DomainNames $DomainNames -forestName $forestName

            $groupMemberSearchParam = @{
                dnsDomain = $groupDomain
                attributes = @("msds-membertransitive", "objectsid", "samAccountName")
                baseDN = $groupDN
                scope = "base"
                filter = "distinguishedName=$groupDN"
            }

            $groupMembers = Search-AD @groupMemberSearchParam

            if ($ReturnSIDs) {
                $groupSID = (New-Object System.Security.Principal.SecurityIdentifier @($groupMembers.Attributes."objectsid".GetValues("byte[]")[0],0)).Value
                if ($admins.Contains($groupSID)) {
                    continue
                }
                [void]$admins.Add($groupSID)
            }
            if (!$EnumeratedDN.Contains($groupDN)) {
                [void]$EnumeratedDN.Add($groupDN)
            }

            if ($groupMembers.Attributes."msds-membertransitive;range=0-4499") {
                $groupSAM = $groupMembers.Attributes.samaccountname[0]
                [void]$largeGroups.Add("$groupDomainNETBios\$groupSAM")
                $memberAttribute = "msds-membertransitive;range=0-4499"
            }
            else {
                $memberAttribute = "msds-membertransitive"
            }

            if (!($groupMembers.Attributes."msds-membertransitive"[0]) -and !($memberAttribute -match "range")) {
                continue
            }

            foreach ($adminDN in $groupMembers.Attributes.$memberAttribute.GetValues("string")) {
                $memberDomain = $adminDN.Substring($adminDN.IndexOf("DC=")).replace("DC=","").replace(",",".").tolower()

                if ($unavailableDomains.Contains($memberDomain) -or !($domainNames.Contains($memberDomain))) {
                    continue
                }
                else {
                    if (-not (Confirm-DomainAvailability $memberDomain)) {
                        [void]$unavailableDomains.Add($memberDomain)
                        continue
                    }
                }

                if ($EnumeratedDN.Contains($adminDN)) {
                    continue
                }
                [void]$EnumeratedDN.Add($adminDN)

                if (!$ReturnSIDs) {
                    continue
                }
                try {
                    if ($adminDN -match "CN=ForeignSecurityPrincipals") {
                        if (!($admins.Contains($adminDN))) {
                            [void]$admins.Add($adminDN)
                        }

                        if (!($failedToEnumerate.Contains($adminDN))) {
                            [void]$failedToEnumerate.Add($adminDN)
                        }

                        continue
                    }

                    $adminSearchParam = @{
                        dnsDomain = $memberDomain
                        attributes = "objectsid"
                        baseDN = $adminDN
                        scope = "base"
                        filter = "distinguishedName=$adminDN"
                    }

                    $adminObject = Search-AD @adminSearchParam

                    $adminSID = (New-Object System.Security.Principal.SecurityIdentifier @($adminObject.Attributes."objectsid".GetValues("byte[]")[0],0)).Value
                    if ($admins.Contains($adminSID)) {
                        continue
                    }
                    [void]$admins.Add($adminSID)
                }
                catch {
                    if (!($failedToEnumerate.Contains($adminDN))) {
                        [void]$failedToEnumerate.Add($adminDN)
                    }
                }
            }
        }
        catch {
            if (!($failedToEnumerate.Contains($adminDN))) {
                [void]$failedToEnumerate.Add($adminDN)
            }
        }
    }
    if (!$ReturnSIDs) {
        $admins = $EnumeratedDN
    }
    return $admins, $failedToEnumerate, $largeGroups, $unavailableDomains
}

<#
.SYNOPSIS
Update output ArrayList to include additional fields
.DESCRIPTION
Returns an updated version of the indicator's output ArrayList if more processing is needed
.PARAMETER OutputObjects
ArrayList of objects to be returned by the indicator
.PARAMETER StartAttackWindow
DateTime specifying the start of the attack
.PARAMETER EndAttackWindow
DateTime specifying the end of the attack
.EXAMPLE
Update-IndicatorOutputAttackWindow -OutputObjects $outputObjects -StartAttackWindow $StartAttackWindow -EndAttackWindow $EndAttackWindow
#>
Function Update-IndicatorOutputAttackWindow {
    [OutputType([System.Collections.ArrayList])]
    [CmdletBinding(SupportsShouldProcess)]
    param (
        $ThisOutput,

        $StartAttackWindow,

        $EndAttackWindow,

        $Metadata,

        [switch]$ValueMetadata,

        [switch]$ExcludeTimestamp
    )
    if($PSCmdlet.ShouldProcess($Metadata.ValueDN)) {
        if(!($ThisOutput)) {
            return $null
        }

        if ($StartAttackWindow -and $EndAttackWindow) {
            if(!($ThisOutput.psobject.Properties.name -contains "EventTimestamp")) {
                if (!($Metadata)) {
                    return $null
                }
                $timestamp = [DateTime]::Parse($Metadata.LastOriginatingChange).ToUniversalTime()
                $ThisOutput | Add-Member -Type NoteProperty -Name "EventTimestamp" -Value $timestamp
            }
            if ($ThisOutput."EventTimestamp" -gt $StartAttackWindow -and $ThisOutput."EventTimestamp" -lt $EndAttackWindow) {
                $ThisOutput | Add-Member -Type NoteProperty -Name "InAttackWindow" -Value "True"
            }
            else {
                $ThisOutput | Add-Member -Type NoteProperty -Name "InAttackWindow" -Value "False"
            }
        }
        else {
            if(!($ThisOutput.psobject.Properties.name -contains "EventTimestamp") -and (!($ExcludeTimestamp))) {
                if (!($Metadata)) {
                    return $null
                }
                $timestamp = [DateTime]::Parse($Metadata.LastOriginatingChange).ToUniversalTime()
                $ThisOutput | Add-Member -Type NoteProperty -Name "EventTimestamp" -Value $timestamp
            }
        }
    }
}

<#
.SYNOPSIS
Proxy to the Search-AD function

.DESCRIPTION
Utilize Search-AD to send ldap queries but applies some logic before calling search-ad to split into several calls

.PARAMETER dnsDomain
DNS name of the domain that will be searched against

.PARAMETER attributes
Attributes to return from the search, should be lowercase as convention

.PARAMETER baseDN
Root distinguished name for the search

.PARAMETER scope
Scope of the search

.PARAMETER filter
Filter to be applied to the search

.PARAMETER pageSize
Page size for the search
Default = 1000

.PARAMETER ASQ
Attribute name for Attribute Scope Query

.PARAMETER scriptblockFilter
Scriptblock that gets an entry as parameter, analyze the entry and returns whether the indicator needs this entry or not
This parameter is used to do analysis of the ldap results before building a list of object to reduce memory usage

.PARAMETER MetaType
For PK-PRE indicators, chose whether we should search on msds-replattributemetadata or msds-replvaluemetadata

.PARAMETER StartAttackWindow
Start time of the time window supplied by PK-PRE

.PARAMETER EndAttackWindow
End time of the time window supplied by PK-PRE

.PARAMETER PropertyFilter
Attribute name of the attribute we are looking for in the metadata (for PK-PRE indicators)

.PARAMETER StartMetaFilter
Start time of the time window supplied by the indicator in his normal case (for indicators that utilized metadata before PK-PRE)

.PARAMETER EndMetaFilter
End time of the time window supplied by the indicator in his normal case (for indicators that utilized metadata before PK-PRE)

.PARAMETER MinimumVersionThreshold
Minimum dwVersion to filter by in the metadata (for indicators that utilized metadata before PK-PRE)

.PARAMETER MaximumVersionThreshold
Maximum dwVersion to filter by in the metadata (for indicators that utilized metadata before PK-PRE)

.EXAMPLE
Get all users with their SamAccountName and UserAccountControl
$DN = Get-DN test.lab
Search-ADHelper -dnsDomain test.lab -attributes "samaccountname", "useraccountcontrol" -baseDN $DN -scope "Subtree" -filter "(&(objectCategory=person)(objectClass=user))"
#>
Function Search-ADHelper
{
    param(
        [parameter(Mandatory)]
        [string] $dnsDomain,

        [string[]] $attributes = "",

        [string] $baseDN,

        [string] $scope = "subtree",

        [parameter(Mandatory)]
        [string] $filter,

        [int] $pageSize = 1000,

        [string] $ASQ,

        [scriptblock[]] $scriptblockFilter = @(),

        [string] $MetaType = "Attribute",

        $StartAttackWindow,

        $EndAttackWindow,

        [string] $PropertyFilter,

        [DateTime] $StartMetaFilter = "Monday, 1 January 0001 0:00:00",

        [DateTime] $EndMetaFilter = "Friday, 31 December 9999 23:59:59",

        [string] $MinimumVersionThreshold = "0",

        [string] $MaximumVersionThreshold = "99999",

        [switch] $ExcludeScriptblock
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    $attributes = $attributes | ForEach-Object {$_.ToLower()}
    $searchADParameters = @{
        dnsDomain = $dnsDomain
        attributes = $attributes
        baseDN = $baseDN
        scope = $scope
        filter = $filter
        pageSize = $pageSize
        ASQ = $ASQ
        scriptblockFilter = $scriptblockFilter
    }

    $metaDateFormat = "yyyy-MM-ddTHH:mm:ssZ"

    if ($PropertyFilter) {
        if (!($ExcludeScriptblock) -or ($StartAttackWindow -and $EndAttackWindow)) {
            # Check which is earlier, the indicator default start date or the start of the attack window
            if ($StartAttackWindow -and $StartAttackWindow -lt $StartMetaFilter) {
                $StartMetaFilter = $StartAttackWindow
            }

            # updateMetadataArguments will be used when using $updateMetadata.GetNewClosure() although we are not passing it anywhere
            $updateMetadataArguments = @{
                PropertyFilter = $PropertyFilter
                MinimumVersionThreshold = $MinimumVersionThreshold
                MaximumVersionThreshold = $MaximumVersionThreshold
                StartOriginatingChangeThreshold = get-date $StartMetaFilter -Format $metaDateFormat
                EndOriginatingChangeThreshold = get-date $EndMetaFilter -Format $metaDateFormat
            }

            # This line doesn't really do anything, it is just to bypass the powershell script analyzer saying this variable is assigned but not used
            $updateMetadataArguments.MinimumVersionThreshold = $MinimumVersionThreshold

            # We need to make sure we are getting the metadata
            if ($MetaType -eq "attribute") {
                if(!($attributes.Contains("msds-replattributemetadata"))) {
                    $attributes += "msds-replattributemetadata"
                }
            }
            elseif($MetaType -eq "value") {
                if(!($attributes.Contains("msds-replvaluemetadata"))) {
                    $attributes += "msds-replvaluemetadata"
                }
            }
            $searchADParameters.attributes = $attributes
            $updateMetadata = UpdateMetaHelper -MetaType $MetaType

            $searchADParameters.scriptblockFilter += $updateMetadata.GetNewClosure()
            return Search-AD @searchADParameters
        }
    }
    # We hit this code if we don't have PropertyFilter or we were provided $excludeScriptblockFitler AND were not provided with an attack window
    return Search-AD @searchADParameters
}

<#
.SYNOPSIS
Check domain availability before passing to Search-ADHelper when querying the Configuration partition

.DESCRIPTION
Determine if a domain is available before running Search-ADHelper for querying the Configuration partition

.PARAMETER forestName
DNS name of the forest whose SID we want

.PARAMETER domainNames
List of domain names within forest

.PARAMETER searchADParameters
Hashtable. Parameters to pass to Search-ADHelper

.EXAMPLE
Search-ADConfig -forestName "test.lab" -domainNames "test.lab","child.test.lab" -searchADParameters $searchParams
#>
Function Search-ADConfig
{
    param(
        [parameter(Mandatory)]
        [string] $forestName,

        [parameter(Mandatory)]
        [string[]] $domainNames,

        [parameter(Mandatory)]
        [hashtable] $searchADParameters
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    $unavailableDomains = [System.Collections.ArrayList]@()

    if ($domainNames.Contains($forestName.ToLower()) -and (Confirm-DomainAvailability $forestName)) {
        $searchADParameters.dnsDomain = $forestName
    }
    else {
        if ($domainNames.Contains($forestName)) {
            [void]$unavailableDomains.Add($forestName.ToLower())
        }

        foreach ($domain in $domainNames) {
            if (-not $domain.ToLower().Equals($forestName.ToLower()) -and (Confirm-DomainAvailability $domain)) {
                $searchADParameters.dnsDomain = $domain
                break
            }
            elseif (-not $domain.ToLower().Equals($forestName.ToLower())) {
                [void]$unavailableDomains.Add($domain)
            }
        }
    }

    if ($unavailableDomains.Count -eq $domainNames.Count) {
        return [System.Collections.ArrayList]@(), $unavailableDomains
    }

    return (Search-ADHelper @searchADParameters), $unavailableDomains
}

<#
.SYNOPSIS
Build scriptblock to be passed by Search-ADHelper to Search-AD base on $MetaType parameter

.DESCRIPTION
Build scriptblock to be passed by Search-ADHelper to Search-AD base on $MetaType parameter

.PARAMETER MetaType
For PK-PRE indicators, chose whether we should search on msds-replattributemetadata or msds-replvaluemetadata

.Example
$updateMetadata = UpdateMetaHelper -MetaType $MetaType
#>
Function UpdateMetaHelper {
    param (
        [string]$MetaType
    )
    if ($MetaType -eq "Attribute") {
        $updateMetaSB = {
            param ([System.DirectoryServices.Protocols.SearchResultEntry] $Entry)
            Update-AttributeMetadata @updateMetadataArguments -entry $Entry
        }
    }
    elseif ($MetaType -eq "Value") {
        $updateMetaSB = {
            param ([System.DirectoryServices.Protocols.SearchResultEntry] $Entry)
            Update-ValueMetadata @updateMetadataArguments -entry $Entry
        }
    }
    return $updateMetaSB
}

<#
.SYNOPSIS
Get a string representation of MsDS-SupportedEncryptionTypes from its decimal value

.DESCRIPTION
Get a string representation of MsDS-SupportedEncryptionTypes from its decimal value

.PARAMETER SupportedEncryptionTypesValue
The decimal value of MsDS-SupportedEncryptionTypes

.EXAMPLE
Get-SupportedEncryptionTypesSet $supportedEncryptionType

#>
Function Get-SupportedEncryptionTypesSet{
    param(
        [parameter(Mandatory = $true)]
        [int]$SupportedEncryptionTypesValue
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    $SupportedEncryptionTypesKeys = @{
        0 = "RC4_HMAC_MD5"
        1 = "DES_DES_CBC_CRC"
        2 = "DES_CBC_MD5"
        4 = "RC4_HMAC_MD5"
        8 = "AES 128"
        16 = "AES 256"
    }

    $supportedEncryptionTypeList = [System.Collections.ArrayList]@()

    foreach($k in $SupportedEncryptionTypesKeys.Keys){
        if($SupportedEncryptionTypesValue -band $k){
            $supportedEncryptionTypeList.Add($SupportedEncryptionTypesKeys[$k]) | out-null
        }
    }
    $supportedEncryptionTypeList
}

<#
.SYNOPSIS
Get a string representation of MsDS-SupportedEncryptionTypes from its decimal value

.DESCRIPTION
Get a string representation of MsDS-SupportedEncryptionTypes from its decimal value

.PARAMETER SupportedEncryptionTypesValue
The decimal value of MsDS-SupportedEncryptionTypes

.EXAMPLE
Get-SupportedEncryptionTypesSet $supportedEncryptionType

#>
Function Get-SupportedEncryptionTypesSet{
    param(
        [parameter(Mandatory = $true)]
        [int]$SupportedEncryptionTypesValue
    )
    Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

    $SupportedEncryptionTypesKeys = @{
        0 = "RC4_HMAC_MD5"
        1 = "DES_DES_CBC_CRC"
        2 = "DES_CBC_MD5"
        4 = "RC4_HMAC_MD5"
        8 = "AES 128"
        16 = "AES 256"
    }

    $supportedEncryptionTypeList = [System.Collections.ArrayList]@()

    foreach($k in $SupportedEncryptionTypesKeys.Keys){
        if($SupportedEncryptionTypesValue -band $k){
            $supportedEncryptionTypeList.Add($SupportedEncryptionTypesKeys[$k]) | out-null
        }
    }
    $supportedEncryptionTypeList
}

<#
.SYNOPSIS
Get the services that map to the HOST SPN

.DESCRIPTION
Get the services that map to the HOST SPN from the Configuration partition

.PARAMETER DomainNames
The DNS name of the domain

.PARAMETER ForestName
The DNS name of the forest root

.EXAMPLE
Get-HostSPNMapping -dnsDomain $domain -forestName $forestName

#>
Function Get-HostSPNMapping {
    param(
        [parameter(Mandatory = $true)]
        [string[]]$DomainNames,

        [parameter(Mandatory = $true)]
        [string]$ForestName
    )

    $spnMappings = [System.Collections.ArrayList]@()

    $forestDN = Get-DN -dnsDomain $forestName
    $dsDN = "CN=Directory Service,CN=Windows NT,CN=Services,CN=Configuration,$forestDN"
    $filter = "(sPNMappings=*)"

    $searchParams = @{
        attributes = "sPNMappings"
        baseDN = $dsDN
        scope = "base"
        filter = $filter
    }

    $spnMappingObject = (Search-ADConfig -forestName $forestName -domainNames $domainNames -searchADParameters $searchParams)[0]
    foreach ($mapping in $spnMappingObject.Attributes.'spnmappings'.GetValues("string")) {
        if ($mapping.ToLower().StartsWith("host=")) {
            foreach ($hostMapping in $mapping.Substring(5).Split(',')) {
                [void]$spnMappings.Add($hostMapping.ToLower())
            }
        }
    }

    return $spnMappings
}

<#
.SYNOPSIS
Gets the NetBIOS name of a given domain

.DESCRIPTION
Queries a given domain for its NetBIOS name

.PARAMETER DomainNames
Array of existing domains

.PARAMETER ForestName
DNS name of the forest root

.PARAMETER dnsDomain
DNS name of the domain whose NetBIOS name we want

.EXAMPLE
Get-NetBIOSName -dnsDomain "child.test.lab" -forestName "test.lab"
#>
Function Get-NetBIOSName {
    # Gets the domain NetBIOS name
    param(
        [parameter(Mandatory = $true)]
        [string[]]$DomainNames,

        [parameter(Mandatory = $true)]
        [string]$ForestName,

        [parameter(Mandatory = $true)]
        [string]$dnsDomain
    )

    $forestDN = Get-DN -dnsDomain $forestName
    $configDN = "CN=Configuration,$forestDN"
    $netbiosFilter = "(&(netbiosname=*)(dnsroot=$dnsDomain))"

    $searchParams = @{
        attributes = "netbiosname"
        baseDN = $configDN
        scope = "subtree"
        filter = $netbiosFilter
    }

    $netBIOSObject = Search-ADConfig -forestName $forestName -domainNames $DomainNames -searchADParameters $searchParams
    return $netBIOSObject.Attributes.netbiosname.GetValues("string")[0]
}

<#
.SYNOPSIS
Convert and compare Old and New values from a DSP search

.DESCRIPTION
Convert string values from DSP to arrays and compare

.PARAMETER dspResult
The object returned from a DSP search

.EXAMPLE
Compare-DSPMultiValuedAttribute -dspResult $dsp_result
#>
Function Compare-DSPMultiValuedAttribute {
    param(
        [parameter(Mandatory = $true)]
        [object]$dspResult
    )
    $ret = @{}
    $fromValue = $dspResult.From_StringValue
    $toValue = $dspResult.To_StringValue

    if ($fromValue) { $ret["from"] = $fromValue.TrimStart('[').TrimEnd(']').Split(',') }
    if ($toValue) { $ret["to"] = $toValue.TrimStart('[').TrimEnd(']').Split(',') }

    $ret["removed"] = $ret["from"] | Where-Object {$ret["to"] -notcontains $_}
    $ret["added"] = $ret["to"] | Where-Object {$ret["from"] -notcontains $_}

    return $ret
}

<#
.SYNOPSIS
Update output to include additional fields
.DESCRIPTION
Returns an updated version of the indicator's output if more processing is needed
.PARAMETER OutputObjects
ArrayList of objects to be returned by the indicator
.PARAMETER StartAttackWindow
DateTime specifying the start of the attack
.PARAMETER EndAttackWindow
DateTime specifying the end of the attack
.EXAMPLE
Update-GraphApiAADIndicatorOutputAttackWindow -ThisOutput $thisOutput -StartAttackWindow $StartAttackWindow -EndAttackWindow $EndAttackWindow -DateToCheck <A Date>
#>
Function Update-GraphApiAADIndicatorOutputAttackWindow {
    [OutputType([System.Collections.ArrayList])]
    [CmdletBinding(SupportsShouldProcess)]
    param (
        $ThisOutput,

        $StartAttackWindow,

        $EndAttackWindow,

        $DateToCheck


    )
    if($PSCmdlet.ShouldProcess($DateToCheck)) {
        if(!($ThisOutput)) {
            return $null
        }

        if ($StartAttackWindow -and $EndAttackWindow) {
            if(!($ThisOutput.psobject.Properties.name -contains "EventTimestamp")) {
                if (!($DateToCheck)) {
                    return $null
                }
                if ($DateToCheck){
                    if (-not ($DateToCheck.GetType().Name -eq "DateTime"))
                    {
                        $timestamp = [DateTime]::Parse($DateToCheck).ToUniversalTime()
                    }
                    else {
                        $timestamp = $DateToCheck.ToUniversalTime()
                    }
                }
                $ThisOutput | Add-Member -Type NoteProperty -Name "EventTimestamp" -Value $timestamp
            }
            if ($ThisOutput."EventTimestamp" -gt $StartAttackWindow -and $ThisOutput."EventTimestamp" -lt $EndAttackWindow) {
                $ThisOutput | Add-Member -Type NoteProperty -Name "InAttackWindow" -Value "True"
            }
            else {
                $ThisOutput | Add-Member -Type NoteProperty -Name "InAttackWindow" -Value "False"
            }
        }
    }
}

<#GPO Functions#>
$GPOValueCode = @"
using System;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
namespace GPPS
{
    public class Class1
    {
        [DllImport("kernel32.dll", CharSet = CharSet.Unicode)]
        public static extern uint GetPrivateProfileString(
        string lpAppName,
        string lpKeyName,
        string lpDefault,
        StringBuilder lpReturnedString,
        uint nSize,
        string lpFileName);
        public static string[] getValue(string fullPath, string app, string key)
        {
            StringBuilder sb2 = new StringBuilder(500);
            uint res2 = GetPrivateProfileString(app, key, "", sb2, (uint)sb2.Capacity, fullPath);
            string[] resultsArray = new string[2];
            if(res2 > 0)
            {
                resultsArray[0] = fullPath;
                resultsArray[1] = sb2.ToString();
            }
            else
            {
                resultsArray[0] = fullPath;
                resultsArray[1] = "-1";
            }

            return resultsArray;
        }

        public static Task<string[]> Check(string fullPath, string app, string key)
        {
            return Task.Run(() => getValue(fullPath, app, key));
        }
    }
}
"@
Add-Type -TypeDefinition $GPOValueCode -Language CSharp
Function Get-GPOValue {
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory)]
        [String]
        $Path,

        [Parameter(Mandatory)]
        [String]
        $app,

        [Parameter(Mandatory, ValueFromPipeline)]
        [string]
        $key
    )
    begin {
        $tasks = [System.Collections.Generic.List[System.Threading.Tasks.Task]]::new()

    }
    process{
        $out = [GPPS.Class1]::Check($path, $app, $key)
        [void]$tasks.Add($out)
    }
    end{
        [System.Threading.Tasks.Task]::WhenAll($tasks).Wait()
        $output= @()
        foreach($task in $tasks)
        {
            $tempObj = [PSCustomObject]@{
                Path = $task.Result[0]
                User = $task.Result[1]
                Priv = $key
            }
            $output += $tempObj
        }
        return $output
    }
}
$GPOPrivs = @(
    [PSCustomObject] @{
        PrivRightName = "SeTakeOwnershipPrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = "Can be used to grant yourself ownership on any file, registry key, etc."
        MsDescription = "Take ownership of files or other objects"
    },
    [PSCustomObject] @{
        PrivRightName = "SeSyncAgentPrivilege"
        GrantsRemoteAccess = $true
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = ""
        MsDescription = "Synchronize directory service data"
    },
    [PSCustomObject] @{
        PrivRightName = "SeTcbPrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = "Lets you impersonate any other user."
        MsDescription = "Act as part of the operating system"
    },
    [PSCustomObject] @{
        PrivRightName = "SeRestorePrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = "Can be used to overwrite/modify any file."
        MsDescription = "Restore files and directories"
    },
    [PSCustomObject] @{
        PrivRightName = "SeAssignPrimaryTokenPrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = "Lets you impersonate accounts (after some backflips) look at the various 'potato' attacks, i.e. Rotten, Juicy, etc."
        MsDescription = "Replace a process level token"
    },
    [PSCustomObject] @{
        PrivRightName = "SeBackupPrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = "Lets you override file and directory permissions to read any file on the FS."
        MsDescription = "Back up files and directories"
    },
    [PSCustomObject] @{
        PrivRightName = "SeCreateTokenPrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = "Lets you grant yourself any access you want. Locally."
        MsDescription = "Create a token object"
    },
    [PSCustomObject] @{
        PrivRightName = "SeDebugPrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = "Lets you do the mimikatz thing, you know, dump lsass.exe, cool stuff like that."
        MsDescription = "Debug programs"
    },
    [PSCustomObject] @{
        PrivRightName = "SeImpersonatePrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = "Lets you impersonate accounts (after some backflips) look at the various 'potato' attacks, i.e. Rotten, Juicy, etc."
        MsDescription = "Impersonate a client after authentication"
    },
    [PSCustomObject] @{
        PrivRightName = "SeLoadDriverPrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = "Lets you load device drivers. Privesc in this case is usually going to mean loading a known-vulnerable driver and then exploiting the known vulnerability."
        MsDescription = "Load and unload device drivers"
    },
    [PSCustomObject] @{
        PrivRightName = "SeCreateSymbolicLinkPrivilege"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $true
        LocalPrivescDesc = ""
        MsDescription = "Create symbolic links"
    },
    [PSCustomObject] @{
        PrivRightName = "SeServiceLogonRight"
        GrantsRemoteAccess = $false
        RemoteAccessDesc = ""
        LocalPrivesc = $false
        LocalPrivescDesc = ""
        MsDescription = "Log on as a service"
    }
)


Function Get-linkedOU {
    param(
        [parameter(Mandatory = $true)]
        [string]$ForestName
    )
    $policies  = [System.Collections.ArrayList]@()
    $forestDN = Get-DN $ForestName
    $searchParams =
    @{
        dnsDomain = $forestName
        attributes = @("gplink", "gpoptions", "name", "displayname")
        baseDN = $forestDN
        scope = "subtree"
        filter = "(&(|(objectClass=organizationalUnit)(objectClass=site)(objectClass=domain))(gplink=*))"
    }

    $results = Search-AD @searchParams
    foreach($result in $results)
    {
        $gplink = $result.Attributes.gplink[0]
        if($gplink -match "LDAP")
        {
            $gpList = $gplink.split("\[").split("\]")
            $priority = -1
            $pCount = 0
            foreach($link in $gpList)
            {
                $temp = $link  -match "LDAP://cn=({([A-Z0-9a-z]+-){4}[A-Za-z0-9]+}),"
                if($temp -and $Matches[1])
                {
                    $pCount = ($gpList.count -1)/2
                    $priority = $priority + 1
                    $policy = $Matches[1]
                    $state = $link.Split(";")[1]
                    $tempState = switch ($state)
                    {
                        "0" {"Enabled, Unenforced"}
                        "1" {"Disabled, Unenforced"}
                        "2" {"Disabled, Enforced"}
                        "3" {"Enabled, Enforced"}
                    }
                    $tempP = "OU: {0}; State: {1}, Priority: {2}" -f $result.DistinguishedName,$tempState,($pCount - $priority).ToString()

                    if($policies){
                        if($policies.Policy.Contains($policy))
                        {
                            $currPolicy = $policies | Where-Object policy -eq $policy
                            $pArr = $currPolicy.PolicyInfo
                            [void]$pArr.Add($tempP)
                        }
                        else
                        {
                            $newP = [PSCustomObject][Ordered] @{
                                Policy = $policy
                                PolicyInfo = [System.Collections.ArrayList]@()
                            }
                            [void]$newp.PolicyInfo.Add($tempP)
                            [void]$policies.Add($newP)
                        }
                    }
                    else
                    {
                        $newP = [PSCustomObject][Ordered] @{
                            Policy = $policy
                            PolicyInfo = [System.Collections.ArrayList]@()
                        }
                        [void]$newp.PolicyInfo.Add($tempP)
                        [void]$policies.Add($newP)
                    }
                }
            }
        }
    }
    return $policies
}


function Get-LinkedOUsFromGpoCn {
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory = $True)]
        $LinkedOUsHM,

        [Parameter(Mandatory = $True)]
        $gpoCN
    )

    $pState = ""

    if($LinkedOUsHM)
    {
        try{
            $tempP = $hash | Where-Object Policy -eq $gpoCN
            $pStateArr = $tempP.PolicyInfo
            $pState = $pStateArr -join " ;"
        }
        catch{
            $pState = "Unknown"
        }
    }
    else
    {
        $pState = ""
    }

    return $pState
}

function Get-UsersFromACL{
    [CmdletBinding()]
    [OutputType([System.Object[]])]
    Param(
        [Parameter(Mandatory = $True)]
        $Path,
        [Parameter(Mandatory = $True)]
        $Trusted

    )

    $acls = @("CreateFiles", "AppendData", "DeleteSubdirectoriesAndFiles", "Delete", "ChangePermissions", "TakeOwnership", "FullControl", "Write", "Modify")

    $usersFolder = [System.Collections.ArrayList]@()
    $usersFile = [System.Collections.ArrayList]@()
    $outMessage = ""

    $testPath = Test-Path -Path $Path
    # This part of the script is checking the ACL on the parent folder
    $folderPath = Split-Path -Path $Path
    try{
        $pFolderACL = Get-ACL $folderPath
    }
    catch{
        continue
    }
    foreach($acl in $pFolderACL.Access)
    {
        # Get the account SID
        $tempName = $acl.IdentityReference.ToString()
        $objUser = New-Object System.Security.Principal.NTAccount($tempName)
        try{
            $strSID = $objUser.Translate([System.Security.Principal.SecurityIdentifier])
            $sid = $strSID.Value
        }
        catch
        {
            $sid = $tempName
        }
        if($trusted.Contains($sid))
        {
            continue
        }
        # If the account is not trusted
        else
        {
            $systemRights = $acl.FileSystemRights.toString()
            foreach($right in $acls)
            {
                if($systemRights -match $right -and (!$usersFolder.Contains($acl.IdentityReference)))
                {
                    [void]$usersFolder.add($acl.IdentityReference)
                    break
                }
            }
        }
    }
    #end here the check for the ACL of the parent folder
    # If the path exists - check also the ACL on the file itself
    if($testPath)
    {
        # Message
        $outMessage =  "The file exists but some low privilege user(s) can modify it."
        #This part of the script is checking the ACL on the file itself
        try{
            $fileACL = Get-Acl $Path -ErrorAction Stop
        }
        catch{
            continue
        }
        foreach($acl in $fileACL.Access)
        {
            # Get the account SID
            $tempName = $acl.IdentityReference.ToString()
            $objUser = New-Object System.Security.Principal.NTAccount($tempName)
            try{
                $strSID = $objUser.Translate([System.Security.Principal.SecurityIdentifier])
                $sid = $strSID.Value
            }
            catch
            {
                $sid = $tempName
            }
            # Check if the SID is trusted
            if($trusted.Contains($sid))
            {
                continue
            }
            # If the account is not trusted
            else
            {
                $systemRights = $acl.FileSystemRights.toString()
                foreach($right in $acls)
                {
                    if($systemRights -match $right -and (!$usersFile.Contains($acl.IdentityReference)))
                    {
                        [void]$usersFile.add($acl.IdentityReference)
                        break
                    }
                }
            }
        }
        #end here the check for the ACL of the file itself
    }
    else
    {
        $outMessage =  "The file doesn't exists but some low privilege user(s) can modify the parent folder."
    }

    # Create valid string for low privilege users that can modify the parent folder
    if($usersFolder)
    {
        $outFolder = $usersFolder -join " ;"
    }
    else
    {
        $outFolder = "None"
    }
    # Create valid string for low privilege users that can modify the file itself
    if($usersFile)
    {
        $outFile = $usersFile -join " ;"
    }
    else
    {
        $outFile = "None"
    }

    return $outFile, $outFolder, $outMessage

}


function Get-IniContent {
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory = $True)]
        [String[]]
        $Path,

        [Management.Automation.PSCredential]
        [Management.Automation.CredentialAttribute()]
        $Credential = [Management.Automation.PSCredential]::Empty,

        [Switch]
        $OutputObject
    )

    PROCESS {
        ForEach ($TargetPath in $Path) {
            if (Test-Path -Path $TargetPath) {
                if ($PSBoundParameters['OutputObject']) {
                    $IniObject = New-Object PSObject
                }
                else {
                    $IniObject = @{}
                }
                Switch -Regex -File $TargetPath {
                    "^\[(.+)\]" # Section
                    {
                        $Section = $matches[1].Trim()
                        if ($PSBoundParameters['OutputObject']) {
                            $Section = $Section.Replace(' ', '')
                            $SectionObject = New-Object PSObject
                            $IniObject | Add-Member Noteproperty $Section $SectionObject
                        }
                        else {
                            $IniObject[$Section] = @{}
                        }
                        $CommentCount = 0
                    }
                    "^(;.*)$" # Comment
                    {
                        $Value = $matches[1].Trim()
                        $CommentCount = $CommentCount + 1
                        $Name = 'Comment' + $CommentCount
                        if ($PSBoundParameters['OutputObject']) {
                            $Name = $Name.Replace(' ', '')
                            $IniObject.$Section | Add-Member Noteproperty $Name $Value
                        }
                        else {
                            $IniObject[$Section][$Name] = $Value
                        }
                    }
                    "(.+?)\s*=(.*)" # Key
                    {
                        $Name, $Value = $matches[1..2]
                        $Name = $Name.Trim()
                        $Values = $Value.split(',') | ForEach-Object { $_.Trim() }

                        # if ($Values -isnot [System.Array]) { $Values = @($Values) }

                        if ($PSBoundParameters['OutputObject']) {
                            $Name = $Name.Replace(' ', '')
                            $IniObject.$Section | Add-Member Noteproperty $Name $Values
                        }
                        else {
                            $IniObject[$Section][$Name] = $Values
                        }
                    }
                }
                $IniObject
            }
        }
    }
}

<#
.SYNOPSIS
Convert $DomainNames variable elements to be all lowercase
.DESCRIPTION
Returns a $DomainNames variable with all the elements lowercased
.PARAMETER DomainNames
ArrayList of strings to be lowercased and returned by the indicator
.EXAMPLE
$DomainNames = ConvertTo-Lowercase -DomainNames $DomainNames
#>
function ConvertTo-Lowercase {
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory = $True)]
        [String[]]
        $DomainNames
    )

    PROCESS {
        Out-Log -Operation $MyInvocation.MyCommand.Name -Data $PSBoundParameters

        $out = [string[]]@()
        foreach ($domain in $DomainNames) {
            $out += $domain.ToLower()
        }

        return $out
    }
}

Export-ModuleMember -Function Out-Log
Export-ModuleMember -Function Search-AD
Export-ModuleMember -Function Get-DN
Export-ModuleMember -Function Get-DomainSID
Export-ModuleMember -Function Get-UACSet
Export-ModuleMember -Function Get-SupportedEncryptionTypesSet
Export-ModuleMember -Function Update-GraphApiAADIndicatorOutputAttackWindow
Export-ModuleMember -Function Get-ADSearchFlag
Export-ModuleMember -Function Convert-StringToXml
Export-ModuleMember -Function Get-TrustAttributesSet
Export-ModuleMember -Function Update-NTSecurityDescriptor
Export-ModuleMember -Function Update-AttributeMetadata
Export-ModuleMember -Function Update-ValueMetadata
Export-ModuleMember -Function Get-AdminsInDomain
Export-ModuleMember -Function Update-IndicatorOutputAttackWindow
Export-ModuleMember -Function Search-ADHelper
Export-ModuleMember -Function Get-NetBIOSName
Export-ModuleMember -Function Get-HostSPNMapping
Export-ModuleMember -Function Compare-DSPMultiValuedAttribute
Export-ModuleMember -Function Confirm-DomainAvailability
Export-ModuleMember -Function Get-ForestSID
Export-ModuleMember -Function Search-ADConfig
Export-ModuleMember -Function Get-linkedOU
Export-ModuleMember -Function Get-LinkedOUsFromGpoCn
Export-ModuleMember -Function Get-UsersFromACL
Export-ModuleMember -Function Get-GPOValue
Export-ModuleMember -Function Get-IniContent
Export-ModuleMember -Function ConvertTo-Lowercase
Export-ModuleMember -Variable GPOPrivs
# SIG # Begin signature block
# MIItmQYJKoZIhvcNAQcCoIItijCCLYYCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCD9sHCMBGfIEyYE
# AMJv4rfPpSwOaOg9EKt6Uqzv/jT95KCCJrkwggWNMIIEdaADAgECAhAOmxiO+dAt
# 5+/bUOIIQBhaMA0GCSqGSIb3DQEBDAUAMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNV
# BAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0yMjA4MDEwMDAwMDBa
# Fw0zMTExMDkyMzU5NTlaMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lD
# ZXJ0IFRydXN0ZWQgUm9vdCBHNDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAL/mkHNo3rvkXUo8MCIwaTPswqclLskhPfKK2FnC4SmnPVirdprNrnsbhA3E
# MB/zG6Q4FutWxpdtHauyefLKEdLkX9YFPFIPUh/GnhWlfr6fqVcWWVVyr2iTcMKy
# unWZanMylNEQRBAu34LzB4TmdDttceItDBvuINXJIB1jKS3O7F5OyJP4IWGbNOsF
# xl7sWxq868nPzaw0QF+xembud8hIqGZXV59UWI4MK7dPpzDZVu7Ke13jrclPXuU1
# 5zHL2pNe3I6PgNq2kZhAkHnDeMe2scS1ahg4AxCN2NQ3pC4FfYj1gj4QkXCrVYJB
# MtfbBHMqbpEBfCFM1LyuGwN1XXhm2ToxRJozQL8I11pJpMLmqaBn3aQnvKFPObUR
# WBf3JFxGj2T3wWmIdph2PVldQnaHiZdpekjw4KISG2aadMreSx7nDmOu5tTvkpI6
# nj3cAORFJYm2mkQZK37AlLTSYW3rM9nF30sEAMx9HJXDj/chsrIRt7t/8tWMcCxB
# YKqxYxhElRp2Yn72gLD76GSmM9GJB+G9t+ZDpBi4pncB4Q+UDCEdslQpJYls5Q5S
# UUd0viastkF13nqsX40/ybzTQRESW+UQUOsxxcpyFiIJ33xMdT9j7CFfxCBRa2+x
# q4aLT8LWRV+dIPyhHsXAj6KxfgommfXkaS+YHS312amyHeUbAgMBAAGjggE6MIIB
# NjAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBTs1+OC0nFdZEzfLmc/57qYrhwP
# TzAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzAOBgNVHQ8BAf8EBAMC
# AYYweQYIKwYBBQUHAQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdp
# Y2VydC5jb20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNv
# bS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcnQwRQYDVR0fBD4wPDA6oDigNoY0
# aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENB
# LmNybDARBgNVHSAECjAIMAYGBFUdIAAwDQYJKoZIhvcNAQEMBQADggEBAHCgv0Nc
# Vec4X6CjdBs9thbX979XB72arKGHLOyFXqkauyL4hxppVCLtpIh3bb0aFPQTSnov
# Lbc47/T/gLn4offyct4kvFIDyE7QKt76LVbP+fT3rDB6mouyXtTP0UNEm0Mh65Zy
# oUi0mcudT6cGAxN3J0TU53/oWajwvy8LpunyNDzs9wPHh6jSTEAZNUZqaVSwuKFW
# juyk1T3osdz9HNj0d1pcVIxv76FQPfx2CWiEn2/K2yCNNWAcAgPLILCsWKAOQGPF
# mCLBsln1VWvPJ6tsds5vIy30fnFqI2si/xK4VC0nftg62fC2h5b9W9FcrBjDTZ9z
# twGpn1eqXijiuZQwggWiMIIEiqADAgECAhB4AxhCRXCKQc9vAbjutKlUMA0GCSqG
# SIb3DQEBDAUAMEwxIDAeBgNVBAsTF0dsb2JhbFNpZ24gUm9vdCBDQSAtIFIzMRMw
# EQYDVQQKEwpHbG9iYWxTaWduMRMwEQYDVQQDEwpHbG9iYWxTaWduMB4XDTIwMDcy
# ODAwMDAwMFoXDTI5MDMxODAwMDAwMFowUzELMAkGA1UEBhMCQkUxGTAXBgNVBAoT
# EEdsb2JhbFNpZ24gbnYtc2ExKTAnBgNVBAMTIEdsb2JhbFNpZ24gQ29kZSBTaWdu
# aW5nIFJvb3QgUjQ1MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAti3F
# MN166KuQPQNysDpLmRZhsuX/pWcdNxzlfuyTg6qE9aNDm5hFirhjV12bAIgEJen4
# aJJLgthLyUoD86h/ao+KYSe9oUTQ/fU/IsKjT5GNswWyKIKRXftZiAULlwbCmPgs
# pzMk7lA6QczwoLB7HU3SqFg4lunf+RuRu4sQLNLHQx2iCXShgK975jMKDFlrjrz0
# q1qXe3+uVfuE8ID+hEzX4rq9xHWhb71hEHREspgH4nSr/2jcbCY+6R/l4ASHrTDT
# DI0DfFW4FnBcJHggJetnZ4iruk40mGtwEd44ytS+ocCc4d8eAgHYO+FnQ4S2z/x0
# ty+Eo7+6CTc9Z2yxRVwZYatBg/WsHet3DUZHc86/vZWV7Z0riBD++ljop1fhs8+o
# WukHJZsSxJ6Acj2T3IyU3ztE5iaA/NLDA/CMDNJF1i7nj5ie5gTuQm5nfkIWcWLn
# BPlgxmShtpyBIU4rxm1olIbGmXRzZzF6kfLUjHlufKa7fkZvTcWFEivPmiJECKiF
# N84HYVcGFxIkwMQxc6GYNVdHfhA6RdktpFGQmKmgBzfEZRqqHGsWd/enl+w/GTCZ
# bzH76kCy59LE+snQ8FB2dFn6jW0XMr746X4D9OeHdZrUSpEshQMTAitCgPKJajbP
# yEygzp74y42tFqfT3tWbGKfGkjrxgmPxLg4kZN8CAwEAAaOCAXcwggFzMA4GA1Ud
# DwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzAPBgNVHRMBAf8EBTADAQH/
# MB0GA1UdDgQWBBQfAL9GgAr8eDm3pbRD2VZQu86WOzAfBgNVHSMEGDAWgBSP8Et/
# qC5FJK5NUPpjmove4t0bvDB6BggrBgEFBQcBAQRuMGwwLQYIKwYBBQUHMAGGIWh0
# dHA6Ly9vY3NwLmdsb2JhbHNpZ24uY29tL3Jvb3RyMzA7BggrBgEFBQcwAoYvaHR0
# cDovL3NlY3VyZS5nbG9iYWxzaWduLmNvbS9jYWNlcnQvcm9vdC1yMy5jcnQwNgYD
# VR0fBC8wLTAroCmgJ4YlaHR0cDovL2NybC5nbG9iYWxzaWduLmNvbS9yb290LXIz
# LmNybDBHBgNVHSAEQDA+MDwGBFUdIAAwNDAyBggrBgEFBQcCARYmaHR0cHM6Ly93
# d3cuZ2xvYmFsc2lnbi5jb20vcmVwb3NpdG9yeS8wDQYJKoZIhvcNAQEMBQADggEB
# AKz3zBWLMHmoHQsoiBkJ1xx//oa9e1ozbg1nDnti2eEYXLC9E10dI645UHY3qkT9
# XwEjWYZWTMytvGQTFDCkIKjgP+icctx+89gMI7qoLao89uyfhzEHZfU5p1GCdeHy
# L5f20eFlloNk/qEdUfu1JJv10ndpvIUsXPpYd9Gup7EL4tZ3u6m0NEqpbz308w2V
# Xeb5ekWwJRcxLtv3D2jmgx+p9+XUnZiM02FLL8Mofnrekw60faAKbZLEtGY/fadY
# 7qz37MMIAas4/AocqcWXsojICQIZ9lyaGvFNbDDUswarAGBIDXirzxetkpNiIHd1
# bL3IMrTcTevZ38GQlim9wX8wggauMIIElqADAgECAhAHNje3JFR82Ees/ShmKl5b
# MA0GCSqGSIb3DQEBCwUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lD
# ZXJ0IFRydXN0ZWQgUm9vdCBHNDAeFw0yMjAzMjMwMDAwMDBaFw0zNzAzMjIyMzU5
# NTlaMGMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjE7MDkG
# A1UEAxMyRGlnaUNlcnQgVHJ1c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBUaW1lU3Rh
# bXBpbmcgQ0EwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDGhjUGSbPB
# PXJJUVXHJQPE8pE3qZdRodbSg9GeTKJtoLDMg/la9hGhRBVCX6SI82j6ffOciQt/
# nR+eDzMfUBMLJnOWbfhXqAJ9/UO0hNoR8XOxs+4rgISKIhjf69o9xBd/qxkrPkLc
# Z47qUT3w1lbU5ygt69OxtXXnHwZljZQp09nsad/ZkIdGAHvbREGJ3HxqV3rwN3mf
# XazL6IRktFLydkf3YYMZ3V+0VAshaG43IbtArF+y3kp9zvU5EmfvDqVjbOSmxR3N
# Ng1c1eYbqMFkdECnwHLFuk4fsbVYTXn+149zk6wsOeKlSNbwsDETqVcplicu9Yem
# j052FVUmcJgmf6AaRyBD40NjgHt1biclkJg6OBGz9vae5jtb7IHeIhTZgirHkr+g
# 3uM+onP65x9abJTyUpURK1h0QCirc0PO30qhHGs4xSnzyqqWc0Jon7ZGs506o9UD
# 4L/wojzKQtwYSH8UNM/STKvvmz3+DrhkKvp1KCRB7UK/BZxmSVJQ9FHzNklNiyDS
# LFc1eSuo80VgvCONWPfcYd6T/jnA+bIwpUzX6ZhKWD7TA4j+s4/TXkt2ElGTyYwM
# O1uKIqjBJgj5FBASA31fI7tk42PgpuE+9sJ0sj8eCXbsq11GdeJgo1gJASgADoRU
# 7s7pXcheMBK9Rp6103a50g5rmQzSM7TNsQIDAQABo4IBXTCCAVkwEgYDVR0TAQH/
# BAgwBgEB/wIBADAdBgNVHQ4EFgQUuhbZbU2FL3MpdpovdYxqII+eyG8wHwYDVR0j
# BBgwFoAU7NfjgtJxXWRM3y5nP+e6mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMGA1Ud
# JQQMMAoGCCsGAQUFBwMIMHcGCCsGAQUFBwEBBGswaTAkBggrBgEFBQcwAYYYaHR0
# cDovL29jc3AuZGlnaWNlcnQuY29tMEEGCCsGAQUFBzAChjVodHRwOi8vY2FjZXJ0
# cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNydDBDBgNVHR8E
# PDA6MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVz
# dGVkUm9vdEc0LmNybDAgBgNVHSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEw
# DQYJKoZIhvcNAQELBQADggIBAH1ZjsCTtm+YqUQiAX5m1tghQuGwGC4QTRPPMFPO
# vxj7x1Bd4ksp+3CKDaopafxpwc8dB+k+YMjYC+VcW9dth/qEICU0MWfNthKWb8RQ
# TGIdDAiCqBa9qVbPFXONASIlzpVpP0d3+3J0FNf/q0+KLHqrhc1DX+1gtqpPkWae
# LJ7giqzl/Yy8ZCaHbJK9nXzQcAp876i8dU+6WvepELJd6f8oVInw1YpxdmXazPBy
# oyP6wCeCRK6ZJxurJB4mwbfeKuv2nrF5mYGjVoarCkXJ38SNoOeY+/umnXKvxMfB
# wWpx2cYTgAnEtp/Nh4cku0+jSbl3ZpHxcpzpSwJSpzd+k1OsOx0ISQ+UzTl63f8l
# Y5knLD0/a6fxZsNBzU+2QJshIUDQtxMkzdwdeDrknq3lNHGS1yZr5Dhzq6YBT70/
# O3itTK37xJV77QpfMzmHQXh6OOmc4d0j/R0o08f56PGYX/sr2H7yRp11LB4nLCbb
# bxV7HhmLNriT1ObyF5lZynDwN7+YAN8gFk8n+2BnFqFmut1VwDophrCYoCvtlUG3
# OtUVmDG0YgkPCr2B2RP+v6TR81fZvAT6gt4y3wSJ8ADNXcL50CN/AAvkdgIm2fBl
# dkKmKYcJRyvmfxqkhQ/8mJb2VVQrH4D6wPIOK+XW+6kvRBVK5xMOHds3OBqhK/bt
# 1nz8MIIGwjCCBKqgAwIBAgIQBUSv85SdCDmmv9s/X+VhFjANBgkqhkiG9w0BAQsF
# ADBjMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4xOzA5BgNV
# BAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0YW1w
# aW5nIENBMB4XDTIzMDcxNDAwMDAwMFoXDTM0MTAxMzIzNTk1OVowSDELMAkGA1UE
# BhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMSAwHgYDVQQDExdEaWdpQ2Vy
# dCBUaW1lc3RhbXAgMjAyMzCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIB
# AKNTRYcdg45brD5UsyPgz5/X5dLnXaEOCdwvSKOXejsqnGfcYhVYwamTEafNqrJq
# 3RApih5iY2nTWJw1cb86l+uUUI8cIOrHmjsvlmbjaedp/lvD1isgHMGXlLSlUIHy
# z8sHpjBoyoNC2vx/CSSUpIIa2mq62DvKXd4ZGIX7ReoNYWyd/nFexAaaPPDFLnkP
# G2ZS48jWPl/aQ9OE9dDH9kgtXkV1lnX+3RChG4PBuOZSlbVH13gpOWvgeFmX40Qr
# StWVzu8IF+qCZE3/I+PKhu60pCFkcOvV5aDaY7Mu6QXuqvYk9R28mxyyt1/f8O52
# fTGZZUdVnUokL6wrl76f5P17cz4y7lI0+9S769SgLDSb495uZBkHNwGRDxy1Uc2q
# TGaDiGhiu7xBG3gZbeTZD+BYQfvYsSzhUa+0rRUGFOpiCBPTaR58ZE2dD9/O0V6M
# qqtQFcmzyrzXxDtoRKOlO0L9c33u3Qr/eTQQfqZcClhMAD6FaXXHg2TWdc2PEnZW
# pST618RrIbroHzSYLzrqawGw9/sqhux7UjipmAmhcbJsca8+uG+W1eEQE/5hRwqM
# /vC2x9XH3mwk8L9CgsqgcT2ckpMEtGlwJw1Pt7U20clfCKRwo+wK8REuZODLIivK
# 8SgTIUlRfgZm0zu++uuRONhRB8qUt+JQofM604qDy0B7AgMBAAGjggGLMIIBhzAO
# BgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEF
# BQcDCDAgBgNVHSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEwHwYDVR0jBBgw
# FoAUuhbZbU2FL3MpdpovdYxqII+eyG8wHQYDVR0OBBYEFKW27xPn783QZKHVVqll
# MaPe1eNJMFoGA1UdHwRTMFEwT6BNoEuGSWh0dHA6Ly9jcmwzLmRpZ2ljZXJ0LmNv
# bS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1lU3RhbXBpbmdDQS5j
# cmwwgZAGCCsGAQUFBwEBBIGDMIGAMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5k
# aWdpY2VydC5jb20wWAYIKwYBBQUHMAKGTGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1lU3RhbXBpbmdD
# QS5jcnQwDQYJKoZIhvcNAQELBQADggIBAIEa1t6gqbWYF7xwjU+KPGic2CX/yyzk
# zepdIpLsjCICqbjPgKjZ5+PF7SaCinEvGN1Ott5s1+FgnCvt7T1IjrhrunxdvcJh
# N2hJd6PrkKoS1yeF844ektrCQDifXcigLiV4JZ0qBXqEKZi2V3mP2yZWK7Dzp703
# DNiYdk9WuVLCtp04qYHnbUFcjGnRuSvExnvPnPp44pMadqJpddNQ5EQSviANnqlE
# 0PjlSXcIWiHFtM+YlRpUurm8wWkZus8W8oM3NG6wQSbd3lqXTzON1I13fXVFoaVY
# JmoDRd7ZULVQjK9WvUzF4UbFKNOt50MAcN7MmJ4ZiQPq1JE3701S88lgIcRWR+3a
# EUuMMsOI5ljitts++V+wQtaP4xeR0arAVeOGv6wnLEHQmjNKqDbUuXKWfpd5OEhf
# ysLcPTLfddY2Z1qJ+Panx+VPNTwAvb6cKmx5AdzaROY63jg7B145WPR8czFVoIAR
# yxQMfq68/qTreWWqaNYiyjvrmoI1VygWy2nyMpqy0tg6uLFGhmu6F/3Ed2wVbK6r
# r3M66ElGt9V/zLY4wNjsHPW2obhDLN9OTH0eaHDAdwrUAuBcYLso/zjlUlrWrBci
# I0707NMX+1Br/wd3H3GXREHJuEbTbDJ8WC9nR2XlG3O2mflrLAZG70Ee8PBf4NvZ
# rZCARK+AEEGKMIIG5jCCBM6gAwIBAgIQd70OA6G3CPhUqwZyENkERzANBgkqhkiG
# 9w0BAQsFADBTMQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFsU2lnbiBudi1z
# YTEpMCcGA1UEAxMgR2xvYmFsU2lnbiBDb2RlIFNpZ25pbmcgUm9vdCBSNDUwHhcN
# MjAwNzI4MDAwMDAwWhcNMzAwNzI4MDAwMDAwWjBZMQswCQYDVQQGEwJCRTEZMBcG
# A1UEChMQR2xvYmFsU2lnbiBudi1zYTEvMC0GA1UEAxMmR2xvYmFsU2lnbiBHQ0Mg
# UjQ1IENvZGVTaWduaW5nIENBIDIwMjAwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAw
# ggIKAoICAQDWQk3540/GI/RsHYGmMPdIPc/Q5Y3lICKWB0Q1XQbPDx1wYOYmVPpT
# I2ACqF8CAveOyW49qXgFvY71TxkkmXzPERabH3tr0qN7aGV3q9ixLD/TcgYyXFus
# UGcsJU1WBjb8wWJMfX2GFpWaXVS6UNCwf6JEGenWbmw+E8KfEdRfNFtRaDFjCvhb
# 0N66WV8xr4loOEA+COhTZ05jtiGO792NhUFVnhy8N9yVoMRxpx8bpUluCiBZfomj
# WBWXACVp397CalBlTlP7a6GfGB6KDl9UXr3gW8/yDATS3gihECb3svN6LsKOlsE/
# zqXa9FkojDdloTGWC46kdncVSYRmgiXnQwp3UrGZUUL/obLdnNLcGNnBhqlAHUGX
# Yoa8qP+ix2MXBv1mejaUASCJeB+Q9HupUk5qT1QGKoCvnsdQQvplCuMB9LFurA6o
# 44EZqDjIngMohqR0p0eVfnJaKnsVahzEaeawvkAZmcvSfVVOIpwQ4KFbw7MueovE
# 3vFLH4woeTBFf2wTtj0s/y1KiirsKA8tytScmIpKbVo2LC/fusviQUoIdxiIrTVh
# lBLzpHLr7jaep1EnkTz3ohrM/Ifll+FRh2npIsyDwLcPRWwH4UNP1IxKzs9jsbWk
# EHr5DQwosGs0/iFoJ2/s+PomhFt1Qs2JJnlZnWurY3FikCUNCCDx/wIDAQABo4IB
# rjCCAaowDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMBIGA1Ud
# EwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFNqzjcAkkKNrd9MMoFndIWdkdgt4MB8G
# A1UdIwQYMBaAFB8Av0aACvx4ObeltEPZVlC7zpY7MIGTBggrBgEFBQcBAQSBhjCB
# gzA5BggrBgEFBQcwAYYtaHR0cDovL29jc3AuZ2xvYmFsc2lnbi5jb20vY29kZXNp
# Z25pbmdyb290cjQ1MEYGCCsGAQUFBzAChjpodHRwOi8vc2VjdXJlLmdsb2JhbHNp
# Z24uY29tL2NhY2VydC9jb2Rlc2lnbmluZ3Jvb3RyNDUuY3J0MEEGA1UdHwQ6MDgw
# NqA0oDKGMGh0dHA6Ly9jcmwuZ2xvYmFsc2lnbi5jb20vY29kZXNpZ25pbmdyb290
# cjQ1LmNybDBWBgNVHSAETzBNMEEGCSsGAQQBoDIBMjA0MDIGCCsGAQUFBwIBFiZo
# dHRwczovL3d3dy5nbG9iYWxzaWduLmNvbS9yZXBvc2l0b3J5LzAIBgZngQwBBAEw
# DQYJKoZIhvcNAQELBQADggIBAAiIcibGr/qsXwbAqoyQ2tCywKKX/24TMhZU/T70
# MBGfj5j5m1Ld8qIW7tl4laaafGG4BLX468v0YREz9mUltxFCi9hpbsf/lbSBQ6l+
# rr+C1k3MEaODcWoQXhkFp+dsf1b0qFzDTgmtWWu4+X6lLrj83g7CoPuwBNQTG8cn
# qbmqLTE7z0ZMnetM7LwunPGHo384aV9BQGf2U33qQe+OPfup1BE4Rt886/bNIr0T
# zfDh5uUzoL485HjVG8wg8jBzsCIc9oTWm1wAAuEoUkv/EktA6u6wGgYGnoTm5/Db
# hEb7c9krQrbJVzTHFsCm6yG5qg73/tvK67wXy7hn6+M+T9uplIZkVckJCsDZBHFK
# EUtaZMO8eHitTEcmZQeZ1c02YKEzU7P2eyrViUA8caWr+JlZ/eObkkvdBb0LDHgG
# K89T2L0SmlsnhoU/kb7geIBzVN+nHWcrarauTYmAJAhScFDzAf9Eri+a4OFJCOHh
# W9c40Z4Kip2UJ5vKo7nb4jZq42+5WGLgNng2AfrBp4l6JlOjXLvSsuuKy2MIL/4e
# 81Yp4jWb2P/ppb1tS1ksiSwvUru1KZDaQ0e8ct282b+Awdywq7RLHVg2N2Trm+GF
# F5opov3mCNKS/6D4fOHpp9Ewjl8mUCvHouKXd4rv2E0+JuuZQGDzPGcMtghyKTVT
# gTTcMIIHHDCCBQSgAwIBAgIMSUXsaWVAFuihgz78MA0GCSqGSIb3DQEBCwUAMFkx
# CzAJBgNVBAYTAkJFMRkwFwYDVQQKExBHbG9iYWxTaWduIG52LXNhMS8wLQYDVQQD
# EyZHbG9iYWxTaWduIEdDQyBSNDUgQ29kZVNpZ25pbmcgQ0EgMjAyMDAeFw0yMjAz
# MDIxMTQ4MzlaFw0yNDA2MTQxNDMwNDRaMIGJMQswCQYDVQQGEwJVUzERMA8GA1UE
# CBMITmV3IFlvcmsxETAPBgNVBAcTCE5ldyBZb3JrMRYwFAYDVQQKEw1TRU1QRVJJ
# UyBJTkMuMRYwFAYDVQQDEw1TRU1QRVJJUyBJTkMuMSQwIgYJKoZIhvcNAQkBFhVj
# b2Rlc2lnbkBzZW1wZXJpcy5jb20wggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIK
# AoICAQCvq2XdKWZ+xqIQ58x22njO9MSseih/t2TsnekKeiimUgSIKHG7TLypz2Vx
# 0+PumvxEm9JsbXXQKx9XobNZ69lRUlaZCq8Tia6DMvGEUAEvi3SvwdlJJr+AowSd
# 1evdJSHB1suaDfwX7K7PIaiIdjnl5wwc9ajPz20FiV7OUhIjYLT4T9FALz9zxJN+
# WlcUJUSmoXAmdjXHmvP6MQNO7whPI2auFvrq7OiA1N2CyXTFqwz1OkTlUqtZIsCD
# fuIWuPQZGPB3+nNN87e1eEfjO3vbwXTge649/7cXcRMuFKyuTWUE3AxX5QFWM8mt
# WujVuu8QefsPzwGJwaRP+oWHT5HmNS7s7wEn7+DggenjIo62n85ySnTliLofjl9f
# flxB10Hq3BYgVsZyNEYBp1hRKRpxQ58ZHMAkpA4liY3chI4klPWGgcIW1H2yqM3S
# lBraTVuT9skaZOua3u5Q8nlRyeXWoeBmQ8PlH0RFdeNnTRxfmY7HmXSCnG+D7TVH
# hzJPLGBHMC4FTjqro/wRKHiGLIkz6edqjz54TxKv9U39ULsIK0sQZaXb8STUp6eK
# K5lKt0wWedojQbi2qob3uZIgGg+4rAEKT4Wm8ZdDy7eN3aZTw/i5mjiEfwyJvgvz
# WT/yhJz6+ZDMQGIFJj9L2DaPnUSejLMC7K8kyZwJUXVGn1NRTQIDAQABo4IBsTCC
# Aa0wDgYDVR0PAQH/BAQDAgeAMIGbBggrBgEFBQcBAQSBjjCBizBKBggrBgEFBQcw
# AoY+aHR0cDovL3NlY3VyZS5nbG9iYWxzaWduLmNvbS9jYWNlcnQvZ3NnY2NyNDVj
# b2Rlc2lnbmNhMjAyMC5jcnQwPQYIKwYBBQUHMAGGMWh0dHA6Ly9vY3NwLmdsb2Jh
# bHNpZ24uY29tL2dzZ2NjcjQ1Y29kZXNpZ25jYTIwMjAwVgYDVR0gBE8wTTBBBgkr
# BgEEAaAyATIwNDAyBggrBgEFBQcCARYmaHR0cHM6Ly93d3cuZ2xvYmFsc2lnbi5j
# b20vcmVwb3NpdG9yeS8wCAYGZ4EMAQQBMAkGA1UdEwQCMAAwRQYDVR0fBD4wPDA6
# oDigNoY0aHR0cDovL2NybC5nbG9iYWxzaWduLmNvbS9nc2djY3I0NWNvZGVzaWdu
# Y2EyMDIwLmNybDATBgNVHSUEDDAKBggrBgEFBQcDAzAfBgNVHSMEGDAWgBTas43A
# JJCja3fTDKBZ3SFnZHYLeDAdBgNVHQ4EFgQUeXYlhOREXUlIPR2ZpkQmsxInWlkw
# DQYJKoZIhvcNAQELBQADggIBALVBZWrw/axZwZgcg+pO0wYi/phXxE1hUOxLaeRJ
# SxTEYZk7QasYoM2fR6mjUzV1da1sQKSK9OtUDbHcq7/JXKwcdKWIttwv9AI+RtdP
# yGvezvVfne1sBCn3qIxK3/5FRFdYftucE4It87UF4f25rS5FgGnLc5h+4MQgKjCv
# hI1F/aJjTBKhZnzlCwGUyAssXKYmWf0+PDxJixU5iTB8yK/VpPu4LFWWRi0pb1uS
# bJse4qIR25DBJ+zkbN6BIUz5Zx8LgmoGFVdEHFCWNfndjWtvd4DwJUtMdk9Wpdcz
# x2LrbzLUjKVN1PNdyuIWMKYxhnSt74U2VUFPoqEn6c/6lxKattXL2qD8FfV9APlc
# ZPrdV1Cnip9Mrbm5YdYebQaU4xH9LILkcSakBJSQvm0ev6f70ahHJLwiNCinftEn
# WEqgFeUfRrLJBYe8tCMIzkfH4BFIOrcd16ordOgC5QMn4bmqCvQE/df0Cngrt9Kf
# ynSBp+kx1IFZGeZ8iE9rQThcQwD75dC1/L6ADiev/TMPTm4H6+lKWL3umGvp/d25
# p+Y0lsfjTPvGWDZ65MF3sKRS0jcrAWrSm92CWfReyOXk2FHjJjtj2hAiNZ32igVh
# arGemFH0l5wHc/4KRUDcvtfCq7k8hdJn5s+EXlrzI01OFiabyDbR2Xe8JpmJfn9E
# e+3yMYIGNjCCBjICAQEwaTBZMQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFs
# U2lnbiBudi1zYTEvMC0GA1UEAxMmR2xvYmFsU2lnbiBHQ0MgUjQ1IENvZGVTaWdu
# aW5nIENBIDIwMjACDElF7GllQBbooYM+/DANBglghkgBZQMEAgEFAKB8MBAGCisG
# AQQBgjcCAQwxAjAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQB
# gjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEiBCAPHtAwP1YHqLND
# qtXQzw83p3pVMwIVGvyuDC2Tf0CuMjANBgkqhkiG9w0BAQEFAASCAgBiMLIUy+pc
# XSS7qD1VmjnBBGR3k4HC5ciblP1VFUi4ljH28D3Td6Pwh88SzkDEVaKeDnAnymIF
# IJx6ZHg5xu4cRV4YNgf+xhjeiTK0U2rgiUAxxzh2t4cUNobohDBexDIY0KyKpIjJ
# squa7KfZRpCM3rfv91JvS5clfB7+MTUj1x8fKmTa2yk2JeZs59sruG8RyDVOZWm/
# hRYCdjQS7FTXh++MsyFsM8Gmz18vNXFeAHu6FHu1YfboNTPldbWP/PY2AKMbuW2G
# c8VuPxApFmlG1LHlXGcDSuKgJuazrDCILuj3QuTfEW29W55UUFBVzr2kjrOI1aMt
# DCz/cKR7jBAk+GdOMelqmGmZWgswdUcCqM3mlJTmPmx1HEobXyk7P6gRTOOrLiCt
# VysnSBOWeKo4cSyEoUrPbnw+M7SIC83HkJgOsFGAiQYSn9FWYUZvwZkbpgw1/QCC
# dolzaPHx8fx8Fc7kD7vHPhP6MAWLBNWkDFTB+4I5vZ17J6yA6IMZn867iUWdUeXX
# cXnkJjll+VQw5oEGhoXfRnZ4xmTUy0zM1pk8uSFfp07TTukPDHh3mNCiUdhAGZvC
# qULLeJ5E0PGGyIiwnif5Q880VvllNzmewsptwliXWdmSSpFs+WvtPjHYv0D7Sp3Z
# pEuu/xYwJSOPa90PKaZAfc4OdtVAXwAaGqGCAyAwggMcBgkqhkiG9w0BCQYxggMN
# MIIDCQIBATB3MGMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5j
# LjE7MDkGA1UEAxMyRGlnaUNlcnQgVHJ1c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBU
# aW1lU3RhbXBpbmcgQ0ECEAVEr/OUnQg5pr/bP1/lYRYwDQYJYIZIAWUDBAIBBQCg
# aTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0yMzEy
# MDUxMTM4MjNaMC8GCSqGSIb3DQEJBDEiBCCohrfHpKCa5FFbdjjPyB3a9x2VpcwX
# Tdr37K8ZJccsAjANBgkqhkiG9w0BAQEFAASCAgBuw/Y4w2576ikZGrRVuestry5T
# uTLorHt5L9joGoLaoyorEpb9etxnYUWDIknBEnt4awRTj7R/kNUldbxdJjQlL2Y3
# uk6VvG7cBDdtmK8tb9sh4a8nccPPDbFIz/Ijolfg7qSp8VSt+UTF80s8sDBmWVG5
# +8PLtrRnjNxUNkFRDmi+uOf+GicM1nY3c3AK8JWJXn0V9id3xa+6+KoNkI5sMrhR
# ak/KwuZ8QLYGbK8Z3of591SF4JlXj1dNt6o73atuTp76kA2J+pIelFGnQFX7rmos
# 7Z4LZwAywf3uhuJz07cip6Y4iwomyp2RKihvbgpU0c2J3/1kT7rIza2aQojVfw5V
# cakNH/tDUbpG+ZAOqJT+lLUSSf5UluCdkI2oh2gNxM9Nf9/CAzBZ0Wge0fpjrdCN
# StI1wFuUQxCh66B5k3lCgLzk9ynJqzZqe5+5mt7lRImwPLNcPMsx3/pgYWZkYDL0
# eh1+o3QEG6yRxscQI3bbHZRQAjhfVkQZuE6nrT43yAxg5P4P9Ztgo7TpS4+YolbA
# 19GXXxSNlwE/R1hyXuhU2CVTP2pKoIDn30HZlcybVzJVqXDwnH3cKOnXV1reDNp+
# Mnz04sauLmeLBoUXkULi1g5e9qYl0DTyEHv0YxLRyg6qRXqraQt1TRXnNptsRpCS
# 9fvl1GNFhBvtp3fNvw==
# SIG # End signature block
