﻿[CmdletBinding()]
Param(
    [Parameter(ParameterSetName='Execution')]$ForestName,
    [Parameter(ParameterSetName='Execution')]$DomainNames,
    [Parameter(ParameterSetName='Execution')][String]$TenantId,
    [Parameter(ParameterSetName='Execution')][String]$TenantAppId,
    [Parameter(ParameterSetName='Execution')][String]$TenantAppSecret,
    [Parameter(Mandatory=$False,ParameterSetName='Execution')]$StartAttackWindow,
    [Parameter(Mandatory=$False,ParameterSetName='Execution')]$EndAttackWindow,
    [Parameter(ParameterSetName='Metadata',Mandatory)][switch]$Metadata
)

$Global:self = @{
    ID = 206
    UUID = 'bfdae692-a6bd-40c3-ac0d-17e9e1cd43a6'
    Version = '1.40.1'
    CategoryID = 6
    ShortName = 'SI000206'
    Name = 'Application Name and Geographic Location additional contexts are disabled on MFA'
    ScriptName = 'AAD_ApplicationNameAndGeographicLocationMFA'
    Description = 'This indicator checks if Application Name and Geographic Location additional contexts are enabled on MFA. Required permissions: Policy.Read.All'
    Weight = 6
    Severity = 'Warning'
    
    LikelihoodOfCompromise = 'MFA bombing is a tactic where an attacker spams the user with MFA authentication requests and the user unknowingly or unwillingly accepts the attempt. Enabling the application name and geographic location additional contexts on MFA provides another level of security for a user sign-in. That is, when these additional contexts are enabled, the name of the application and the location based on the IP address where the sign-in originated from are displayed, which helps the user verify that the authentication request is legitimate. https://learn.microsoft.com/en-us/azure/active-directory/authentication/how-to-mfa-additional-context'
    ResultMessage = 'Application name and geographic location additional contexts on MFA are disabled.'
    Remediation = 'Enable application name and geographic location on MFA so that users will see the name of the application requesting access and the location based on the IP address where the sign-in originated from to verify that an authentication request is legitimate. To enable Application Name additional context on MFA go to -> Azure portal -> Azure Active Directory -> Security -> Authentication methods -> Microsoft Authenticator -> Configure tab, under the configure tab you will see Show application name in push and passwordless notifications, turn the status to Enabled and set to `"all users`". To enable Geographic location addidtional context on MFA go to -> Azure portal -> Azure Active Directory -> Security -> Authentication methods -> Microsoft Authenticator -> Configure tab, under the configure tab you will see Show geographic location in push and passwordless notifications, turn the status to Enabled and set to all users. Under the Basic tab select all users.'
    Types = @('IoE')
    DataSources = @('AAD.GraphAPI')
    Targets = @('AAD')
    Permissions = @('AAD.GraphAPI/Policy.Read.All')
    SecurityFrameworks = @(
        @{ Name = 'MITRE ATT&CK'; Tags = @('Initial Access') },
        @{ Name = 'MITRE D3FEND'; Tags = @('Harden - Multi-factor Authentication') }
    )
    Products = @(
        @{ Name = 'HYD'; MinVersion = '1.0'; MaxVersion = '3.0'; Licenses = @('Cloud') },
        @{ Name = 'DSP'; MinVersion = '3.6'; MaxVersion = '10'; Licenses = @('DSP-I-AAD') },
        @{ Name = 'PK'; MinVersion = '1.5'; MaxVersion = '10'; Licenses = @('Community', 'Post-Breach', 'BPIR') }
    )
    Selected = 1
}
if($Metadata){ return $self | ConvertTo-Json -Depth 8 -Compress }

Import-Module -Name Semperis-Lib

try {
    if ($PSBoundParameters['ForestName'] -and $PSBoundParameters['DomainNames']) {
        $ForestName = $ForestName.ToLower()
        $DomainNames = ConvertTo-Lowercase -DomainNames $DomainNames
    }
    $res = New-Object Semperis.PSSecurityIndicatorResult.SecurityIndicatorResult

    If (-not $TenantAppSecret) {
        $res.Status = 'NotRelevant'
        $res.ResultMessage = 'Azure AD not connected. Please complete the Identity analytics server (Hybrid server) setup.'
        return $res
    }

    $getToken = @{
        TenantId = $TenantId
        TenantAppId = $TenantAppId
        TenantAppSecret = $TenantAppSecret
    }

    $graphAccess = Get-GraphApiToken @getToken

    $getMicrosoftAuthPolicies = @{
        Uri = "https://graph.microsoft.com/beta/authenticationMethodsPolicy/authenticationMethodConfigurations/MicrosoftAuthenticator"
        AccessToken = $graphAccess
    }

    try {
        $microsoftAuthPolicy = Invoke-GraphApiRequest @getMicrosoftAuthPolicies
    }
    catch {
        return $_ | ConvertTo-GraphApiErrorResult -RequiredPrivileges 'Read Authorization Policy' -TenantAppId $TenantAppId -AccessToken $graphAccess
    }

    $displayAppInformation = $false
    $displayLocationInformation = $false

    $displayAppInformationRequiredState = $microsoftAuthPolicy.featureSettings.displayAppInformationRequiredState
    $displayLocationInformationRequiredState = $microsoftAuthPolicy.featureSettings.displayLocationInformationRequiredState

    if ($displayLocationInformationRequiredState.state -eq "enabled" -and $displayLocationInformationRequiredState.includeTarget.targetType -eq "group" -and $displayLocationInformationRequiredState.includeTarget.id -eq "all_users" ) {
        $displayLocationInformation = $true
    }
    if ($displayAppInformationRequiredState.state -eq "enabled" -and $displayAppInformationRequiredState.includeTarget.targetType -eq "group" -and $displayAppInformationRequiredState.includeTarget.id -eq "all_users" ) {
        $displayAppInformation = $true
    }


    if ((!$displayAppInformation) -and (!$displayLocationInformation))
    {
        $res.Score = 0
        $res.ResultMessage = "Application name and geographic location additional contexts on MFA are disabled."
        $res.Remediation = "Enable application name and geographic location on MFA so the users will see on their app the application name attempting to ask for sign-in approval and additionally will receive the location based on the IP address from where the sign-in request was initiated for the sign-in approval request. To enable Application Name additional context on MFA go to -> Azure portal -> Azure Active Directory -> Security -> Authentication methods -> Microsoft Authenticator -> Configure tab, under the configure tab you will see Show application name in push and passwordless notifications, turn the status to Enabled and set to `"all users`". To enable Geographic location addidtional context on MFA go to -> Azure portal -> Azure Active Directory -> Security -> Authentication methods -> Microsoft Authenticator -> Configure tab, under the configure tab you will see Show geographic location in push and passwordless notifications, turn the status to Enabled and set to `"all users`". Under the `"Basic`" tab select all users."
        $res.Status = [Semperis.PSSecurityIndicatorResult.ScriptStatus]"Failed"
    }
    elseif ($displayAppInformation -and (!$displayLocationInformation))
    {
        $res.Score = 0
        $res.ResultMessage = "Geographic location additional context on MFA is disabled."
        $res.Remediation = "Enable geographic location on MFA so the users will receive the location based on the IP address from where the sign-in request was initiated for the sign-in approval request. To enable Geographic location addidtional context on MFA go to -> Azure portal -> Azure Active Directory -> Security -> Authentication methods -> Microsoft Authenticator -> Configure tab, under the configure tab you will see Show geographic location in push and passwordless notifications, turn the status to Enabled and set to `"all users`". Under the `"Basic`" tab select all users"
        $res.Status = [Semperis.PSSecurityIndicatorResult.ScriptStatus]"Failed"
    }
    elseif ($displayLocationInformation -and (!$displayAppInformation)){
        $res.Score = 0
        $res.ResultMessage = "Application name additional context on MFA is disabled."
        $res.Remediation = "Enable application name on MFA so the users will see on their app the application name attempting to ask for sign-in approval. To enable Application Name additional context on MFA go to -> Azure portal -> Azure Active Directory -> Security -> Authentication methods -> Microsoft Authenticator -> Configure tab, under the configure tab you will see Show application name in push and passwordless notifications, turn the status to Enabled and set to `"all users`". Under the `"Basic`" tab select all users."
        $res.Status = [Semperis.PSSecurityIndicatorResult.ScriptStatus]"Failed"
    }
    else {
        $res.Score = 100
        $res.ResultMessage = "No evidence of exposure."
        $res.Remediation = "None"
        $res.Status = [Semperis.PSSecurityIndicatorResult.ScriptStatus]"Pass"
    }
}
catch {
    return ConvertTo-ErrorResult $_
}
return $res

# SIG # Begin signature block
# MIItmQYJKoZIhvcNAQcCoIItijCCLYYCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBWosSEH6jqEvS7
# FLA8D7zpFnWK0GU8mJMPpTkk5LwRlKCCJrkwggWNMIIEdaADAgECAhAOmxiO+dAt
# 5+/bUOIIQBhaMA0GCSqGSIb3DQEBDAUAMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNV
# BAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0yMjA4MDEwMDAwMDBa
# Fw0zMTExMDkyMzU5NTlaMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lD
# ZXJ0IFRydXN0ZWQgUm9vdCBHNDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAL/mkHNo3rvkXUo8MCIwaTPswqclLskhPfKK2FnC4SmnPVirdprNrnsbhA3E
# MB/zG6Q4FutWxpdtHauyefLKEdLkX9YFPFIPUh/GnhWlfr6fqVcWWVVyr2iTcMKy
# unWZanMylNEQRBAu34LzB4TmdDttceItDBvuINXJIB1jKS3O7F5OyJP4IWGbNOsF
# xl7sWxq868nPzaw0QF+xembud8hIqGZXV59UWI4MK7dPpzDZVu7Ke13jrclPXuU1
# 5zHL2pNe3I6PgNq2kZhAkHnDeMe2scS1ahg4AxCN2NQ3pC4FfYj1gj4QkXCrVYJB
# MtfbBHMqbpEBfCFM1LyuGwN1XXhm2ToxRJozQL8I11pJpMLmqaBn3aQnvKFPObUR
# WBf3JFxGj2T3wWmIdph2PVldQnaHiZdpekjw4KISG2aadMreSx7nDmOu5tTvkpI6
# nj3cAORFJYm2mkQZK37AlLTSYW3rM9nF30sEAMx9HJXDj/chsrIRt7t/8tWMcCxB
# YKqxYxhElRp2Yn72gLD76GSmM9GJB+G9t+ZDpBi4pncB4Q+UDCEdslQpJYls5Q5S
# UUd0viastkF13nqsX40/ybzTQRESW+UQUOsxxcpyFiIJ33xMdT9j7CFfxCBRa2+x
# q4aLT8LWRV+dIPyhHsXAj6KxfgommfXkaS+YHS312amyHeUbAgMBAAGjggE6MIIB
# NjAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBTs1+OC0nFdZEzfLmc/57qYrhwP
# TzAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzAOBgNVHQ8BAf8EBAMC
# AYYweQYIKwYBBQUHAQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdp
# Y2VydC5jb20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNv
# bS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcnQwRQYDVR0fBD4wPDA6oDigNoY0
# aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENB
# LmNybDARBgNVHSAECjAIMAYGBFUdIAAwDQYJKoZIhvcNAQEMBQADggEBAHCgv0Nc
# Vec4X6CjdBs9thbX979XB72arKGHLOyFXqkauyL4hxppVCLtpIh3bb0aFPQTSnov
# Lbc47/T/gLn4offyct4kvFIDyE7QKt76LVbP+fT3rDB6mouyXtTP0UNEm0Mh65Zy
# oUi0mcudT6cGAxN3J0TU53/oWajwvy8LpunyNDzs9wPHh6jSTEAZNUZqaVSwuKFW
# juyk1T3osdz9HNj0d1pcVIxv76FQPfx2CWiEn2/K2yCNNWAcAgPLILCsWKAOQGPF
# mCLBsln1VWvPJ6tsds5vIy30fnFqI2si/xK4VC0nftg62fC2h5b9W9FcrBjDTZ9z
# twGpn1eqXijiuZQwggWiMIIEiqADAgECAhB4AxhCRXCKQc9vAbjutKlUMA0GCSqG
# SIb3DQEBDAUAMEwxIDAeBgNVBAsTF0dsb2JhbFNpZ24gUm9vdCBDQSAtIFIzMRMw
# EQYDVQQKEwpHbG9iYWxTaWduMRMwEQYDVQQDEwpHbG9iYWxTaWduMB4XDTIwMDcy
# ODAwMDAwMFoXDTI5MDMxODAwMDAwMFowUzELMAkGA1UEBhMCQkUxGTAXBgNVBAoT
# EEdsb2JhbFNpZ24gbnYtc2ExKTAnBgNVBAMTIEdsb2JhbFNpZ24gQ29kZSBTaWdu
# aW5nIFJvb3QgUjQ1MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAti3F
# MN166KuQPQNysDpLmRZhsuX/pWcdNxzlfuyTg6qE9aNDm5hFirhjV12bAIgEJen4
# aJJLgthLyUoD86h/ao+KYSe9oUTQ/fU/IsKjT5GNswWyKIKRXftZiAULlwbCmPgs
# pzMk7lA6QczwoLB7HU3SqFg4lunf+RuRu4sQLNLHQx2iCXShgK975jMKDFlrjrz0
# q1qXe3+uVfuE8ID+hEzX4rq9xHWhb71hEHREspgH4nSr/2jcbCY+6R/l4ASHrTDT
# DI0DfFW4FnBcJHggJetnZ4iruk40mGtwEd44ytS+ocCc4d8eAgHYO+FnQ4S2z/x0
# ty+Eo7+6CTc9Z2yxRVwZYatBg/WsHet3DUZHc86/vZWV7Z0riBD++ljop1fhs8+o
# WukHJZsSxJ6Acj2T3IyU3ztE5iaA/NLDA/CMDNJF1i7nj5ie5gTuQm5nfkIWcWLn
# BPlgxmShtpyBIU4rxm1olIbGmXRzZzF6kfLUjHlufKa7fkZvTcWFEivPmiJECKiF
# N84HYVcGFxIkwMQxc6GYNVdHfhA6RdktpFGQmKmgBzfEZRqqHGsWd/enl+w/GTCZ
# bzH76kCy59LE+snQ8FB2dFn6jW0XMr746X4D9OeHdZrUSpEshQMTAitCgPKJajbP
# yEygzp74y42tFqfT3tWbGKfGkjrxgmPxLg4kZN8CAwEAAaOCAXcwggFzMA4GA1Ud
# DwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzAPBgNVHRMBAf8EBTADAQH/
# MB0GA1UdDgQWBBQfAL9GgAr8eDm3pbRD2VZQu86WOzAfBgNVHSMEGDAWgBSP8Et/
# qC5FJK5NUPpjmove4t0bvDB6BggrBgEFBQcBAQRuMGwwLQYIKwYBBQUHMAGGIWh0
# dHA6Ly9vY3NwLmdsb2JhbHNpZ24uY29tL3Jvb3RyMzA7BggrBgEFBQcwAoYvaHR0
# cDovL3NlY3VyZS5nbG9iYWxzaWduLmNvbS9jYWNlcnQvcm9vdC1yMy5jcnQwNgYD
# VR0fBC8wLTAroCmgJ4YlaHR0cDovL2NybC5nbG9iYWxzaWduLmNvbS9yb290LXIz
# LmNybDBHBgNVHSAEQDA+MDwGBFUdIAAwNDAyBggrBgEFBQcCARYmaHR0cHM6Ly93
# d3cuZ2xvYmFsc2lnbi5jb20vcmVwb3NpdG9yeS8wDQYJKoZIhvcNAQEMBQADggEB
# AKz3zBWLMHmoHQsoiBkJ1xx//oa9e1ozbg1nDnti2eEYXLC9E10dI645UHY3qkT9
# XwEjWYZWTMytvGQTFDCkIKjgP+icctx+89gMI7qoLao89uyfhzEHZfU5p1GCdeHy
# L5f20eFlloNk/qEdUfu1JJv10ndpvIUsXPpYd9Gup7EL4tZ3u6m0NEqpbz308w2V
# Xeb5ekWwJRcxLtv3D2jmgx+p9+XUnZiM02FLL8Mofnrekw60faAKbZLEtGY/fadY
# 7qz37MMIAas4/AocqcWXsojICQIZ9lyaGvFNbDDUswarAGBIDXirzxetkpNiIHd1
# bL3IMrTcTevZ38GQlim9wX8wggauMIIElqADAgECAhAHNje3JFR82Ees/ShmKl5b
# MA0GCSqGSIb3DQEBCwUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lD
# ZXJ0IFRydXN0ZWQgUm9vdCBHNDAeFw0yMjAzMjMwMDAwMDBaFw0zNzAzMjIyMzU5
# NTlaMGMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjE7MDkG
# A1UEAxMyRGlnaUNlcnQgVHJ1c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBUaW1lU3Rh
# bXBpbmcgQ0EwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDGhjUGSbPB
# PXJJUVXHJQPE8pE3qZdRodbSg9GeTKJtoLDMg/la9hGhRBVCX6SI82j6ffOciQt/
# nR+eDzMfUBMLJnOWbfhXqAJ9/UO0hNoR8XOxs+4rgISKIhjf69o9xBd/qxkrPkLc
# Z47qUT3w1lbU5ygt69OxtXXnHwZljZQp09nsad/ZkIdGAHvbREGJ3HxqV3rwN3mf
# XazL6IRktFLydkf3YYMZ3V+0VAshaG43IbtArF+y3kp9zvU5EmfvDqVjbOSmxR3N
# Ng1c1eYbqMFkdECnwHLFuk4fsbVYTXn+149zk6wsOeKlSNbwsDETqVcplicu9Yem
# j052FVUmcJgmf6AaRyBD40NjgHt1biclkJg6OBGz9vae5jtb7IHeIhTZgirHkr+g
# 3uM+onP65x9abJTyUpURK1h0QCirc0PO30qhHGs4xSnzyqqWc0Jon7ZGs506o9UD
# 4L/wojzKQtwYSH8UNM/STKvvmz3+DrhkKvp1KCRB7UK/BZxmSVJQ9FHzNklNiyDS
# LFc1eSuo80VgvCONWPfcYd6T/jnA+bIwpUzX6ZhKWD7TA4j+s4/TXkt2ElGTyYwM
# O1uKIqjBJgj5FBASA31fI7tk42PgpuE+9sJ0sj8eCXbsq11GdeJgo1gJASgADoRU
# 7s7pXcheMBK9Rp6103a50g5rmQzSM7TNsQIDAQABo4IBXTCCAVkwEgYDVR0TAQH/
# BAgwBgEB/wIBADAdBgNVHQ4EFgQUuhbZbU2FL3MpdpovdYxqII+eyG8wHwYDVR0j
# BBgwFoAU7NfjgtJxXWRM3y5nP+e6mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMGA1Ud
# JQQMMAoGCCsGAQUFBwMIMHcGCCsGAQUFBwEBBGswaTAkBggrBgEFBQcwAYYYaHR0
# cDovL29jc3AuZGlnaWNlcnQuY29tMEEGCCsGAQUFBzAChjVodHRwOi8vY2FjZXJ0
# cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNydDBDBgNVHR8E
# PDA6MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVz
# dGVkUm9vdEc0LmNybDAgBgNVHSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEw
# DQYJKoZIhvcNAQELBQADggIBAH1ZjsCTtm+YqUQiAX5m1tghQuGwGC4QTRPPMFPO
# vxj7x1Bd4ksp+3CKDaopafxpwc8dB+k+YMjYC+VcW9dth/qEICU0MWfNthKWb8RQ
# TGIdDAiCqBa9qVbPFXONASIlzpVpP0d3+3J0FNf/q0+KLHqrhc1DX+1gtqpPkWae
# LJ7giqzl/Yy8ZCaHbJK9nXzQcAp876i8dU+6WvepELJd6f8oVInw1YpxdmXazPBy
# oyP6wCeCRK6ZJxurJB4mwbfeKuv2nrF5mYGjVoarCkXJ38SNoOeY+/umnXKvxMfB
# wWpx2cYTgAnEtp/Nh4cku0+jSbl3ZpHxcpzpSwJSpzd+k1OsOx0ISQ+UzTl63f8l
# Y5knLD0/a6fxZsNBzU+2QJshIUDQtxMkzdwdeDrknq3lNHGS1yZr5Dhzq6YBT70/
# O3itTK37xJV77QpfMzmHQXh6OOmc4d0j/R0o08f56PGYX/sr2H7yRp11LB4nLCbb
# bxV7HhmLNriT1ObyF5lZynDwN7+YAN8gFk8n+2BnFqFmut1VwDophrCYoCvtlUG3
# OtUVmDG0YgkPCr2B2RP+v6TR81fZvAT6gt4y3wSJ8ADNXcL50CN/AAvkdgIm2fBl
# dkKmKYcJRyvmfxqkhQ/8mJb2VVQrH4D6wPIOK+XW+6kvRBVK5xMOHds3OBqhK/bt
# 1nz8MIIGwjCCBKqgAwIBAgIQBUSv85SdCDmmv9s/X+VhFjANBgkqhkiG9w0BAQsF
# ADBjMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4xOzA5BgNV
# BAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0YW1w
# aW5nIENBMB4XDTIzMDcxNDAwMDAwMFoXDTM0MTAxMzIzNTk1OVowSDELMAkGA1UE
# BhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMSAwHgYDVQQDExdEaWdpQ2Vy
# dCBUaW1lc3RhbXAgMjAyMzCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIB
# AKNTRYcdg45brD5UsyPgz5/X5dLnXaEOCdwvSKOXejsqnGfcYhVYwamTEafNqrJq
# 3RApih5iY2nTWJw1cb86l+uUUI8cIOrHmjsvlmbjaedp/lvD1isgHMGXlLSlUIHy
# z8sHpjBoyoNC2vx/CSSUpIIa2mq62DvKXd4ZGIX7ReoNYWyd/nFexAaaPPDFLnkP
# G2ZS48jWPl/aQ9OE9dDH9kgtXkV1lnX+3RChG4PBuOZSlbVH13gpOWvgeFmX40Qr
# StWVzu8IF+qCZE3/I+PKhu60pCFkcOvV5aDaY7Mu6QXuqvYk9R28mxyyt1/f8O52
# fTGZZUdVnUokL6wrl76f5P17cz4y7lI0+9S769SgLDSb495uZBkHNwGRDxy1Uc2q
# TGaDiGhiu7xBG3gZbeTZD+BYQfvYsSzhUa+0rRUGFOpiCBPTaR58ZE2dD9/O0V6M
# qqtQFcmzyrzXxDtoRKOlO0L9c33u3Qr/eTQQfqZcClhMAD6FaXXHg2TWdc2PEnZW
# pST618RrIbroHzSYLzrqawGw9/sqhux7UjipmAmhcbJsca8+uG+W1eEQE/5hRwqM
# /vC2x9XH3mwk8L9CgsqgcT2ckpMEtGlwJw1Pt7U20clfCKRwo+wK8REuZODLIivK
# 8SgTIUlRfgZm0zu++uuRONhRB8qUt+JQofM604qDy0B7AgMBAAGjggGLMIIBhzAO
# BgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEF
# BQcDCDAgBgNVHSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEwHwYDVR0jBBgw
# FoAUuhbZbU2FL3MpdpovdYxqII+eyG8wHQYDVR0OBBYEFKW27xPn783QZKHVVqll
# MaPe1eNJMFoGA1UdHwRTMFEwT6BNoEuGSWh0dHA6Ly9jcmwzLmRpZ2ljZXJ0LmNv
# bS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1lU3RhbXBpbmdDQS5j
# cmwwgZAGCCsGAQUFBwEBBIGDMIGAMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5k
# aWdpY2VydC5jb20wWAYIKwYBBQUHMAKGTGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1lU3RhbXBpbmdD
# QS5jcnQwDQYJKoZIhvcNAQELBQADggIBAIEa1t6gqbWYF7xwjU+KPGic2CX/yyzk
# zepdIpLsjCICqbjPgKjZ5+PF7SaCinEvGN1Ott5s1+FgnCvt7T1IjrhrunxdvcJh
# N2hJd6PrkKoS1yeF844ektrCQDifXcigLiV4JZ0qBXqEKZi2V3mP2yZWK7Dzp703
# DNiYdk9WuVLCtp04qYHnbUFcjGnRuSvExnvPnPp44pMadqJpddNQ5EQSviANnqlE
# 0PjlSXcIWiHFtM+YlRpUurm8wWkZus8W8oM3NG6wQSbd3lqXTzON1I13fXVFoaVY
# JmoDRd7ZULVQjK9WvUzF4UbFKNOt50MAcN7MmJ4ZiQPq1JE3701S88lgIcRWR+3a
# EUuMMsOI5ljitts++V+wQtaP4xeR0arAVeOGv6wnLEHQmjNKqDbUuXKWfpd5OEhf
# ysLcPTLfddY2Z1qJ+Panx+VPNTwAvb6cKmx5AdzaROY63jg7B145WPR8czFVoIAR
# yxQMfq68/qTreWWqaNYiyjvrmoI1VygWy2nyMpqy0tg6uLFGhmu6F/3Ed2wVbK6r
# r3M66ElGt9V/zLY4wNjsHPW2obhDLN9OTH0eaHDAdwrUAuBcYLso/zjlUlrWrBci
# I0707NMX+1Br/wd3H3GXREHJuEbTbDJ8WC9nR2XlG3O2mflrLAZG70Ee8PBf4NvZ
# rZCARK+AEEGKMIIG5jCCBM6gAwIBAgIQd70OA6G3CPhUqwZyENkERzANBgkqhkiG
# 9w0BAQsFADBTMQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFsU2lnbiBudi1z
# YTEpMCcGA1UEAxMgR2xvYmFsU2lnbiBDb2RlIFNpZ25pbmcgUm9vdCBSNDUwHhcN
# MjAwNzI4MDAwMDAwWhcNMzAwNzI4MDAwMDAwWjBZMQswCQYDVQQGEwJCRTEZMBcG
# A1UEChMQR2xvYmFsU2lnbiBudi1zYTEvMC0GA1UEAxMmR2xvYmFsU2lnbiBHQ0Mg
# UjQ1IENvZGVTaWduaW5nIENBIDIwMjAwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAw
# ggIKAoICAQDWQk3540/GI/RsHYGmMPdIPc/Q5Y3lICKWB0Q1XQbPDx1wYOYmVPpT
# I2ACqF8CAveOyW49qXgFvY71TxkkmXzPERabH3tr0qN7aGV3q9ixLD/TcgYyXFus
# UGcsJU1WBjb8wWJMfX2GFpWaXVS6UNCwf6JEGenWbmw+E8KfEdRfNFtRaDFjCvhb
# 0N66WV8xr4loOEA+COhTZ05jtiGO792NhUFVnhy8N9yVoMRxpx8bpUluCiBZfomj
# WBWXACVp397CalBlTlP7a6GfGB6KDl9UXr3gW8/yDATS3gihECb3svN6LsKOlsE/
# zqXa9FkojDdloTGWC46kdncVSYRmgiXnQwp3UrGZUUL/obLdnNLcGNnBhqlAHUGX
# Yoa8qP+ix2MXBv1mejaUASCJeB+Q9HupUk5qT1QGKoCvnsdQQvplCuMB9LFurA6o
# 44EZqDjIngMohqR0p0eVfnJaKnsVahzEaeawvkAZmcvSfVVOIpwQ4KFbw7MueovE
# 3vFLH4woeTBFf2wTtj0s/y1KiirsKA8tytScmIpKbVo2LC/fusviQUoIdxiIrTVh
# lBLzpHLr7jaep1EnkTz3ohrM/Ifll+FRh2npIsyDwLcPRWwH4UNP1IxKzs9jsbWk
# EHr5DQwosGs0/iFoJ2/s+PomhFt1Qs2JJnlZnWurY3FikCUNCCDx/wIDAQABo4IB
# rjCCAaowDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMBIGA1Ud
# EwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFNqzjcAkkKNrd9MMoFndIWdkdgt4MB8G
# A1UdIwQYMBaAFB8Av0aACvx4ObeltEPZVlC7zpY7MIGTBggrBgEFBQcBAQSBhjCB
# gzA5BggrBgEFBQcwAYYtaHR0cDovL29jc3AuZ2xvYmFsc2lnbi5jb20vY29kZXNp
# Z25pbmdyb290cjQ1MEYGCCsGAQUFBzAChjpodHRwOi8vc2VjdXJlLmdsb2JhbHNp
# Z24uY29tL2NhY2VydC9jb2Rlc2lnbmluZ3Jvb3RyNDUuY3J0MEEGA1UdHwQ6MDgw
# NqA0oDKGMGh0dHA6Ly9jcmwuZ2xvYmFsc2lnbi5jb20vY29kZXNpZ25pbmdyb290
# cjQ1LmNybDBWBgNVHSAETzBNMEEGCSsGAQQBoDIBMjA0MDIGCCsGAQUFBwIBFiZo
# dHRwczovL3d3dy5nbG9iYWxzaWduLmNvbS9yZXBvc2l0b3J5LzAIBgZngQwBBAEw
# DQYJKoZIhvcNAQELBQADggIBAAiIcibGr/qsXwbAqoyQ2tCywKKX/24TMhZU/T70
# MBGfj5j5m1Ld8qIW7tl4laaafGG4BLX468v0YREz9mUltxFCi9hpbsf/lbSBQ6l+
# rr+C1k3MEaODcWoQXhkFp+dsf1b0qFzDTgmtWWu4+X6lLrj83g7CoPuwBNQTG8cn
# qbmqLTE7z0ZMnetM7LwunPGHo384aV9BQGf2U33qQe+OPfup1BE4Rt886/bNIr0T
# zfDh5uUzoL485HjVG8wg8jBzsCIc9oTWm1wAAuEoUkv/EktA6u6wGgYGnoTm5/Db
# hEb7c9krQrbJVzTHFsCm6yG5qg73/tvK67wXy7hn6+M+T9uplIZkVckJCsDZBHFK
# EUtaZMO8eHitTEcmZQeZ1c02YKEzU7P2eyrViUA8caWr+JlZ/eObkkvdBb0LDHgG
# K89T2L0SmlsnhoU/kb7geIBzVN+nHWcrarauTYmAJAhScFDzAf9Eri+a4OFJCOHh
# W9c40Z4Kip2UJ5vKo7nb4jZq42+5WGLgNng2AfrBp4l6JlOjXLvSsuuKy2MIL/4e
# 81Yp4jWb2P/ppb1tS1ksiSwvUru1KZDaQ0e8ct282b+Awdywq7RLHVg2N2Trm+GF
# F5opov3mCNKS/6D4fOHpp9Ewjl8mUCvHouKXd4rv2E0+JuuZQGDzPGcMtghyKTVT
# gTTcMIIHHDCCBQSgAwIBAgIMSUXsaWVAFuihgz78MA0GCSqGSIb3DQEBCwUAMFkx
# CzAJBgNVBAYTAkJFMRkwFwYDVQQKExBHbG9iYWxTaWduIG52LXNhMS8wLQYDVQQD
# EyZHbG9iYWxTaWduIEdDQyBSNDUgQ29kZVNpZ25pbmcgQ0EgMjAyMDAeFw0yMjAz
# MDIxMTQ4MzlaFw0yNDA2MTQxNDMwNDRaMIGJMQswCQYDVQQGEwJVUzERMA8GA1UE
# CBMITmV3IFlvcmsxETAPBgNVBAcTCE5ldyBZb3JrMRYwFAYDVQQKEw1TRU1QRVJJ
# UyBJTkMuMRYwFAYDVQQDEw1TRU1QRVJJUyBJTkMuMSQwIgYJKoZIhvcNAQkBFhVj
# b2Rlc2lnbkBzZW1wZXJpcy5jb20wggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIK
# AoICAQCvq2XdKWZ+xqIQ58x22njO9MSseih/t2TsnekKeiimUgSIKHG7TLypz2Vx
# 0+PumvxEm9JsbXXQKx9XobNZ69lRUlaZCq8Tia6DMvGEUAEvi3SvwdlJJr+AowSd
# 1evdJSHB1suaDfwX7K7PIaiIdjnl5wwc9ajPz20FiV7OUhIjYLT4T9FALz9zxJN+
# WlcUJUSmoXAmdjXHmvP6MQNO7whPI2auFvrq7OiA1N2CyXTFqwz1OkTlUqtZIsCD
# fuIWuPQZGPB3+nNN87e1eEfjO3vbwXTge649/7cXcRMuFKyuTWUE3AxX5QFWM8mt
# WujVuu8QefsPzwGJwaRP+oWHT5HmNS7s7wEn7+DggenjIo62n85ySnTliLofjl9f
# flxB10Hq3BYgVsZyNEYBp1hRKRpxQ58ZHMAkpA4liY3chI4klPWGgcIW1H2yqM3S
# lBraTVuT9skaZOua3u5Q8nlRyeXWoeBmQ8PlH0RFdeNnTRxfmY7HmXSCnG+D7TVH
# hzJPLGBHMC4FTjqro/wRKHiGLIkz6edqjz54TxKv9U39ULsIK0sQZaXb8STUp6eK
# K5lKt0wWedojQbi2qob3uZIgGg+4rAEKT4Wm8ZdDy7eN3aZTw/i5mjiEfwyJvgvz
# WT/yhJz6+ZDMQGIFJj9L2DaPnUSejLMC7K8kyZwJUXVGn1NRTQIDAQABo4IBsTCC
# Aa0wDgYDVR0PAQH/BAQDAgeAMIGbBggrBgEFBQcBAQSBjjCBizBKBggrBgEFBQcw
# AoY+aHR0cDovL3NlY3VyZS5nbG9iYWxzaWduLmNvbS9jYWNlcnQvZ3NnY2NyNDVj
# b2Rlc2lnbmNhMjAyMC5jcnQwPQYIKwYBBQUHMAGGMWh0dHA6Ly9vY3NwLmdsb2Jh
# bHNpZ24uY29tL2dzZ2NjcjQ1Y29kZXNpZ25jYTIwMjAwVgYDVR0gBE8wTTBBBgkr
# BgEEAaAyATIwNDAyBggrBgEFBQcCARYmaHR0cHM6Ly93d3cuZ2xvYmFsc2lnbi5j
# b20vcmVwb3NpdG9yeS8wCAYGZ4EMAQQBMAkGA1UdEwQCMAAwRQYDVR0fBD4wPDA6
# oDigNoY0aHR0cDovL2NybC5nbG9iYWxzaWduLmNvbS9nc2djY3I0NWNvZGVzaWdu
# Y2EyMDIwLmNybDATBgNVHSUEDDAKBggrBgEFBQcDAzAfBgNVHSMEGDAWgBTas43A
# JJCja3fTDKBZ3SFnZHYLeDAdBgNVHQ4EFgQUeXYlhOREXUlIPR2ZpkQmsxInWlkw
# DQYJKoZIhvcNAQELBQADggIBALVBZWrw/axZwZgcg+pO0wYi/phXxE1hUOxLaeRJ
# SxTEYZk7QasYoM2fR6mjUzV1da1sQKSK9OtUDbHcq7/JXKwcdKWIttwv9AI+RtdP
# yGvezvVfne1sBCn3qIxK3/5FRFdYftucE4It87UF4f25rS5FgGnLc5h+4MQgKjCv
# hI1F/aJjTBKhZnzlCwGUyAssXKYmWf0+PDxJixU5iTB8yK/VpPu4LFWWRi0pb1uS
# bJse4qIR25DBJ+zkbN6BIUz5Zx8LgmoGFVdEHFCWNfndjWtvd4DwJUtMdk9Wpdcz
# x2LrbzLUjKVN1PNdyuIWMKYxhnSt74U2VUFPoqEn6c/6lxKattXL2qD8FfV9APlc
# ZPrdV1Cnip9Mrbm5YdYebQaU4xH9LILkcSakBJSQvm0ev6f70ahHJLwiNCinftEn
# WEqgFeUfRrLJBYe8tCMIzkfH4BFIOrcd16ordOgC5QMn4bmqCvQE/df0Cngrt9Kf
# ynSBp+kx1IFZGeZ8iE9rQThcQwD75dC1/L6ADiev/TMPTm4H6+lKWL3umGvp/d25
# p+Y0lsfjTPvGWDZ65MF3sKRS0jcrAWrSm92CWfReyOXk2FHjJjtj2hAiNZ32igVh
# arGemFH0l5wHc/4KRUDcvtfCq7k8hdJn5s+EXlrzI01OFiabyDbR2Xe8JpmJfn9E
# e+3yMYIGNjCCBjICAQEwaTBZMQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFs
# U2lnbiBudi1zYTEvMC0GA1UEAxMmR2xvYmFsU2lnbiBHQ0MgUjQ1IENvZGVTaWdu
# aW5nIENBIDIwMjACDElF7GllQBbooYM+/DANBglghkgBZQMEAgEFAKB8MBAGCisG
# AQQBgjcCAQwxAjAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQB
# gjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEiBCCrOUmAWVzxK+hT
# aL+pngesLgS3WeXrxxYZbgjhL/jKmzANBgkqhkiG9w0BAQEFAASCAgClyH0aqa+F
# dO69GlXIPJ7j5S89nXVQofuXdBKjV0zuRHn/qECsTM47slh7BWSB4IoFBHNLYILG
# E3PblEQ8X/tZdV6fik8Wroi/nxP5ykGKXZxHxZuMi81+3A/D65Ov5T3tHo/X9s+O
# ZoBUguzxJ0hTkzgoS8UVrw89+BGdYdAqEu579oBM7O6BBL+g6pm3IgVGDoWyrb0B
# 8yuQbjcvoBkK5OjELdAaynIRpy+yR6oexMq0GoEs7+Er6hThQeRTTe4y2+NbbgVb
# V65MT168SICHElBaybxiZa7tWN8bmo+ysKWImnooHxr+0yoSBz3yQqXLUUrwVya2
# 4o3OJAjgMi4F/lNK+bV7Q7OCKOXzL5SgKx2/AKUekqTr4+bOjn1zDFvDYNZK8U+D
# 19csMlPNv/WxdBUzz6OuTUWL8j5StYSg/IM6iGQtSJuKL3cxCUubcBlsppqpDxBH
# AxA9Fr4biHGUSbJi9gDYor4U/BKvdu1faj3fNjXAJch/ccd/K/HvnF0yxD2hQhFq
# gaLDEH5PcxBGdzVES4rPEUvHcoWGzixg3oFUuZ9Hz2u3hLL74t5orDl8YdP/p+JV
# 57mNYut0WG1cXgJgcafYq3dahBnZ/yOCL3Gxk2mnlzMPerBdC5f4y3WsSM+kEPIE
# XqKMNVvzpLf7NLmMMh5iMB3LE1RkKYLfEqGCAyAwggMcBgkqhkiG9w0BCQYxggMN
# MIIDCQIBATB3MGMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5j
# LjE7MDkGA1UEAxMyRGlnaUNlcnQgVHJ1c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBU
# aW1lU3RhbXBpbmcgQ0ECEAVEr/OUnQg5pr/bP1/lYRYwDQYJYIZIAWUDBAIBBQCg
# aTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0yMzEy
# MDUxMTM4MjhaMC8GCSqGSIb3DQEJBDEiBCB/a2O9xJp8RivhF3MJYLk56QJqbfN+
# cXvRyvrztj7NvDANBgkqhkiG9w0BAQEFAASCAgB4z+ccGun0K/TybzZ46W47mwIf
# osNU3QeXvwr/XkEXafPh0coWhmH2pWjHBOpFdDfg5iiieGU9AUhYBzIax5Mx8jII
# jRGom3PkVBuiSI5tP7dzDAarQGr6TdcGH2aaZcAck1MpFUNkrVRcj54qINpqnvvR
# IHjd3vTyDItvifLlNmZcDUMM/UDktGzGaXeEo3yQPEfU0uBkWfenOnGUkbROiDPl
# NeZTHoXUcuiHk0y3XK3eyu9wWFu5VixAHwnxXXEjS+1FgJw7oj9GXanobGdIF4rO
# tFPy3urYGrZEU64EYbDhZySc2QEOUac/GDBl2Tdz+lDRPTp/yxYf86ndZsK2anFa
# dt5gFHnk7MssmYqLXnDcN63Wno19dzYQQeQxGpMx4NL8gbaM3nAHFn01gUJnNi0P
# rsOMYzMwHmzTIHNRlWRHNbeTCoIj415Uw80yw3qesrGo+mXEZ9Vlszsapva765ZQ
# 3abaHRvFXRwb+M6bBw1NRY+MinntQDOY3ykEYLrDM0WzNdegOrAAG/Vc7jioh7Gw
# mTL8bKVXCYMXNzADGFDjJ85Jfb1e9A7LI3kyvjqQDdRqtCtTO1xs3CdacYYrydqa
# OrWUzNSNsO/22X2c/0xZzRkYj8RdhXS533s+SqL3/yh+dyMyZ/DwUv52DFvpmM18
# WrVrwcXx2n4BiPdKDQ==
# SIG # End signature block
