﻿[CmdletBinding()]
Param(
    [Parameter(Mandatory=$True,ParameterSetName='Execution')][String]$ForestName,
    [Parameter(Mandatory=$True,ParameterSetName='Execution')][String[]]$DomainNames,
    [Parameter(Mandatory=$False,ParameterSetName='Execution')]$StartAttackWindow,
    [Parameter(Mandatory=$False,ParameterSetName='Execution')]$EndAttackWindow,
    [Parameter(ParameterSetName='Metadata',Mandatory)][switch]$Metadata
)

$Global:self = @{
    ID = 154
    UUID = '0d9236c4-98a1-4763-913b-783fdfe1de4c'
    Version = '1.48.0'
    CategoryID = 3
    ShortName = 'SI000154'
    Name = 'SMB Signing is not required on Domain Controllers'
    ScriptName = 'SmbSigningIsNotRequired'
    Description = 'This indicator looks for domain controllers where SMB signing is not required.'
    Weight = 8
    Severity = 'Critical'
    Impact = 8
    LikelihoodOfCompromise = 'Unsigned network traffic is susceptible to attacks abusing the NTLM challenge-response protocol. A common example of such attacks is SMB Relay, where an attacker is positioned between the client and the server in order to capture data packets transmitted between the two, thus gaining unauthorized access to the server or other servers on the network.'
    ResultMessage = 'Found {0} DCs that do not require SMB Signing.'
    Remediation = 'The following Group Policies need to be enabled in order to enforce SMB Signing on DCs:
		<br>1. Computer Configuration > Policies > Windows Settings > Security Settings > Local Policies > Security Options > Microsoft network server: Digitally sign communications (always): This policy controls whether the server providing SMB required signing. It determines if SMB signing will have to be negotiated prior to further communication.</br>
		<br>2. Computer Configuration > Policies > Windows Settings > Security Settings > Local Policies > Security Options > Microsoft network server: Digitally sign communications (if client agrees): This policy determines if SMB server will negotiate SMB signing with clients that request it.</br>
    <br>Read more <a href="https://techcommunity.microsoft.com/t5/storage-at-microsoft/configure-smb-signing-with-confidence/ba-p/2418102" target="_blank">here</a></br>'
    Types = @('IoE')
    DataSources = @('AD.LDAP')
    Targets = @('AD')
    Permissions = @()
    SecurityFrameworks = @(
        @{ Name = 'MITRE ATT&CK'; Tags = @('Privilege Escalation', 'Credential Access') }
    )
    Products = @(
        @{ Name = 'HYD'; MinVersion = '1.0'; MaxVersion = '3.0'; Licenses = @('Cloud') },
        @{ Name = 'DSP'; MinVersion = '3.5'; MaxVersion = '10'; Licenses = @('DSP-I') },
        @{ Name = 'PK'; MinVersion = '1.4'; MaxVersion = '10'; Licenses = @('Community', 'Post-Breach', 'BPIR') }
    )
    Selected = 1
}
if($Metadata){ return $self | ConvertTo-Json -Depth 8 -Compress }

Import-Module -Name Semperis-Lib

$code = @"
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace SmbSigning
{
    public class SmbSigningChecking
    {
        public struct SMB_ERROR
        {
            public byte ErrorClass;
            public byte Reserved;
            public byte ErrorCode1;
            public byte ErrorCode2;

            public SMB_ERROR(byte errorClass)
            {
                ErrorClass = errorClass;
                Reserved = 0x00;
                ErrorCode1 = 0x00;
                ErrorCode2 = 0x00;
            }

        }

        [Flags]
        public enum Flags
        {
            SMB_FLAGS_LOCK_AND_READ_OK = 0x01,
            SMB_FLAGS_BUF_AVAIL = 0x02,
            Reserved = 0x04,
            SMB_FLAGS_CASE_INSENSITIVE = 0x08,
            SMB_FLAGS_CANONICALIZED_PATHS = 0x10,
            SMB_FLAGS_OPLOCK = 0x20,
            SMB_FLAGS_OPBATCH = 0x40,
            SMB_FLAGS_REPLY = 0x80,

        }

        [Flags]
        public enum Flags2
        {
            SMB_FLAGS2_LONG_NAMES = 0x0001,
            SMB_FLAGS2_EAS = 0x0002,
            SMB_FLAGS2_SMB_SECURITY_SIGNATURE = 0x0004,
            SMB_FLAGS2_IS_LONG_NAME = 0x0040,
            SMB_FLAGS2_DFS = 0x1000,
            SMB_FLAGS2_PAGING_IO = 0x2000,
            SMB_FLAGS2_NT_STATUS = 0x4000,
            SMB_FLAGS2_UNICODE = 0x8000,
        }


        public struct SMB1_HEADER
        {

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)] public byte[] Protocol; // size 4 bytes // The value is "0xFF+'SMB'".
            public byte Command;
            public SMB_ERROR Status;
            public byte Flags;
            public ushort Flags2;  //Here there are 14 bytes of data which is used differently among different dialects.
            public ushort PIDHigh;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)] public byte[] SecurityFeatures; // size 8 bytes a signature in SMB_COM_NEGOTITATE it's 0's
            public ushort Reserved;
            public ushort TID; // A tree identifier
            public ushort PIDLow;
            public ushort UID;
            public ushort MID;
            public SMB1_HEADER(byte command, byte flags, ushort flags2)
            {
                Protocol = new byte[] { 0xFF, 0x53, 0x4D, 0x42 };
                Command = command;
                Status = new SMB_ERROR(0x00);
                Flags = flags;
                Flags2 = flags2;
                PIDHigh = 0x00;
                SecurityFeatures = new byte[] { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
                Reserved = 0x0000;
                Random random = new Random();
                PIDLow = (ushort)random.Next(1 << 16);
                TID = 0x0000;
                UID = 0x0000;
                MID = 0x0001;

            }
        }

        // Reference https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-cifs/25c8c3c9-58fc-4bb8-aa8f-0272dede84c5

        public struct SMB_PARAMETERS
        {
            public byte WordCount;

            public SMB_PARAMETERS(byte wordCount)
            {
                WordCount = wordCount;
            }
        }

        public struct SMB_DATA
        {
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2)] public byte[] BytesCount;

            public SMB_DATA(byte[] bytesCount)
            {
                BytesCount = bytesCount;
            }
        }
        public struct Dialect
        {
            public byte BufferFormat;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 10)] public byte[] Name;

            public Dialect(byte[] name)
            {
                BufferFormat = 0x02;
                Name = name;
            }
        }

        public struct NetBIOS
        {
            public byte MessageType;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)] public byte[] Size; // size 3 bytes

            public NetBIOS(byte[] size)
            {
                MessageType = 0x00;
                Size = size;
            }
        }

        [Flags]
        public enum SecurityMode : byte
        {
            NEGOTIATE_USER_SECURITY = 0x01,
            NEGOTIATE_ENCRYPT_PASSWORDS = 0x02,
            NEGOTIATE_SECURITY_SIGNATURES_ENABLED = 0x04,
            NEGOTIATE_SECURITY_SIGNATURES_REQUIRED = 0x08,
        }


        public struct SMB_NEGOTIATE_RESPONSE
        {
            public byte WordCount;
            public ushort SelectedIndex;
            public SecurityMode SecurityMode;
            public ushort MaxMpxCount;
            public ushort MaxVCs;
            public uint MaxBufferSize; // size 4
            public uint MinBufferSize; // size 4
            public uint SessionKey; // size 4
            public uint Capabilities; // size 4
            public string SystemTime; // size 8
            public ushort ServerTimeZone;
            public byte ChallengeLength;
            public ushort ByteCount;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)] public byte[] ServerGuid;

            public SMB_NEGOTIATE_RESPONSE(ushort selectedIndex)
            {
                WordCount = 0;
                SelectedIndex = selectedIndex;
                SecurityMode = 0;
                MaxMpxCount = 0;
                MaxVCs = 0;
                MaxBufferSize = 0;
                MinBufferSize = 0;
                SessionKey = 0;
                Capabilities = 0;
                SystemTime = "00000000";
                ServerTimeZone = 0;
                ChallengeLength = 0;
                ByteCount = 0;
                ServerGuid = new byte[] { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36 };
            }

        }

        public struct SMB2_HEADER
        {

            // Always 64 bytes

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)] public byte[] ProtocolID;
            public ushort HeaderLength;
            public ushort CreditCharge;
            public ushort ChanngelSequence;
            public ushort Reserved;
            public ushort Command;
            public ushort CreditsRequested;
            public uint Flags;
            public uint ChainOffest;
            public ulong MessageID;
            public uint PID;
            public uint TreeID;
            public ulong SessionID;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)] public byte[] Signature;

            public SMB2_HEADER(ushort command, uint flags)
            {
                ProtocolID = new byte[] { 0xFE, 0x53, 0x4D, 0x42 };
                HeaderLength = 64;
                CreditCharge = 0;
                ChanngelSequence = 0;
                Reserved = 0;
                Command = command;
                CreditsRequested = 0;
                Flags = flags;
                ChainOffest = 0;
                MessageID = 0;
                PID = 0;
                TreeID = 0;
                SessionID = 0;
                Signature = new byte[] { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36 };
            }

        }

        public struct SMB2_NEGOTIATE_REQUEST
        {
            public ushort StructureSize;
            public ushort DialectCount;
            public ushort SecurityMode;
            public ushort Resereved;
            public uint Capabilities;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)] public byte[] ClientGUID;
            public uint NegotiateContextOffset;
            public ushort NegotiateContextCount;
            public ushort Reserved2;

            public SMB2_NEGOTIATE_REQUEST(ushort dialectCount)
            {
                StructureSize = 36;
                DialectCount = dialectCount;
                SecurityMode = 0;
                Resereved = 0;
                Capabilities = 0;
                ClientGUID = new byte[] { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36 };
                NegotiateContextOffset = 0;
                NegotiateContextCount = 0;
                Reserved2 = 0;

            }

        }

        public struct SMB2_Dialect
        {
            public ushort Dialect;

            public SMB2_Dialect(ushort dialect)
            {
                Dialect = dialect;
            }
        }

        [Flags]
        public enum SecurityModeSMB2 : ushort
        {
            SMB2_NEGOTIATE_SIGNING_ENABLED = 0x0001,
            SMB2_NEGOTIATE_SIGNING_REQUIRED = 0x0002,
        }

        public struct SMB2_NEGOTIATE_RESPONSE
        {
            public ushort StructureSize;
            public SecurityModeSMB2 SecurityMode;
            public ushort DialectRevision;
            public ushort NegotiateContextCount;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)] public byte[] ServerGuid;
            public uint Capabilities;
            public uint MaxTransactSize;
            public uint MaxReadSize;
            public uint MaxWriteSize;
            public ulong SystemTime;
            public ulong ServerStartTime;
            public ushort SecurityBufferOffset;
            public ushort SecurityBufferLength;
            public uint NegotiateContextOffset;

            public SMB2_NEGOTIATE_RESPONSE(ushort structureSize)
            {
                StructureSize = 0;
                SecurityMode = 0;
                DialectRevision = 0;
                ServerGuid = new byte[] { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36 };
                Capabilities = 0;
                MaxTransactSize = 0;
                MaxReadSize = 0;
                MaxWriteSize = 0;
                SystemTime = 0;
                ServerStartTime = 0;
                SecurityBufferOffset = 0;
                SecurityBufferLength = 0;
                NegotiateContextCount = 0;
                NegotiateContextOffset = 0;
            }

        }
        public static byte[] getBytes<T>(T header)
        {

            int size = Marshal.SizeOf(header);
            byte[] arr = new byte[size];

            IntPtr ptr = Marshal.AllocHGlobal(size);
            Marshal.StructureToPtr(header, ptr, true);
            Marshal.Copy(ptr, arr, 0, size);
            Marshal.FreeHGlobal(ptr);
            return arr;
        }
        public static byte[] Combine(byte[] first, byte[] second)
        {
            byte[] ret = new byte[first.Length + second.Length];
            Buffer.BlockCopy(first, 0, ret, 0, first.Length);
            Buffer.BlockCopy(second, 0, ret, first.Length, second.Length);
            return ret;
        }

        public static string BytesCountFromBytes(byte[] bytes_array)
        {
            int decimal_size = bytes_array.Length;
            return String.Format("{0:X2}", decimal_size);
        }

        public static byte[] HexStringToByteArray(string hex)
        {
            return Enumerable.Range(0, hex.Length)
                             .Where(x => x % 2 == 0)
                             .Select(x => Convert.ToByte(hex.Substring(x, 2), 16))
                             .ToArray();
        }
        public static byte[] BuildSMBNegotitateRequest()
        {
            byte[] trailing = { 0x00 };
            byte[] trailing2 = { 0x00, 0x00 };

            SMB1_HEADER SMBh = new SMB1_HEADER(0x72, 0x18, 0x6845);

            byte[] dialectNTBytes = { 0x4e, 0x54, 0x20, 0x4c, 0x4d, 0x20, 0x30, 0x2e, 0x31, 0x32 };
            Dialect dialect = new Dialect(dialectNTBytes);
            byte[] dialectSize = Combine(HexStringToByteArray(BytesCountFromBytes(getBytes(dialect))), trailing);

            SMB_DATA dialects = new SMB_DATA(dialectSize);
            SMB_PARAMETERS smbParameters = new SMB_PARAMETERS(0x00);
            byte[] negotiateProtocolRequest = Combine(getBytes(smbParameters), Combine(getBytes(dialects), Combine(getBytes(dialect), trailing)));
            byte[] smbRequest = Combine(getBytes(SMBh), negotiateProtocolRequest);
            byte[] sizeOfRequest = Combine(trailing2, HexStringToByteArray(BytesCountFromBytes(smbRequest)));

            NetBIOS netBiosh = new NetBIOS(sizeOfRequest);

            byte[] bytesToSend = Combine(getBytes(netBiosh), smbRequest);

            return bytesToSend;

        }
        public static byte[] BuildSMB2NegotitateRequest()
        {
            byte[] trailing2 = { 0x00, 0x00 };

            SMB2_HEADER SMBh = new SMB2_HEADER(0, 0);

            SMB2_Dialect dialect1 = new SMB2_Dialect(0x0202);
            SMB2_Dialect dialect2 = new SMB2_Dialect(0x0210);
            SMB2_Dialect dialect3 = new SMB2_Dialect(0x0300);
            SMB2_Dialect dialect4 = new SMB2_Dialect(0x0302);


            byte[] dialectesBytes = Combine(getBytes(dialect1), Combine(getBytes(dialect2), Combine(getBytes(dialect3), getBytes(dialect4))));
            SMB2_NEGOTIATE_REQUEST negotiateRequest = new SMB2_NEGOTIATE_REQUEST(4);
            byte[] negotiateRequestBytes = Combine(getBytes(negotiateRequest), dialectesBytes);
            byte[] smbRequest = Combine(getBytes(SMBh), negotiateRequestBytes);


            byte[] sizeOfRequest = Combine(trailing2, HexStringToByteArray(BytesCountFromBytes(smbRequest)));

            NetBIOS netBiosh = new NetBIOS(sizeOfRequest);


            byte[] bytesToSend = Combine(getBytes(netBiosh), smbRequest);


            return bytesToSend;

        }
        public static byte ReadByte(byte[] buffer, int offset)
        {
            return buffer[offset];
        }

        public static byte[] ReadBytes(byte[] buffer, int offset, int length)
        {
            byte[] result = new byte[length];
            Array.Copy(buffer, offset, result, 0, length);
            return result;
        }

        public static bool GetIfSecurityModeSMB1(byte[] bytesReceived)
        {

            SMB_NEGOTIATE_RESPONSE smbResponse = new SMB_NEGOTIATE_RESPONSE(0);

            smbResponse.SecurityMode = (SecurityMode)ReadByte(bytesReceived, 39);

            if ((smbResponse.SecurityMode & SecurityMode.NEGOTIATE_SECURITY_SIGNATURES_REQUIRED) == SecurityMode.NEGOTIATE_SECURITY_SIGNATURES_REQUIRED)
            {
                return true;
            }

            return false;
        }

        public static bool GetIfSecurityModeSMB2(byte[] bytesReceived)
        {
            SMB2_NEGOTIATE_RESPONSE smbResponse = new SMB2_NEGOTIATE_RESPONSE();
            smbResponse.SecurityMode = (SecurityModeSMB2)ReadByte(bytesReceived, 70);

            if ((smbResponse.SecurityMode & SecurityModeSMB2.SMB2_NEGOTIATE_SIGNING_REQUIRED) == SecurityModeSMB2.SMB2_NEGOTIATE_SIGNING_REQUIRED)
            {
                return true;
            }
            return false;
        }
        public static Socket ConnectSocketNew(string server, int port)
        {
            Socket s = null;
            IPHostEntry hostEntry = null;

            // Get host related information.
            hostEntry = Dns.GetHostEntry(server);

            foreach (IPAddress address in hostEntry.AddressList)
            {
                IPEndPoint ipe = new IPEndPoint(address, port);
                Socket tempSocket =
                    new Socket(ipe.AddressFamily, SocketType.Stream, ProtocolType.Tcp);

                tempSocket.Connect(ipe);

                if (tempSocket.Connected)
                {
                    s = tempSocket;
                    break;
                }
                else
                {
                    continue;
                }
            }
            return s;
        }

        public static string ByteArrayToString(byte[] ba)
        {
            StringBuilder hex = new StringBuilder(ba.Length * 2);
            foreach (byte b in ba)
                hex.AppendFormat("{0:x2}", b);
            return hex.ToString();
        }

        public enum SMBVersionType
        {
            SMB1_PROTOCOL,
            SMB2_PROTOCOL,
            VERSION_VALIDATION_FAILED

        }
        public static SMBVersionType CheckAndValidateResponseVersion(byte [] bytesReceived)
        {
            byte [] SMB1_SIGNATURE = new byte[] { 0xFF, 0x53, 0x4D, 0x42 };
            byte [] SMB2_SIGNATURE = new byte[] { 0xFE, 0x53, 0x4D, 0x42 };

            byte [] ProtocolID = ReadBytes(bytesReceived, 4, 4);

            if (SMB1_SIGNATURE.SequenceEqual(ProtocolID))
                return SMBVersionType.SMB1_PROTOCOL;

            if (SMB2_SIGNATURE.SequenceEqual(ProtocolID))
                return SMBVersionType.SMB2_PROTOCOL;

            return SMBVersionType.VERSION_VALIDATION_FAILED;
        }
        public static string CheckInternal(string server, int port)
        {
            SMBVersionType version = SMBVersionType.VERSION_VALIDATION_FAILED;
            int bytes = 0;
            try
            {

                Byte[] bytesSentSMB1 = BuildSMBNegotitateRequest();
                Byte[] bytesReceivedSMB1 = new Byte[512];
                using (Socket socket = ConnectSocketNew(server, port))
                {
                    if (socket == null)
                    {
                        return "Unreachable";
                    }
                    else
                    {
                        socket.ReceiveTimeout = 3000;
                    }


                    socket.Send(bytesSentSMB1, bytesSentSMB1.Length, 0);

                    bytes = socket.Receive(bytesReceivedSMB1, bytesReceivedSMB1.Length, 0);

                    version = CheckAndValidateResponseVersion(bytesReceivedSMB1);
                    if (version == SMBVersionType.SMB1_PROTOCOL)
                    {
                        bool signinigRequired = GetIfSecurityModeSMB1(bytesReceivedSMB1);
                        return signinigRequired.ToString();
                    }
                    socket.Close();
                }
            }
            catch
            {
                if (bytes == 0)
                {
                    version = SMBVersionType.SMB2_PROTOCOL;
                }


            }
            if (version == SMBVersionType.SMB2_PROTOCOL)
            {
                try
                {
                    Byte[] bytesSentSMB2 = BuildSMB2NegotitateRequest();
                    Byte[] bytesReceivedSMB2 = new Byte[512];
                    using (Socket socket = ConnectSocketNew(server, port))
                    {
                        if (socket == null)
                        {
                            return "Unreachable";
                        }
                        else
                        {
                            socket.ReceiveTimeout = 3000;
                        }

                        socket.Send(bytesSentSMB2, bytesSentSMB2.Length, 0);

                        bytes = 0;

                        bytes = socket.Receive(bytesReceivedSMB2, bytesReceivedSMB2.Length, 0);

                        version = CheckAndValidateResponseVersion(bytesReceivedSMB2);
                        if (version == SMBVersionType.SMB2_PROTOCOL)
                        {
                            bool signinigRequired = GetIfSecurityModeSMB2(bytesReceivedSMB2);
                            return signinigRequired.ToString();
                        }
                        socket.Close();
                    }
                }
                catch
                {
                    return "Unreachable";
                }
            }
            return "Unrechable";

        }

        public static string[] ResultFromCheckInternal(string host, int port, string DN)
        {
            string[] resultsArray = new string[3];
            resultsArray[0] = host;
            resultsArray[1] = DN;
            resultsArray[2] = CheckInternal(host, port);

            return resultsArray;

        }

        public static Task<string[]> Check(string host, string DN)
        {
            int port = 445;

            return Task.Run(() => ResultFromCheckInternal(host, port, DN));
        }

    }
}

"@

Add-Type -TypeDefinition $code -Language CSharp

# Script Global Variables
$outputObjects = [System.Collections.ArrayList]@()
$domainControllersFailedCount = 0

try {
    if ($PSBoundParameters['ForestName'] -and $PSBoundParameters['DomainNames']) {
        $ForestName = $ForestName.ToLower()
        $DomainNames = ConvertTo-Lowercase -DomainNames $DomainNames
    }
    $outputObjects = [System.Collections.ArrayList]@()
    $res = Initialize-res


    $dcs = @()
    $unavailableDomains = [System.Collections.ArrayList]@()
    foreach ($domain in $DomainNames)
    {
        if (-not (Confirm-DomainAvailability $domain)) {
            [void]$unavailableDomains.Add($domain)
            continue
        }
        $DN = Get-DN $domain
        $dcs += Search-AD -dnsDomain $domain -attributes "dnshostname" -baseDN $DN -scope "Subtree" -filter "(&(objectCategory=computer)(primaryGroupID=516))"

    }

    $tasks = @()

    foreach ($dc in $dcs)
    {
        $dcHostName = $dc.Attributes.dnshostname[$GET_STRING]
        $tasks += [SmbSigning.SmbSigningChecking]::Check($dcHostName, $dc.DistinguishedName)
    }
    [System.Threading.Tasks.Task]::WhenAll($tasks).Wait()
    $unreachables = 0

    foreach ($result in $tasks)
    {
        $resultsArray = $result.Result
        $isSMBSigningRequired = $resultsArray[2];
        $dcDN = $resultsArray[1]
        $hostName = $resultsArray[0]

        if ($isSMBSigningRequired -eq "False"){
            $domainControllersFailedCount++
            $thisOutput = [PSCustomObject][Ordered] @{
                DCName = $hostName
                DistinguishedName=$dcDN
                State="SMB Signing is not required"
            }
            [void]$outputObjects.Add($thisOutput)
        }
        elseif($isSMBSigningRequired -eq "Unreachable"){
            $domainControllersFailedCount++
            $unreachables++
            $thisOutput = [PSCustomObject][Ordered] @{
                DCName = $hostName
                DistinguishedName=$dcDN
                State="Unreachable"
            }
            [void]$outputObjects.Add($thisOutput)
        }
    }

    if ($outputObjects){
        $configArgs = @{
            ScriptName = $self.ScriptName
            Path = $MyInvocation.MyCommand.ScriptBlock.File
            Fields = $outputObjects[0]
        }
        $config = Resolve-Configuration @configArgs
        $outputObjects | Set-IgnoredFlag -Configuration $config
        $scoreOutput = $outputObjects | Get-Score -Impact $self.Impact
        if($scoreOutput.Score -lt 100)
        {

            if ($unreachables -eq 0)
            {
                $res.ResultObjects = $outputObjects
                $res.ResultMessage = "Found $($domainControllersFailedCount) DCs that do not require SMB Signing."
                $res.Remediation = $self.Remediation
                $res.Status = [Semperis.PSSecurityIndicatorResult.ScriptStatus]"Failed"
                $res.Score = $scoreOutput.Score
            }
            else
            {
                $res.ResultObjects = $outputObjects
                $res.ResultMessage = "Found $($domainControllersFailedCount-$unreachables) DCs that do not require SMB Signing, And $($unreachables) DCs are Unreachable."
                $res.Remediation = $self.Remediation
                $res.Status = [Semperis.PSSecurityIndicatorResult.ScriptStatus]"Failed"
                $res.Score = $scoreOutput.Score
            }
        }

        if ($scoreOutput.Ignoredcount -gt 0)
        {
            $res.ResultMessage += " ($($scoreOutput.Ignoredcount) Objects ignored)."
            $res.ResultObjects = $outputObjects
        }

        $badObjectCount = $outputObjects.Count - $scoreOutput.Ignoredcount
        if ($badObjectCount -eq 0)
        {
            $res.ResultObjects = $outputObjects
            $res.Remediation = $self.Remediation
            $res.Status = [Semperis.PSSecurityIndicatorResult.ScriptStatus]"Pass"
            $res.Score = $scoreOutput.Score
        }
        elseif ($unreachables -ge $badObjectCount){
            $res.ResultObjects = $outputObjects
            $res.ResultMessage = "There are {0} DCs that are unreachable." -f $unreachables
            $res.Remediation = $self.Remediation
            $res.Status = [Semperis.PSSecurityIndicatorResult.ScriptStatus]"Error"
        }
    }

    # deal with unavailabile domains
    if ($unavailableDomains.Count -gt 0) {
        $res.Status = [Semperis.PSSecurityIndicatorResult.ScriptStatus]"Error"
        $res.ResultMessage += " Failed to run because the following domains were unavailable: $($unavailableDomains -join ', ')"
    }
}
catch {
    return ConvertTo-ErrorResult $_
}
return $res

# SIG # Begin signature block
# MIItmQYJKoZIhvcNAQcCoIItijCCLYYCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCCyh9cHvg/cBxpY
# jG7hu1VIpIF8nxNLauY3tWzso4xdBqCCJrkwggWNMIIEdaADAgECAhAOmxiO+dAt
# 5+/bUOIIQBhaMA0GCSqGSIb3DQEBDAUAMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNV
# BAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0yMjA4MDEwMDAwMDBa
# Fw0zMTExMDkyMzU5NTlaMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lD
# ZXJ0IFRydXN0ZWQgUm9vdCBHNDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAL/mkHNo3rvkXUo8MCIwaTPswqclLskhPfKK2FnC4SmnPVirdprNrnsbhA3E
# MB/zG6Q4FutWxpdtHauyefLKEdLkX9YFPFIPUh/GnhWlfr6fqVcWWVVyr2iTcMKy
# unWZanMylNEQRBAu34LzB4TmdDttceItDBvuINXJIB1jKS3O7F5OyJP4IWGbNOsF
# xl7sWxq868nPzaw0QF+xembud8hIqGZXV59UWI4MK7dPpzDZVu7Ke13jrclPXuU1
# 5zHL2pNe3I6PgNq2kZhAkHnDeMe2scS1ahg4AxCN2NQ3pC4FfYj1gj4QkXCrVYJB
# MtfbBHMqbpEBfCFM1LyuGwN1XXhm2ToxRJozQL8I11pJpMLmqaBn3aQnvKFPObUR
# WBf3JFxGj2T3wWmIdph2PVldQnaHiZdpekjw4KISG2aadMreSx7nDmOu5tTvkpI6
# nj3cAORFJYm2mkQZK37AlLTSYW3rM9nF30sEAMx9HJXDj/chsrIRt7t/8tWMcCxB
# YKqxYxhElRp2Yn72gLD76GSmM9GJB+G9t+ZDpBi4pncB4Q+UDCEdslQpJYls5Q5S
# UUd0viastkF13nqsX40/ybzTQRESW+UQUOsxxcpyFiIJ33xMdT9j7CFfxCBRa2+x
# q4aLT8LWRV+dIPyhHsXAj6KxfgommfXkaS+YHS312amyHeUbAgMBAAGjggE6MIIB
# NjAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBTs1+OC0nFdZEzfLmc/57qYrhwP
# TzAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzAOBgNVHQ8BAf8EBAMC
# AYYweQYIKwYBBQUHAQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdp
# Y2VydC5jb20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNv
# bS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcnQwRQYDVR0fBD4wPDA6oDigNoY0
# aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENB
# LmNybDARBgNVHSAECjAIMAYGBFUdIAAwDQYJKoZIhvcNAQEMBQADggEBAHCgv0Nc
# Vec4X6CjdBs9thbX979XB72arKGHLOyFXqkauyL4hxppVCLtpIh3bb0aFPQTSnov
# Lbc47/T/gLn4offyct4kvFIDyE7QKt76LVbP+fT3rDB6mouyXtTP0UNEm0Mh65Zy
# oUi0mcudT6cGAxN3J0TU53/oWajwvy8LpunyNDzs9wPHh6jSTEAZNUZqaVSwuKFW
# juyk1T3osdz9HNj0d1pcVIxv76FQPfx2CWiEn2/K2yCNNWAcAgPLILCsWKAOQGPF
# mCLBsln1VWvPJ6tsds5vIy30fnFqI2si/xK4VC0nftg62fC2h5b9W9FcrBjDTZ9z
# twGpn1eqXijiuZQwggWiMIIEiqADAgECAhB4AxhCRXCKQc9vAbjutKlUMA0GCSqG
# SIb3DQEBDAUAMEwxIDAeBgNVBAsTF0dsb2JhbFNpZ24gUm9vdCBDQSAtIFIzMRMw
# EQYDVQQKEwpHbG9iYWxTaWduMRMwEQYDVQQDEwpHbG9iYWxTaWduMB4XDTIwMDcy
# ODAwMDAwMFoXDTI5MDMxODAwMDAwMFowUzELMAkGA1UEBhMCQkUxGTAXBgNVBAoT
# EEdsb2JhbFNpZ24gbnYtc2ExKTAnBgNVBAMTIEdsb2JhbFNpZ24gQ29kZSBTaWdu
# aW5nIFJvb3QgUjQ1MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAti3F
# MN166KuQPQNysDpLmRZhsuX/pWcdNxzlfuyTg6qE9aNDm5hFirhjV12bAIgEJen4
# aJJLgthLyUoD86h/ao+KYSe9oUTQ/fU/IsKjT5GNswWyKIKRXftZiAULlwbCmPgs
# pzMk7lA6QczwoLB7HU3SqFg4lunf+RuRu4sQLNLHQx2iCXShgK975jMKDFlrjrz0
# q1qXe3+uVfuE8ID+hEzX4rq9xHWhb71hEHREspgH4nSr/2jcbCY+6R/l4ASHrTDT
# DI0DfFW4FnBcJHggJetnZ4iruk40mGtwEd44ytS+ocCc4d8eAgHYO+FnQ4S2z/x0
# ty+Eo7+6CTc9Z2yxRVwZYatBg/WsHet3DUZHc86/vZWV7Z0riBD++ljop1fhs8+o
# WukHJZsSxJ6Acj2T3IyU3ztE5iaA/NLDA/CMDNJF1i7nj5ie5gTuQm5nfkIWcWLn
# BPlgxmShtpyBIU4rxm1olIbGmXRzZzF6kfLUjHlufKa7fkZvTcWFEivPmiJECKiF
# N84HYVcGFxIkwMQxc6GYNVdHfhA6RdktpFGQmKmgBzfEZRqqHGsWd/enl+w/GTCZ
# bzH76kCy59LE+snQ8FB2dFn6jW0XMr746X4D9OeHdZrUSpEshQMTAitCgPKJajbP
# yEygzp74y42tFqfT3tWbGKfGkjrxgmPxLg4kZN8CAwEAAaOCAXcwggFzMA4GA1Ud
# DwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzAPBgNVHRMBAf8EBTADAQH/
# MB0GA1UdDgQWBBQfAL9GgAr8eDm3pbRD2VZQu86WOzAfBgNVHSMEGDAWgBSP8Et/
# qC5FJK5NUPpjmove4t0bvDB6BggrBgEFBQcBAQRuMGwwLQYIKwYBBQUHMAGGIWh0
# dHA6Ly9vY3NwLmdsb2JhbHNpZ24uY29tL3Jvb3RyMzA7BggrBgEFBQcwAoYvaHR0
# cDovL3NlY3VyZS5nbG9iYWxzaWduLmNvbS9jYWNlcnQvcm9vdC1yMy5jcnQwNgYD
# VR0fBC8wLTAroCmgJ4YlaHR0cDovL2NybC5nbG9iYWxzaWduLmNvbS9yb290LXIz
# LmNybDBHBgNVHSAEQDA+MDwGBFUdIAAwNDAyBggrBgEFBQcCARYmaHR0cHM6Ly93
# d3cuZ2xvYmFsc2lnbi5jb20vcmVwb3NpdG9yeS8wDQYJKoZIhvcNAQEMBQADggEB
# AKz3zBWLMHmoHQsoiBkJ1xx//oa9e1ozbg1nDnti2eEYXLC9E10dI645UHY3qkT9
# XwEjWYZWTMytvGQTFDCkIKjgP+icctx+89gMI7qoLao89uyfhzEHZfU5p1GCdeHy
# L5f20eFlloNk/qEdUfu1JJv10ndpvIUsXPpYd9Gup7EL4tZ3u6m0NEqpbz308w2V
# Xeb5ekWwJRcxLtv3D2jmgx+p9+XUnZiM02FLL8Mofnrekw60faAKbZLEtGY/fadY
# 7qz37MMIAas4/AocqcWXsojICQIZ9lyaGvFNbDDUswarAGBIDXirzxetkpNiIHd1
# bL3IMrTcTevZ38GQlim9wX8wggauMIIElqADAgECAhAHNje3JFR82Ees/ShmKl5b
# MA0GCSqGSIb3DQEBCwUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lD
# ZXJ0IFRydXN0ZWQgUm9vdCBHNDAeFw0yMjAzMjMwMDAwMDBaFw0zNzAzMjIyMzU5
# NTlaMGMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjE7MDkG
# A1UEAxMyRGlnaUNlcnQgVHJ1c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBUaW1lU3Rh
# bXBpbmcgQ0EwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDGhjUGSbPB
# PXJJUVXHJQPE8pE3qZdRodbSg9GeTKJtoLDMg/la9hGhRBVCX6SI82j6ffOciQt/
# nR+eDzMfUBMLJnOWbfhXqAJ9/UO0hNoR8XOxs+4rgISKIhjf69o9xBd/qxkrPkLc
# Z47qUT3w1lbU5ygt69OxtXXnHwZljZQp09nsad/ZkIdGAHvbREGJ3HxqV3rwN3mf
# XazL6IRktFLydkf3YYMZ3V+0VAshaG43IbtArF+y3kp9zvU5EmfvDqVjbOSmxR3N
# Ng1c1eYbqMFkdECnwHLFuk4fsbVYTXn+149zk6wsOeKlSNbwsDETqVcplicu9Yem
# j052FVUmcJgmf6AaRyBD40NjgHt1biclkJg6OBGz9vae5jtb7IHeIhTZgirHkr+g
# 3uM+onP65x9abJTyUpURK1h0QCirc0PO30qhHGs4xSnzyqqWc0Jon7ZGs506o9UD
# 4L/wojzKQtwYSH8UNM/STKvvmz3+DrhkKvp1KCRB7UK/BZxmSVJQ9FHzNklNiyDS
# LFc1eSuo80VgvCONWPfcYd6T/jnA+bIwpUzX6ZhKWD7TA4j+s4/TXkt2ElGTyYwM
# O1uKIqjBJgj5FBASA31fI7tk42PgpuE+9sJ0sj8eCXbsq11GdeJgo1gJASgADoRU
# 7s7pXcheMBK9Rp6103a50g5rmQzSM7TNsQIDAQABo4IBXTCCAVkwEgYDVR0TAQH/
# BAgwBgEB/wIBADAdBgNVHQ4EFgQUuhbZbU2FL3MpdpovdYxqII+eyG8wHwYDVR0j
# BBgwFoAU7NfjgtJxXWRM3y5nP+e6mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMGA1Ud
# JQQMMAoGCCsGAQUFBwMIMHcGCCsGAQUFBwEBBGswaTAkBggrBgEFBQcwAYYYaHR0
# cDovL29jc3AuZGlnaWNlcnQuY29tMEEGCCsGAQUFBzAChjVodHRwOi8vY2FjZXJ0
# cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNydDBDBgNVHR8E
# PDA6MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVz
# dGVkUm9vdEc0LmNybDAgBgNVHSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEw
# DQYJKoZIhvcNAQELBQADggIBAH1ZjsCTtm+YqUQiAX5m1tghQuGwGC4QTRPPMFPO
# vxj7x1Bd4ksp+3CKDaopafxpwc8dB+k+YMjYC+VcW9dth/qEICU0MWfNthKWb8RQ
# TGIdDAiCqBa9qVbPFXONASIlzpVpP0d3+3J0FNf/q0+KLHqrhc1DX+1gtqpPkWae
# LJ7giqzl/Yy8ZCaHbJK9nXzQcAp876i8dU+6WvepELJd6f8oVInw1YpxdmXazPBy
# oyP6wCeCRK6ZJxurJB4mwbfeKuv2nrF5mYGjVoarCkXJ38SNoOeY+/umnXKvxMfB
# wWpx2cYTgAnEtp/Nh4cku0+jSbl3ZpHxcpzpSwJSpzd+k1OsOx0ISQ+UzTl63f8l
# Y5knLD0/a6fxZsNBzU+2QJshIUDQtxMkzdwdeDrknq3lNHGS1yZr5Dhzq6YBT70/
# O3itTK37xJV77QpfMzmHQXh6OOmc4d0j/R0o08f56PGYX/sr2H7yRp11LB4nLCbb
# bxV7HhmLNriT1ObyF5lZynDwN7+YAN8gFk8n+2BnFqFmut1VwDophrCYoCvtlUG3
# OtUVmDG0YgkPCr2B2RP+v6TR81fZvAT6gt4y3wSJ8ADNXcL50CN/AAvkdgIm2fBl
# dkKmKYcJRyvmfxqkhQ/8mJb2VVQrH4D6wPIOK+XW+6kvRBVK5xMOHds3OBqhK/bt
# 1nz8MIIGwjCCBKqgAwIBAgIQBUSv85SdCDmmv9s/X+VhFjANBgkqhkiG9w0BAQsF
# ADBjMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4xOzA5BgNV
# BAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0YW1w
# aW5nIENBMB4XDTIzMDcxNDAwMDAwMFoXDTM0MTAxMzIzNTk1OVowSDELMAkGA1UE
# BhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMSAwHgYDVQQDExdEaWdpQ2Vy
# dCBUaW1lc3RhbXAgMjAyMzCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIB
# AKNTRYcdg45brD5UsyPgz5/X5dLnXaEOCdwvSKOXejsqnGfcYhVYwamTEafNqrJq
# 3RApih5iY2nTWJw1cb86l+uUUI8cIOrHmjsvlmbjaedp/lvD1isgHMGXlLSlUIHy
# z8sHpjBoyoNC2vx/CSSUpIIa2mq62DvKXd4ZGIX7ReoNYWyd/nFexAaaPPDFLnkP
# G2ZS48jWPl/aQ9OE9dDH9kgtXkV1lnX+3RChG4PBuOZSlbVH13gpOWvgeFmX40Qr
# StWVzu8IF+qCZE3/I+PKhu60pCFkcOvV5aDaY7Mu6QXuqvYk9R28mxyyt1/f8O52
# fTGZZUdVnUokL6wrl76f5P17cz4y7lI0+9S769SgLDSb495uZBkHNwGRDxy1Uc2q
# TGaDiGhiu7xBG3gZbeTZD+BYQfvYsSzhUa+0rRUGFOpiCBPTaR58ZE2dD9/O0V6M
# qqtQFcmzyrzXxDtoRKOlO0L9c33u3Qr/eTQQfqZcClhMAD6FaXXHg2TWdc2PEnZW
# pST618RrIbroHzSYLzrqawGw9/sqhux7UjipmAmhcbJsca8+uG+W1eEQE/5hRwqM
# /vC2x9XH3mwk8L9CgsqgcT2ckpMEtGlwJw1Pt7U20clfCKRwo+wK8REuZODLIivK
# 8SgTIUlRfgZm0zu++uuRONhRB8qUt+JQofM604qDy0B7AgMBAAGjggGLMIIBhzAO
# BgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEF
# BQcDCDAgBgNVHSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEwHwYDVR0jBBgw
# FoAUuhbZbU2FL3MpdpovdYxqII+eyG8wHQYDVR0OBBYEFKW27xPn783QZKHVVqll
# MaPe1eNJMFoGA1UdHwRTMFEwT6BNoEuGSWh0dHA6Ly9jcmwzLmRpZ2ljZXJ0LmNv
# bS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1lU3RhbXBpbmdDQS5j
# cmwwgZAGCCsGAQUFBwEBBIGDMIGAMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5k
# aWdpY2VydC5jb20wWAYIKwYBBQUHMAKGTGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1lU3RhbXBpbmdD
# QS5jcnQwDQYJKoZIhvcNAQELBQADggIBAIEa1t6gqbWYF7xwjU+KPGic2CX/yyzk
# zepdIpLsjCICqbjPgKjZ5+PF7SaCinEvGN1Ott5s1+FgnCvt7T1IjrhrunxdvcJh
# N2hJd6PrkKoS1yeF844ektrCQDifXcigLiV4JZ0qBXqEKZi2V3mP2yZWK7Dzp703
# DNiYdk9WuVLCtp04qYHnbUFcjGnRuSvExnvPnPp44pMadqJpddNQ5EQSviANnqlE
# 0PjlSXcIWiHFtM+YlRpUurm8wWkZus8W8oM3NG6wQSbd3lqXTzON1I13fXVFoaVY
# JmoDRd7ZULVQjK9WvUzF4UbFKNOt50MAcN7MmJ4ZiQPq1JE3701S88lgIcRWR+3a
# EUuMMsOI5ljitts++V+wQtaP4xeR0arAVeOGv6wnLEHQmjNKqDbUuXKWfpd5OEhf
# ysLcPTLfddY2Z1qJ+Panx+VPNTwAvb6cKmx5AdzaROY63jg7B145WPR8czFVoIAR
# yxQMfq68/qTreWWqaNYiyjvrmoI1VygWy2nyMpqy0tg6uLFGhmu6F/3Ed2wVbK6r
# r3M66ElGt9V/zLY4wNjsHPW2obhDLN9OTH0eaHDAdwrUAuBcYLso/zjlUlrWrBci
# I0707NMX+1Br/wd3H3GXREHJuEbTbDJ8WC9nR2XlG3O2mflrLAZG70Ee8PBf4NvZ
# rZCARK+AEEGKMIIG5jCCBM6gAwIBAgIQd70OA6G3CPhUqwZyENkERzANBgkqhkiG
# 9w0BAQsFADBTMQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFsU2lnbiBudi1z
# YTEpMCcGA1UEAxMgR2xvYmFsU2lnbiBDb2RlIFNpZ25pbmcgUm9vdCBSNDUwHhcN
# MjAwNzI4MDAwMDAwWhcNMzAwNzI4MDAwMDAwWjBZMQswCQYDVQQGEwJCRTEZMBcG
# A1UEChMQR2xvYmFsU2lnbiBudi1zYTEvMC0GA1UEAxMmR2xvYmFsU2lnbiBHQ0Mg
# UjQ1IENvZGVTaWduaW5nIENBIDIwMjAwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAw
# ggIKAoICAQDWQk3540/GI/RsHYGmMPdIPc/Q5Y3lICKWB0Q1XQbPDx1wYOYmVPpT
# I2ACqF8CAveOyW49qXgFvY71TxkkmXzPERabH3tr0qN7aGV3q9ixLD/TcgYyXFus
# UGcsJU1WBjb8wWJMfX2GFpWaXVS6UNCwf6JEGenWbmw+E8KfEdRfNFtRaDFjCvhb
# 0N66WV8xr4loOEA+COhTZ05jtiGO792NhUFVnhy8N9yVoMRxpx8bpUluCiBZfomj
# WBWXACVp397CalBlTlP7a6GfGB6KDl9UXr3gW8/yDATS3gihECb3svN6LsKOlsE/
# zqXa9FkojDdloTGWC46kdncVSYRmgiXnQwp3UrGZUUL/obLdnNLcGNnBhqlAHUGX
# Yoa8qP+ix2MXBv1mejaUASCJeB+Q9HupUk5qT1QGKoCvnsdQQvplCuMB9LFurA6o
# 44EZqDjIngMohqR0p0eVfnJaKnsVahzEaeawvkAZmcvSfVVOIpwQ4KFbw7MueovE
# 3vFLH4woeTBFf2wTtj0s/y1KiirsKA8tytScmIpKbVo2LC/fusviQUoIdxiIrTVh
# lBLzpHLr7jaep1EnkTz3ohrM/Ifll+FRh2npIsyDwLcPRWwH4UNP1IxKzs9jsbWk
# EHr5DQwosGs0/iFoJ2/s+PomhFt1Qs2JJnlZnWurY3FikCUNCCDx/wIDAQABo4IB
# rjCCAaowDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMBIGA1Ud
# EwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFNqzjcAkkKNrd9MMoFndIWdkdgt4MB8G
# A1UdIwQYMBaAFB8Av0aACvx4ObeltEPZVlC7zpY7MIGTBggrBgEFBQcBAQSBhjCB
# gzA5BggrBgEFBQcwAYYtaHR0cDovL29jc3AuZ2xvYmFsc2lnbi5jb20vY29kZXNp
# Z25pbmdyb290cjQ1MEYGCCsGAQUFBzAChjpodHRwOi8vc2VjdXJlLmdsb2JhbHNp
# Z24uY29tL2NhY2VydC9jb2Rlc2lnbmluZ3Jvb3RyNDUuY3J0MEEGA1UdHwQ6MDgw
# NqA0oDKGMGh0dHA6Ly9jcmwuZ2xvYmFsc2lnbi5jb20vY29kZXNpZ25pbmdyb290
# cjQ1LmNybDBWBgNVHSAETzBNMEEGCSsGAQQBoDIBMjA0MDIGCCsGAQUFBwIBFiZo
# dHRwczovL3d3dy5nbG9iYWxzaWduLmNvbS9yZXBvc2l0b3J5LzAIBgZngQwBBAEw
# DQYJKoZIhvcNAQELBQADggIBAAiIcibGr/qsXwbAqoyQ2tCywKKX/24TMhZU/T70
# MBGfj5j5m1Ld8qIW7tl4laaafGG4BLX468v0YREz9mUltxFCi9hpbsf/lbSBQ6l+
# rr+C1k3MEaODcWoQXhkFp+dsf1b0qFzDTgmtWWu4+X6lLrj83g7CoPuwBNQTG8cn
# qbmqLTE7z0ZMnetM7LwunPGHo384aV9BQGf2U33qQe+OPfup1BE4Rt886/bNIr0T
# zfDh5uUzoL485HjVG8wg8jBzsCIc9oTWm1wAAuEoUkv/EktA6u6wGgYGnoTm5/Db
# hEb7c9krQrbJVzTHFsCm6yG5qg73/tvK67wXy7hn6+M+T9uplIZkVckJCsDZBHFK
# EUtaZMO8eHitTEcmZQeZ1c02YKEzU7P2eyrViUA8caWr+JlZ/eObkkvdBb0LDHgG
# K89T2L0SmlsnhoU/kb7geIBzVN+nHWcrarauTYmAJAhScFDzAf9Eri+a4OFJCOHh
# W9c40Z4Kip2UJ5vKo7nb4jZq42+5WGLgNng2AfrBp4l6JlOjXLvSsuuKy2MIL/4e
# 81Yp4jWb2P/ppb1tS1ksiSwvUru1KZDaQ0e8ct282b+Awdywq7RLHVg2N2Trm+GF
# F5opov3mCNKS/6D4fOHpp9Ewjl8mUCvHouKXd4rv2E0+JuuZQGDzPGcMtghyKTVT
# gTTcMIIHHDCCBQSgAwIBAgIMSUXsaWVAFuihgz78MA0GCSqGSIb3DQEBCwUAMFkx
# CzAJBgNVBAYTAkJFMRkwFwYDVQQKExBHbG9iYWxTaWduIG52LXNhMS8wLQYDVQQD
# EyZHbG9iYWxTaWduIEdDQyBSNDUgQ29kZVNpZ25pbmcgQ0EgMjAyMDAeFw0yMjAz
# MDIxMTQ4MzlaFw0yNDA2MTQxNDMwNDRaMIGJMQswCQYDVQQGEwJVUzERMA8GA1UE
# CBMITmV3IFlvcmsxETAPBgNVBAcTCE5ldyBZb3JrMRYwFAYDVQQKEw1TRU1QRVJJ
# UyBJTkMuMRYwFAYDVQQDEw1TRU1QRVJJUyBJTkMuMSQwIgYJKoZIhvcNAQkBFhVj
# b2Rlc2lnbkBzZW1wZXJpcy5jb20wggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIK
# AoICAQCvq2XdKWZ+xqIQ58x22njO9MSseih/t2TsnekKeiimUgSIKHG7TLypz2Vx
# 0+PumvxEm9JsbXXQKx9XobNZ69lRUlaZCq8Tia6DMvGEUAEvi3SvwdlJJr+AowSd
# 1evdJSHB1suaDfwX7K7PIaiIdjnl5wwc9ajPz20FiV7OUhIjYLT4T9FALz9zxJN+
# WlcUJUSmoXAmdjXHmvP6MQNO7whPI2auFvrq7OiA1N2CyXTFqwz1OkTlUqtZIsCD
# fuIWuPQZGPB3+nNN87e1eEfjO3vbwXTge649/7cXcRMuFKyuTWUE3AxX5QFWM8mt
# WujVuu8QefsPzwGJwaRP+oWHT5HmNS7s7wEn7+DggenjIo62n85ySnTliLofjl9f
# flxB10Hq3BYgVsZyNEYBp1hRKRpxQ58ZHMAkpA4liY3chI4klPWGgcIW1H2yqM3S
# lBraTVuT9skaZOua3u5Q8nlRyeXWoeBmQ8PlH0RFdeNnTRxfmY7HmXSCnG+D7TVH
# hzJPLGBHMC4FTjqro/wRKHiGLIkz6edqjz54TxKv9U39ULsIK0sQZaXb8STUp6eK
# K5lKt0wWedojQbi2qob3uZIgGg+4rAEKT4Wm8ZdDy7eN3aZTw/i5mjiEfwyJvgvz
# WT/yhJz6+ZDMQGIFJj9L2DaPnUSejLMC7K8kyZwJUXVGn1NRTQIDAQABo4IBsTCC
# Aa0wDgYDVR0PAQH/BAQDAgeAMIGbBggrBgEFBQcBAQSBjjCBizBKBggrBgEFBQcw
# AoY+aHR0cDovL3NlY3VyZS5nbG9iYWxzaWduLmNvbS9jYWNlcnQvZ3NnY2NyNDVj
# b2Rlc2lnbmNhMjAyMC5jcnQwPQYIKwYBBQUHMAGGMWh0dHA6Ly9vY3NwLmdsb2Jh
# bHNpZ24uY29tL2dzZ2NjcjQ1Y29kZXNpZ25jYTIwMjAwVgYDVR0gBE8wTTBBBgkr
# BgEEAaAyATIwNDAyBggrBgEFBQcCARYmaHR0cHM6Ly93d3cuZ2xvYmFsc2lnbi5j
# b20vcmVwb3NpdG9yeS8wCAYGZ4EMAQQBMAkGA1UdEwQCMAAwRQYDVR0fBD4wPDA6
# oDigNoY0aHR0cDovL2NybC5nbG9iYWxzaWduLmNvbS9nc2djY3I0NWNvZGVzaWdu
# Y2EyMDIwLmNybDATBgNVHSUEDDAKBggrBgEFBQcDAzAfBgNVHSMEGDAWgBTas43A
# JJCja3fTDKBZ3SFnZHYLeDAdBgNVHQ4EFgQUeXYlhOREXUlIPR2ZpkQmsxInWlkw
# DQYJKoZIhvcNAQELBQADggIBALVBZWrw/axZwZgcg+pO0wYi/phXxE1hUOxLaeRJ
# SxTEYZk7QasYoM2fR6mjUzV1da1sQKSK9OtUDbHcq7/JXKwcdKWIttwv9AI+RtdP
# yGvezvVfne1sBCn3qIxK3/5FRFdYftucE4It87UF4f25rS5FgGnLc5h+4MQgKjCv
# hI1F/aJjTBKhZnzlCwGUyAssXKYmWf0+PDxJixU5iTB8yK/VpPu4LFWWRi0pb1uS
# bJse4qIR25DBJ+zkbN6BIUz5Zx8LgmoGFVdEHFCWNfndjWtvd4DwJUtMdk9Wpdcz
# x2LrbzLUjKVN1PNdyuIWMKYxhnSt74U2VUFPoqEn6c/6lxKattXL2qD8FfV9APlc
# ZPrdV1Cnip9Mrbm5YdYebQaU4xH9LILkcSakBJSQvm0ev6f70ahHJLwiNCinftEn
# WEqgFeUfRrLJBYe8tCMIzkfH4BFIOrcd16ordOgC5QMn4bmqCvQE/df0Cngrt9Kf
# ynSBp+kx1IFZGeZ8iE9rQThcQwD75dC1/L6ADiev/TMPTm4H6+lKWL3umGvp/d25
# p+Y0lsfjTPvGWDZ65MF3sKRS0jcrAWrSm92CWfReyOXk2FHjJjtj2hAiNZ32igVh
# arGemFH0l5wHc/4KRUDcvtfCq7k8hdJn5s+EXlrzI01OFiabyDbR2Xe8JpmJfn9E
# e+3yMYIGNjCCBjICAQEwaTBZMQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFs
# U2lnbiBudi1zYTEvMC0GA1UEAxMmR2xvYmFsU2lnbiBHQ0MgUjQ1IENvZGVTaWdu
# aW5nIENBIDIwMjACDElF7GllQBbooYM+/DANBglghkgBZQMEAgEFAKB8MBAGCisG
# AQQBgjcCAQwxAjAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQB
# gjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEiBCBqL/0w7ze61jGI
# j8hNKN2SVesQIl4G8Qde1PW72ZwL7jANBgkqhkiG9w0BAQEFAASCAgCrK7ueWZae
# EJ33Vc8rr2H1rdIRON16KXKJZUvrPCh6I24fB7yYODp80PuQpYl9ry3LMBKDAxmu
# uObuHYgcZOZXa/yJlCUSxpfeUpf8u93USuJZ87Zx/66o/gHyIhH5JCvQo2TTJxKy
# GhGTeVCM5VKndchAmGP8XPPzBRT4rDHSvNWwbS4J9315OAp6dJwp0soTq2vDYzze
# bjxMiW1cG1qxFzhxTxKxarDDO9h1poqVPwYRrn19BPML3979UIg2t9fvZei4JJfA
# EXqMefYjKp2Fq3+AY3g/TcZnbzFUuW3BXrF+XI1Ni2XG5DpMQ3pQgb0+an4Z1Z1L
# R7bqcwDXa47RCp9CAI3ij4oxGTus0b9NXIAg/ovSxwG9tJG0OlSRZEvXXIXfS8TW
# XfDmqzEG3Ddkatycjjbax4eeIuylz1kUSprMn6iq2roPgSYPuYXNEqOZczM/yXOV
# fm/miZb3QVmRPrxka3bOEmooQEyBlYxfZXfe/2FBRDAisZw/xMi1XXa3KeI5Ne7u
# WiKWagAdTqyKAFOuTkitmT7EPM6zYSdZLIwr4MHpD8MurkcXIiv024xexC9KzFST
# Nju4t3XkMge4uXm4vPzMy2LmogDwEkDhOcl9FSqo5bwnVyL1TYDGct+GWK1tcSaQ
# u/NJlnUH/ppH7ls/kkhMdZJNxhF5V0j39qGCAyAwggMcBgkqhkiG9w0BCQYxggMN
# MIIDCQIBATB3MGMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5j
# LjE7MDkGA1UEAxMyRGlnaUNlcnQgVHJ1c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBU
# aW1lU3RhbXBpbmcgQ0ECEAVEr/OUnQg5pr/bP1/lYRYwDQYJYIZIAWUDBAIBBQCg
# aTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0yMzEy
# MDUxMTQxMDhaMC8GCSqGSIb3DQEJBDEiBCAr9fdmk72yKLSUK4GgIlk3ZXYNPDYd
# nV+AhdgEn+o4qzANBgkqhkiG9w0BAQEFAASCAgAZOOf65O9MlD+ntdl/ZR8SzN+x
# wQvPmU8ue2MBqmuV7jqdVfITerdolvyATZGjCeeWo/2Kg4uVjMfaKD8CCwHcGEaD
# /CXUvae89//9wg1EByyqCLRMIE3MHC1+CjShaCBPAXepvqQVnW9MLxeVZbENbJzB
# /RkEa8iIq3V4kI45quLNX5nGvl7N/zQpSIr+NPPbH8DP2WzKpgYlvyvz0GwMoWVY
# ce3j6c29dM/IVW9J+KXojPLf2wjp4Jyp7G6fsXcuJVfcVOLu/jKNwlDV1o586Zcc
# tfX0ikT6Im+DVuCg3mBRhfJTcNcKij0uorzL21QciGajrZsPMesAhhFKbFC8B3Sx
# DARGCEBuytVRLARjOfFC/E+0894S6yw/gkMCw5nVPtGgIdLUPUCi7j8d8biYdm6l
# FY85JZmCrUkldqfBt9fNifb27m6EllPzAf/oz9bBBR5QfjGfEZoAWg6lUTo0w1Cd
# 26FjzCwTaSgFr4+gO3A5Gh8NBO0NWeyg4BnV3e1XZNWtFTyu5cngWmXO2c54Bw2m
# I1vnSdyowL1fI8UotVqi18Mgsn9v7uaj8cLXH/05YKw13WMRyD5JCrHm06zcttZO
# oSNbExCxDKEG9Ow3P+Ta3nixMKwYmlSwcrQ4O/YSC6QJi722Uzthf7rJYMOzvTu1
# mdrOufj4GrGOjQu3Rw==
# SIG # End signature block
